Package: stdmod
Title: Standardized Moderation Effect and Its Confidence Interval
Version: 0.2.13
Authors@R: 
    c(person(given = "Shu Fai",
           family = "Cheung",
           role = c("aut", "cre"),
           email = "shufai.cheung@gmail.com",
           comment = c(ORCID = "0000-0002-9871-9448")),
      person(given = "David Weng Ngai",
           family = "Vong",
           role = c("ctb")))
Description: Functions for computing a standardized moderation effect
             in moderated regression and forming its confidence interval
             by nonparametric bootstrapping as proposed in
             Cheung, Cheung, Lau, Hui, and Vong (2022)
             <doi:10.1037/hea0001188>. Also includes simple-to-use
             functions for computing conditional effects (unstandardized
             or standardized) and plotting moderation effects.
URL: https://sfcheung.github.io/stdmod/
BugReports: https://github.com/sfcheung/stdmod/issues
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Suggests: testthat, knitr, rmarkdown, visreg, lm.beta
Config/testthat/edition: 3
Config/testthat/parallel: true
Depends: R (>= 4.0.0)
Imports: boot, ggplot2, stats, utils, lavaan, manymome, rlang
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-04 14:12:48 UTC; shufa
Author: Shu Fai Cheung [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9871-9448>),
  David Weng Ngai Vong [ctb]
Maintainer: Shu Fai Cheung <shufai.cheung@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-04 15:00:02 UTC
Built: R 4.6.0; ; 2026-05-05 00:07:22 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(add1,std_selected)
S3method(coef,cond_effect)
S3method(coef,stdmod_lavaan)
S3method(confint,cond_effect)
S3method(confint,std_selected)
S3method(confint,stdmod_lavaan)
S3method(print,cond_effect)
S3method(print,std_selected)
S3method(print,stdmod_lavaan)
S3method(print,summary.std_selected)
S3method(summary,std_selected)
S3method(update,std_selected)
S3method(vcov,std_selected)
export(cond_effect)
export(cond_effect_boot)
export(plotmod)
export(std_selected)
export(std_selected_boot)
export(stdmod)
export(stdmod_boot)
export(stdmod_lavaan)
importFrom(rlang,.data)
importFrom(stats,anova)

<!-- badges: start -->
[![Lifecycle: stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![Project Status: Active - The project has reached a stable, usable state and is being actively developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)
[![CRAN status](https://www.r-pkg.org/badges/version/stdmod?color=blue)](https://CRAN.R-project.org/package=stdmod)
[![CRAN: Release Date](https://www.r-pkg.org/badges/last-release/stdmod?color=blue)](https://cran.r-project.org/package=stdmod)
[![CRAN RStudio mirror downloads](https://cranlogs.r-pkg.org/badges/grand-total/stdmod?color=blue)](https://r-pkg.org/pkg/stdmod)
[![Code size](https://img.shields.io/github/languages/code-size/sfcheung/stdmod.svg)](https://github.com/sfcheung/stdmod)
[![Last Commit at Master](https://img.shields.io/github/last-commit/sfcheung/stdmod.svg)](https://github.com/sfcheung/stdmod/commits/master)
[![R-CMD-check](https://github.com/sfcheung/stdmod/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/sfcheung/stdmod/actions/workflows/R-CMD-check.yaml)
[![DOI](https://img.shields.io/badge/doi-10.1037/hea0001188-blue.svg)](https://doi.org/10.1037/hea0001188)
<!-- badges: end -->


# stdmod: Standardized Moderation <img src="man/figures/logo.png" align="right" height="150" />

(Version 0.2.13, updated on 2026-05-04, [release history](https://sfcheung.github.io/stdmod/news/index.html))

(Important changes since 0.2.0.0: Bootstrap confidence intervals and
variance-covariance matrix of estimates are the defaults of `confint()`
and `vcov()` for the output of `std_selected_boot()`.)

This package includes functions for computing a standardized
moderation effect and forming its confidence interval by
nonparametric bootstrapping correctly. It was described briefly
in the following publication ([OSF project page](https://osf.io/ac8de/)).
It supports moderated regression conducted by `stats::lm()` and path
analysis with product term conducted by `lavaan::lavaan()`.

- Cheung, S. F., Cheung, S.-H., Lau, E. Y. Y., Hui, C. H., & Vong, W. N. (2022) Improving an old way to measure moderation effect in standardized units. *Health Psychology, 41*(7), 502-505. https://doi.org/10.1037/hea0001188.

More information on this package:

https://sfcheung.github.io/stdmod/

## Quick Links:

- [stdmod](https://sfcheung.github.io/stdmod/articles/stdmod.html): A quick start on how to use
  `std_selected()` and `std_selected_boot()`, the
  two main functions, to standardize selected variables
  in a regression model and refit the model.

- [moderation](https://sfcheung.github.io/stdmod/articles/moderation.html): How
  to use `std_selected()` and `std_selected_boot()` to compute standardized
  moderation effect and form its nonparametric bootstrap confidence interval.

- [std_selected](https://sfcheung.github.io/stdmod/articles/std_selected.html): How to use
  `std_selected()` to mean center or standardize selected
  variables in any regression models, and use
  `std_selected_boot()` to form nonparametric
  bootstrap confidence intervals
  for standardized regression coefficients (*betas* in
  psychology literature).

- [plotmod](https://sfcheung.github.io/stdmod/articles/plotmod.html): How to generate a typical plot of
  moderation effect using `plotmod()`.

- [cond_effect](https://sfcheung.github.io/stdmod/articles/cond_effect.html): How to compute conditional
  effects of the predictor for selected
  levels of the moderator, and form nonparametric bootstrap
  confidence intervals these effects.

# Installation

The stable CRAN version can be installed by `install.packages()`:

```
install.packages("stdmod")
```

The latest version of this package at GitHub can be
installed by `remotes::install_github()`:

```
remotes::install_github("sfcheung/stdmod")
```

# Implementation

The main function, `std_selected()`, accepts an `lm()`
output, standardizes variables by users, and update the
results. If interaction terms are present, they will be
formed after the standardization. If bootstrap
confidence intervals are requested using
`std_selected_boot()`, both standardization
and regression will be repeated in each bootstrap sample,
ensuring that the sampling variability of the standardizers
(e.g., the standard deviations of the selected variables),
are also taken into account.

# Issues

If you have any suggestions and found any bugs, please feel
feel to open a GitHub issue. Thanks.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width  = 6,
  fig.height = 4,
  fig.align = "center"
)

## ----plotmod_example, echo = FALSE--------------------------------------------
library(stdmod)
data(sleep_emo_con)
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration")

## ----plotmod_example2, echo = FALSE-------------------------------------------
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration",
        graph_type = "tumble")

## ----plotmod_example_raw------------------------------------------------------
library(stdmod)
data(sleep_emo_con)
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness")

## ----plotmod_example_raw_numeric----------------------------------------------
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        w_method = "percentile")

## ----plotmod_example_raw_cat--------------------------------------------------
set.seed(61452)
sleep_emo_con$city <- sample(c("Alpha", "Beta", "Gamma"),
                             nrow(sleep_emo_con),
                             replace = TRUE)
lm_cat <- lm(sleep_duration ~ age + gender +
                              emotional_stability * city,
             sleep_emo_con)
plotmod(lm_cat,
        x = "emotional_stability",
        w = "city")

## ----plotmod_tumble-----------------------------------------------------------
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        graph_type = "tumble")

## ----plotmod_annotate---------------------------------------------------------
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        x_label = "EMO",
        w_label = "CON",
        y_label = "SLEEP")

## ----plotmod_title------------------------------------------------------------
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        title = "EMO Effects For Low/High CON")

## ----plotmod_line_width-------------------------------------------------------
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        point_size = 8,
        line_width = 2)

## ----plotmod_cond-------------------------------------------------------------
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
p <- plotmod(lm_raw,
              x = "emotional_stability",
              w = "conscientiousness")
library(ggplot2)
p + theme(plot.subtitle = element_blank())

## ----plotmod_levels-----------------------------------------------------------
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
p <- plotmod(lm_raw,
             x = "emotional_stability",
             w = "conscientiousness")
p + theme(plot.caption = element_blank())

## ----plotmod_any_std----------------------------------------------------------
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
lm_std <- std_selected(lm_raw,
                       to_standardize = ~ emotional_stability + conscientiousness)
# Before Version 0.2.6.3 of stdmod, to_center and to_scale need to be used:
# lm_std <- std_selected(lm_raw,
#                        to_center = ~ emotional_stability + conscientiousness,
#                        to_scale = ~ emotional_stability + conscientiousness)

## ----plotmod_std_selected-----------------------------------------------------
plotmod(lm_std,
        x = "emotional_stability",
        w = "conscientiousness")

## ----plotmod_tweak------------------------------------------------------------
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
p <- plotmod(lm_std,
              x = "emotional_stability",
              w = "conscientiousness")
p + scale_color_manual(values = c("blue", "red")) +
    theme_classic()

#' @details
#' [std_selected_boot()] is a wrapper of [std_selected()]. It calls
#' [std_selected()]
#' once
#' for each bootstrap sample, and then computes the nonparametric
#' bootstrap
#' percentile confidence intervals (Cheung, Cheung, Lau, Hui, & Vong, 2022).
#'
#' If `do_boot` is `FALSE`, then the object it returns is identical to that
#' by [std_selected()].
#'
#' This function intentionally does not have an argument for setting the seed
#' for
#' random number. Users are recommended to set the seed, e.g., using
#' [set.seed()]
#' before calling it, to ensure reproducibility.
#'
#' @return
#' Like [std_selected()], [std_selected_boot()] returns the updated [lm()]
#' output, with the class `std_selected` added. The output of [std_selected_boot()]
#' contain these additional elements in the list:
#'
#' - `boot_ci`: A data frame of the bootstrap confidence intervals of the
#'              regression coefficient.
#'
#' - `nboot`: The number of bootstrap samples requested.
#'
#' - `conf`: The level of confidence, in proportion.
#'
#' - `boot_est`: A matrix of the bootstrap estimates of the regression coefficients.
#'               The number of rows equal to `nboot`, and the number of columns
#'               equal to the number of terms in the regression model.
#'
#' - `std_selected_boot_call`: The call to `std_selected_boot()`.
#'
#' - `boot_out`: If available, the original output from `boot::boot()`.
#'
#' @param lm_out The output from [lm()].
#' @param conf The level of confidence for the confidence interval.
#'              Default is .95.
#' @param nboot The number of bootstrap samples. Default is 100.
#' @param boot_args A named list of arguments to be passed to [boot::boot()].
#'                 Default is `NULL`.
#' @param save_boot_est If `TRUE`, the default, the bootstrap estimates will
#'                      be saved in the element
#'                      `boot_est` of the output.
#' @param full_output Whether the full output from [boot::boot()] is returned.
#'                    Default is `FALSE`. If `TRUE`, the full output from
#'                    [boot::boot()] will be saved in the element `boot_out`
#'                    of the output.
#' @param do_boot Whether bootstrapping confidence intervals will be formed.
#'                Default is `TRUE`. If `FALSE`, all arguments related to
#'                bootstrapping will be ignored.
#'
#' @examples
#'
#' dat <- test_x_1_w_1_v_1_cat1_n_500
#' head(dat)
#'
#' # Do a moderated regression by lm
#' lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
#' summary(lm_raw)
#' # Standardize all variables as in std_selected above, and compute the
#' # nonparametric bootstrapping percentile confidence intervals.
#' set.seed(87053)
#' lm_std_boot <- std_selected_boot(lm_raw,
#'                                  to_scale = ~ .,
#'                                  to_center = ~ .,
#'                                  conf = .95,
#'                                  nboot = 100)
#' # In real analysis, nboot should be at least 2000.
#' summary(lm_std_boot)
#'
#' # Use to_standardize as a shortcut
#' set.seed(87053)
#' lm_std_boot2 <- std_selected_boot(lm_raw,
#'                                   to_standardize = ~ .,
#'                                   conf = .95,
#'                                   nboot = 100)
#' # The results are the same
#' confint(lm_std_boot)
#' confint(lm_std_boot2)
#' all.equal(confint(lm_std_boot), confint(lm_std_boot2))
#'
#'
#' @export
#' @describeIn std_selected A wrapper of [std_selected()] that forms
#'                           nonparametric bootstrap confidence intervals.
#' @order 2

std_selected_boot <- function(lm_out,
                              to_scale = NULL,
                              to_center = NULL,
                              to_standardize = NULL,
                              conf = .95,
                              nboot = 100,
                              boot_args = NULL,
                              save_boot_est = TRUE,
                              full_output = FALSE,
                              do_boot = TRUE) {
    if (missing(lm_out)) {
        stop("The arguments lm_out cannot be empty.")
      }

    # Get the data frame.
    # Form the bootstrapping function.
    # Do the bootstrapping.
    # Collect the results.
    # Return the results.

    # Do std_selected

    std_selected_out <- std_selected(lm_out = lm_out,
                                     to_scale = to_scale,
                                     to_center = to_center,
                                     to_standardize = to_standardize)

    if (do_boot) {
        # Get the data frame

        dat <- lm_out$model
        k <- ncol(dat)
        n <- nrow(dat)

        # Create the boot function

        bootfct <- create_boot_selected(lm_out,
                                        to_scale,
                                        to_center,
                                        to_standardize)

        # Do bootstrapping

        boot_out <- do.call(boot::boot,
                      c(list(data = dat, statistic = bootfct, R = nboot),
                      boot_args))

        # Collect output

        p <- length(boot_out$t0)

        cis <- t(sapply(seq_len(p), function(x) {
                    boot::boot.ci(boot_out, conf = conf,
                                  type = "perc", index = x)$percent[4:5]
                  }))
        rownames(cis) <- names(boot_out$t0)
        colnames(cis) <- c("CI Lower", "CI Upper")


        # Append bootstrapping output

        std_selected_out$boot_ci <- cis
        std_selected_out$nboot <- nboot
        std_selected_out$conf <- conf
        tmp <- boot_out$t
        colnames(tmp) <- names(boot_out$t0)
        std_selected_out$boot_est <- tmp
        std_selected_out$std_selected_boot_call <- match.call()
        if (full_output) {
            std_selected_out$boot_out <- boot_out
          }
      }

    std_selected_out
  }

create_boot_selected <- function(lm_out,
                                 to_scale,
                                 to_center,
                                 to_standardize) {
  function(d, ind) {
        force(lm_out)
        lm_out_i <- lm_out
        lm_out_i$model <- d[ind, ]
        out <- std_selected(lm_out = lm_out_i,
                            to_scale = to_scale,
                            to_center = to_center,
                            to_standardize = to_standardize)
        stats::coef(out)
      }
  }

library(testthat)
library(stdmod)

# context("Check update method")

dat <- test_x_1_w_1_v_1_cat1_n_500

lm_0 <- lm(dv ~ iv*mod , dat)

set.seed(649831074)
stdmod_0 <- std_selected_boot(lm_0, to_scale = ~ mod + dv, to_center = ~ mod + dv,
                               nboot = 100,
                               full_output = TRUE)
summary(stdmod_0)
termlabels_0std <- eval(parse(text = deparse(attr(terms(stdmod_0), "term.labels"))))

lm_1 <- update(lm_0, . ~  . + v1)
summary(lm_1)
termlabels_1 <- eval(parse(text = deparse(attr(terms(lm_1), "term.labels"))))
termvariables_1 <- deparse(attr(terms(lm_1), "variables"))
resp_1 <- attr(terms(lm_1), "response")

lm_2 <- update(lm_0, . ~  . - mod)
summary(lm_2)
termlabels_2 <- eval(parse(text = deparse(attr(terms(lm_2), "term.labels"))))
termvariables_2 <- deparse(attr(terms(lm_2), "variables"))
resp_2 <- attr(terms(lm_2), "response")

lm_3 <- update(lm_0, v1 ~ .)
summary(lm_3)
termlabels_3 <- eval(parse(text = deparse(attr(terms(lm_3), "term.labels"))))
termvariables_3 <- deparse(attr(terms(lm_3), "variables"))
resp_3 <- attr(terms(lm_3), "response")

set.seed(649831074)
stdmod_1 <- update(stdmod_0, . ~ . + v1)
termlabels_1std <- eval(parse(text = deparse(attr(terms(stdmod_1), "term.labels"))))
termvariables_1std <- deparse(attr(terms(stdmod_1), "variables"))
resp_1std <- attr(terms(stdmod_1), "response")

set.seed(649831074)
stdmod_2 <- update(stdmod_0, . ~ . - mod)
termlabels_2std <- eval(parse(text = deparse(attr(terms(stdmod_2), "term.labels"))))
termvariables_2std <- deparse(attr(terms(stdmod_2), "variables"))
resp_2std <- attr(terms(stdmod_2), "response")

set.seed(649831074)
stdmod_2b <- update(stdmod_2, . ~ . + mod)
termlabels_2bstd <- eval(parse(text = deparse(attr(terms(stdmod_2b), "term.labels"))))
termvariables_2bstd <- deparse(attr(terms(stdmod_2b), "variables"))
resp_2stdb <- attr(terms(stdmod_2b), "response")

set.seed(649831074)
stdmod_1_check <- std_selected_boot(lm_1, to_scale = ~ mod + dv, to_center = ~ mod + dv,
                               nboot = 100,
                               full_output = TRUE)
coef(stdmod_1)
coef(stdmod_1_check)
confint(stdmod_1, type = "boot")
confint(stdmod_1_check, type = "boot")

set.seed(649831074)
stdmod_2_check <- std_selected_boot(lm_2, to_scale = ~ mod + dv, to_center = ~ mod + dv,
                               nboot = 100,
                               full_output = TRUE)
coef(stdmod_2)
coef(stdmod_2_check)
confint(stdmod_2, type = "boot")
confint(stdmod_2_check, type = "boot")

stdmod_2b_check <- stdmod_0
coef(stdmod_2b)
coef(stdmod_2b_check)
confint(stdmod_2b, type = "boot")
confint(stdmod_2b_check, type = "boot")


# stdmod_3 <- update(stdmod_0, v1 ~ .)
# termlabels_3std <- eval(parse(text = deparse(attr(terms(stdmod_3), "term.labels"))))
# termvariables_3std <- deparse(attr(terms(stdmod_3), "variables"))
# resp_3std <- attr(terms(stdmod_3), "response")

test_that("Check terms labels 1", {
    expect_equal(
        termlabels_1std, termlabels_1,
        ignore_attr = TRUE
      )
  })

test_that("Check terms labels 2", {
    expect_equal(
        termlabels_2std, termlabels_2,
        ignore_attr = TRUE
      )
  })

# test_that("Check terms labels 3", {
#     expect_equal(
#         termlabels_3std, termlabels_3,
#         ignore_attr = TRUE
#       )
#   })

test_that("Check terms variables 1", {
    expect_equal(
        termvariables_1std, termvariables_1,
        ignore_attr = TRUE
      )
  })

test_that("Check terms variables 2", {
    expect_equal(
        termvariables_2std, termvariables_2,
        ignore_attr = TRUE
      )
  })

# test_that("Check terms variables 3", {
#     expect_equal(
#         termvariables_3std, termvariables_3,
#         ignore_attr = TRUE
#       )
#   })

test_that("Check coefs 1", {
    expect_equal(
        coef(stdmod_1), coef(stdmod_1_check),
        ignore_attr = TRUE
      )
  })

test_that("Check coefs 2", {
    expect_equal(
        coef(stdmod_2), coef(stdmod_2_check),
        ignore_attr = TRUE
      )
  })

test_that("Check coefs 2b", {
    expect_equal(
        coef(stdmod_2b), coef(stdmod_2b_check),
        ignore_attr = TRUE
      )
  })

test_that("Check boot ci 1", {
    expect_equal(
        confint(stdmod_1, type = "boot"), confint(stdmod_1_check, type = "boot"),
        ignore_attr = TRUE
      )
  })

test_that("Check boot ci 2", {
    expect_equal(
        confint(stdmod_2, type = "boot"), confint(stdmod_2_check, type = "boot"),
        ignore_attr = TRUE
      )
  })

test_that("Check boot ci 2b", {
    expect_equal(
        confint(stdmod_2b, type = "boot"), confint(stdmod_2b_check, type = "boot"),
        ignore_attr = TRUE
      )
  })

skip_on_cran()

library(testthat)
library(stdmod)
library(boot)

set.seed(8970808)

nboot <- 100

# context("Check standardizing selected variables with std_selected with bootstrapping")

dat <- test_x_1_w_1_v_1_cat1_n_500

transform0 <- function(data, vars) {
    for (x in vars) {
        data[x] <- scale(data[[x]])[, 1]
      }
    data
  }

lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
lm_zx  <- lm(dv ~ iv*mod + v1 + cat1, transform0(dat, "iv"))
lm_zw  <- lm(dv ~ iv*mod + v1 + cat1, transform0(dat, "mod"))
lm_zy  <- lm(dv ~ iv*mod + v1 + cat1, transform0(dat, "dv"))
lm_zxzw  <- lm(dv ~ iv*mod + v1 + cat1, transform0(dat, c("iv", "mod")))
lm_zxzy  <- lm(dv ~ iv*mod + v1 + cat1, transform0(dat, c("iv", "dv")))
lm_zyzw  <- lm(dv ~ iv*mod + v1 + cat1, transform0(dat, c("dv", "mod")))
lm_zall  <- lm(dv ~ iv*mod + v1 + cat1, transform0(dat, c("iv", "dv", "mod")))

dat_tmp <- dat
dat_tmp$iv <- scale(dat$iv, scale = FALSE, center = TRUE)[, 1]
dat_tmp$mod <- scale(dat$mod, scale = sd(dat$mod), center = FALSE)[, 1]
lm_cxsw  <- lm(dv ~ iv*mod + v1 + cat1, dat_tmp)


set.seed(868945)
stdmod_wy <- std_selected_boot(lm_raw, to_scale = ~ mod + dv, to_center = ~ mod + dv, nboot = nboot)
tmpfct <- function(d, i) {
    d_i <- d[i, ]
    lm_raw_i <- update(lm_raw, data = d_i)
    out0 <- std_selected(lm_raw_i, to_scale = ~ mod + dv, to_center = ~ mod + dv)
    coef(out0)
  }
set.seed(868945)
stdmod_wy_check <- boot(dat, tmpfct, R = nboot)
stdmod_wy_check$t
stdmod_wy$boot_est
test_that("Standardize w and y: boot est", {
    expect_equal(
        stdmod_wy_check$t, stdmod_wy$boot_est,
        ignore_attr = TRUE
      )
  })

set.seed(868945)
stdmod_wy_std <- std_selected_boot(lm_raw, to_standardize = ~ mod + dv, nboot = nboot)
test_that("Standardize w and y: boot est, to_standardize", {
    expect_equal(
        confint(stdmod_wy_std),
        confint(stdmod_wy),
        ignore_attr = TRUE
      )
  })

set.seed(80985715)
stdmod_xwy <- std_selected_boot(lm_raw, to_scale = ~ mod + iv + dv, to_center = ~ iv + mod + dv, nboot = nboot)
tmpfct <- function(d, i) {
    d_i <- d[i, ]
    lm_raw_i <- update(lm_raw, data = d_i)
    out0 <- std_selected(lm_raw_i, to_scale = ~ mod + iv + dv, to_center = ~ iv + mod + dv)
    coef(out0)
  }
set.seed(80985715)
stdmod_xwy_check <- boot(dat, tmpfct, R = nboot)
test_that("Standardize x, w and y: boot est", {
    expect_equal(
        stdmod_xwy_check$t, stdmod_xwy$boot_est,
        ignore_attr = TRUE
      )
  })

set.seed(80985715)
stdmod_xwy_std <- std_selected_boot(lm_raw, to_scale = ~ mod, to_center = ~ mod,
                                to_standardize = ~ iv + dv,
                                nboot = nboot)
test_that("Standardize x, w and y: boot est, to_standardize", {
    expect_equal(
        confint(stdmod_xwy_std),
        confint(stdmod_xwy),
        ignore_attr = TRUE
      )
  })

# stdmod2_wy <- stdmod_boot(lm_raw, x = iv, y = dv, w =mod,
#                            x_rescale = FALSE, y_rescale = TRUE, w_rescale = TRUE, nboot = nboot)
# stdmod2_xwy <- stdmod_boot(lm_raw, x = iv, y = dv, w =mod,
#                            x_rescale = TRUE, y_rescale = TRUE, w_rescale = TRUE, nboot = nboot)

stdmod3_wy <- std_selected(lm_raw, to_scale = ~ mod + dv, to_center = ~ dv + mod)
stdmod3_xwy <- std_selected(lm_raw, to_scale = ~ dv + iv + mod,  to_center = ~ mod + iv + dv)


test_that("Standardize w and y: Compare coefficients (selected_boot vs. noboot)", {
    expect_equal(
        coef(stdmod_wy), coef(stdmod3_wy),
        ignore_attr = TRUE
      )
  })

# test_that("Standardize w and y: Compare bootstrapping ci", {
#     expect_equal(
#         stdmod_wy$boot_ci["iv:mod",], stdmod2_wy$ci, tolerance = .01,
#         ignore_attr = TRUE
#       )
#   })



test_that("Standardize x, w and y: Compare coefficients (selected_boot vs. noboot)", {
    expect_equal(
        coef(stdmod_xwy), coef(stdmod3_xwy),
        ignore_attr = TRUE
      )
  })

# test_that("Standardize x, w and y: Compare bootstrapping ci", {
#     expect_equal(
#         stdmod_xwy$boot_ci["iv:mod",], stdmod2_xwy$ci, tolerance = .01,
#         ignore_attr = TRUE
#       )
#   })

skip_on_cran()

library(testthat)
library(stdmod)

# context("Check setting vcov in summary if bootstrapping is done")

dat <- test_x_1_w_1_v_1_cat1_n_500

lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)

set.seed(649831074)
stdmod_wy <- std_selected_boot(lm_raw, to_scale = ~ mod + dv, to_center = ~ mod + dv,
                               nboot = 100,
                               full_output = TRUE)
vcov_method_def <- vcov(stdmod_wy)
vcov_method_boot <- vcov(stdmod_wy, type = "boot")
vcov_method_lm <- vcov(stdmod_wy, type = "lm")
stdmod_wy_lm <- stdmod_wy
class(stdmod_wy_lm) <- "lm"
vcov_lm <- vcov(stdmod_wy_lm)
vcov_check <- cov(stdmod_wy$boot_out$t)
colnames(vcov_check) <- rownames(vcov_check) <- colnames(vcov_lm)

stdmod_wy_no_boot <- std_selected(lm_raw, to_scale = ~ mod + dv, to_center = ~ mod + dv)

test_that("vcov by method with type boot  == vcov from boot estimates", {
    expect_equal(
        vcov_method_boot, vcov_check,
        ignore_attr = TRUE
      )
  })

test_that("vcov by method with type lm == vcov from lm", {
    expect_equal(
        vcov_method_lm, vcov_lm,
        ignore_attr = TRUE
      )
  })

test_that("vcov by method default == vcov from boot since 0.2.0.0", {
    expect_equal(
        vcov_method_def, vcov_check,
        ignore_attr = TRUE
      )
  })

test_that("No boot: vcov by method default == vcov from lm", {
    expect_equal(
        vcov(stdmod_wy_no_boot), vcov_lm,
        ignore_attr = TRUE
      )
  })

test_that("No boot: vcov by method with type boot returns error", {
    expect_error(
        vcov(stdmod_wy_no_boot, type = "boot")
      )
  })

library(testthat)
library(stdmod)

# context("Check setting confint in summary if bootstrapping is done")

dat <- test_x_1_w_1_v_1_cat1_n_500

lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)

set.seed(649831074)
stdmod_wy <- std_selected_boot(lm_raw, to_scale = ~ mod + dv, to_center = ~ mod + dv,
                               nboot = 100,
                               full_output = TRUE)
confint_def <- confint(stdmod_wy)
confint_boot <- confint(stdmod_wy, type = "boot")
confint_lm <- confint(stdmod_wy, type = "lm")
confint_boot_parm <- confint(stdmod_wy, parm = "iv:mod", type = "boot")
confint_boot_conf90 <- confint(stdmod_wy, parm = "iv:mod", level = .90, type = "boot")

stdmod_wy_lm <- std_selected(lm_raw, to_scale = ~ mod + dv, to_center = ~ mod + dv)
confint_lm_no_boot <- confint(stdmod_wy_lm)

confint_boot_check <- summary(stdmod_wy)$coefficients[, c("CI Lower", "CI Upper")]
confint_boot_conf90_check <- boot::boot.ci(stdmod_wy$boot_out, conf = .90,
                                           type = "perc", index = 7)$percent[4:5]

test_that("confint_def == confint by boot since 0.2.0.0", {
    expect_equal(
        confint_def, confint_boot,
        ignore_attr = TRUE
      )
  })

test_that("confint_boot == confint from summary", {
    expect_equal(
        confint_boot, confint_boot_check,
        ignore_attr = TRUE
      )
  })

test_that("confint_boot != confint from lm", {
    expect_false(
        identical(confint_boot, confint_lm_no_boot)
      )
  })

test_that("confint_boot with user confidence level", {
    expect_true(
        all(confint_boot_conf90 == confint_boot_conf90_check)
      )
  })

test_that("confint with type 'lm'", {
    expect_true(
        all(confint_lm == confint_lm_no_boot)
      )
  })

library(testthat)
library(stdmod)
library(boot)

dat <- sleep_emo_con
lm_out <- lm(sleep_duration ~ age + gender + emotional_stability*conscientiousness, dat)
lm_std <- std_selected(lm_out,
                      to_center = ~ .,
                      to_scale = ~ .)
out_ustd <- cond_effect(lm_out, x = "emotional_stability", w = "conscientiousness",
                        w_from_mean_in_sd = 1.25)
out_std <- cond_effect(lm_std, x = "emotional_stability", w = "conscientiousness")

set.seed(875415)
boot_out_ustd <- cond_effect_boot(lm_out, x = "emotional_stability", w = "conscientiousness",
                        w_from_mean_in_sd = 1.25,
                        nboot = 50)
tmpfct <- function(d, i, lm_out) {
    if (missing(i)) {
        d_i <- d
      } else {
        d_i <- d[i, ]
      }
    lm_out_i <- update(lm_out, data = d_i)
    out0 <- cond_effect(lm_out_i, x = "emotional_stability", w = "conscientiousness",
                          w_from_mean_in_sd = 1.25)
    out0[, 3]
  }
tmpfct(dat, lm_out = lm_out)
set.seed(875415)
boot_out_ustd_check <- boot(dat, tmpfct, R = 50, lm_out = lm_out)

test_that("Check ustd boot est", {
    expect_equal(
        attr(boot_out_ustd, "boot_est"),
        boot_out_ustd_check$t,
        ignore_attr = TRUE
      )
  })

set.seed(875415)
boot_out_std <- cond_effect_boot(lm_std, x = "emotional_stability", w = "conscientiousness",
                              nboot = 50)
tmpfct <- function(d, i, lm_out) {
    if (missing(i)) {
        d_i <- d
      } else {
        d_i <- d[i, ]
      }
    lm_out_i <- update(lm_out, data = d_i)
    lm_std_i <- std_selected(lm_out_i,
                          to_standardize = ~ .)
    out0 <- cond_effect(lm_std_i, x = "emotional_stability", w = "conscientiousness")
    out0[, 3]
  }
tmpfct(dat, lm_out = lm_out)
out_std[, 3]
set.seed(875415)
boot_out_std_check <- boot(dat, tmpfct, R = 50, lm_out = lm_out)

test_that("Check std boot est", {
    expect_equal(
        attr(boot_out_std, "boot_est"),
        boot_out_std_check$t,
        ignore_attr = TRUE
      )
  })

# Work on std_selected_boot

lm_std_b <- std_selected_boot(lm_out,
                            to_center = ~ gender + emotional_stability + sleep_duration,
                            to_scale = ~ age + gender,
                            to_standardize = ~ age + conscientiousness + emotional_stability + sleep_duration,
                            nboot = 50)
set.seed(875415)
boot_out_std_b <- cond_effect_boot(lm_std_b, x = "emotional_stability", w = "conscientiousness",
                              nboot = 50)
boot_out_std_b
boot_out_std

test_that("Check cond_effect_boot on std_selected_boot", {
    expect_equal(
        attr(boot_out_std_b, "boot_est"),
        attr(boot_out_std, "boot_est"),
        ignore_attr = TRUE
      )
    expect_equal(
        as.data.frame(boot_out_std_b),
        as.data.frame(boot_out_std),
        ignore_attr = TRUE
      )
  })

library(testthat)
library(stdmod)

# context("Check standardized moderation effect given a lavaan output")

dat <- test_mod1

# Results based on std_selected

lm_med <- lm(med ~ iv * mod + cov1, dat)
lm_std <- std_selected(lm_med, to_scale = ~ ., to_center = ~ .)
summary(lm_std)$coefficients

# Results based on stdmod_lavaan

library(lavaan)
mod <-
"
med ~ iv + mod + iv:mod + cov1
dv ~ med + cov2
"
# Use likelihood = "wishart" to replicate results in lm
fit <- sem(mod, dat, likelihood = "wishart")
coef(fit)
fit_cov_implied <- lavInspect(fit, "implied")
iv_sd <- sqrt(diag(fit_cov_implied$cov)["iv"])
mod_sd <- sqrt(diag(fit_cov_implied$cov)["mod"])
med_sd <- sqrt(diag(fit_cov_implied$cov)["med"])
iv_mod <- coef(fit)["med~iv:mod"]
iv_mod_std <- iv_mod * iv_sd * mod_sd / med_sd
iv_mod_std

out_noboot <- stdmod_lavaan(fit = fit, x = "iv",
                            y = "med",
                            w = "mod",
                            x_w = "iv:mod",
                            use_old_version = TRUE)
out_noboot

set.seed(6589107)
stdmod_xyw_boot <- std_selected_boot(lm_med,
                                      to_scale = ~ .,
                                      to_center = ~ .,
                                      nboot = 100)
stdmod_xyw_boot$boot_ci["iv:mod", ]
set.seed(6589107)
system.time(out_boot <- stdmod_lavaan(fit = fit,
                                      x = "iv",
                                      y = "med",
                                      w = "mod",
                                      x_w = "iv:mod",
                                      boot_ci = TRUE,
                                      R = 100,
                                      use_old_version = TRUE))
out_boot$ci

test_that("stdmod_lavaan", {
  expect_equal(
      out_boot$ci,
      stdmod_xyw_boot$boot_ci["iv:mod", ],
      ignore_attr = TRUE
    )
  expect_equal(
      iv_mod_std,
      coef(stdmod_xyw_boot)["iv:mod"],
      ignore_attr = TRUE
    )
  })


dat <- test_mod2

# Results based on std_selected

lm_dv <- lm(dv ~ med + mod + med:mod + cov2, dat)
summary(lm_dv)$coefficients
lm_std <- std_selected(lm_dv, to_scale = ~ ., to_center = ~ .)
summary(lm_std)$coefficients

# Results based on stdmod_lavaan

library(lavaan)
mod <-
"
med ~ iv + cov1
dv ~ med + mod + med:mod + cov2
"
# Use likelihood = "wishart" to replicate results in lm
fit <- sem(mod, dat, likelihood = "wishart")
coef(fit)
fit_cov_implied <- lavInspect(fit, "implied")
med_sd <- sqrt(diag(fit_cov_implied$cov)["med"])
mod_sd <- sqrt(diag(fit_cov_implied$cov)["mod"])
dv_sd <- sqrt(diag(fit_cov_implied$cov)["dv"])
med_mod <- coef(fit)["dv~med:mod"]
med_mod_std <- med_mod * med_sd * mod_sd / dv_sd
med_mod_std

# dv_sd != sd(dat$dv) because the dv part is not saturated

out_noboot <- stdmod_lavaan(fit = fit, x = "med",
                            y = "dv",
                            w = "mod",
                            x_w = "med:mod",
                            use_old_version = TRUE)
out_noboot$stdmod

tmpfct <- function(d, i) {
    fit_i <- update(fit, data = d[i, ])
    fit_cov_implied <- lavInspect(fit_i, "implied")
    med_sd <- sqrt(diag(fit_cov_implied$cov)["med"])
    mod_sd <- sqrt(diag(fit_cov_implied$cov)["mod"])
    dv_sd <- sqrt(diag(fit_cov_implied$cov)["dv"])
    med_mod <- coef(fit_i)["dv~med:mod"]
    med_mod_std <- med_mod * med_sd * mod_sd / dv_sd
    med_mod_std
  }
set.seed(6589107)
stdmod_xyw_boot <- boot::boot(dat, tmpfct, R = 100)
set.seed(6589107)
system.time(out_boot <- stdmod_lavaan(fit = fit,
                                      x = "med",
                                      y = "dv",
                                      w = "mod",
                                      x_w = "med:mod",
                                      boot_ci = TRUE,
                                      R = 100,
                                      use_old_version = TRUE))
out_boot$boot_out$t

test_that("stdmod_lavaan", {
  expect_equal(
      out_boot$boot_out$t,
      stdmod_xyw_boot$t,
      ignore_attr = TRUE
    )
  })

library(testthat)
library(stdmod)

# context("Check standardizing selected variables with std_selected for factors")

dat <- test_x_1_w_1_v_1_cat1_n_500
dat$cat1 <- as.factor(dat$cat1)

transform0 <- function(data, vars) {
    for (x in vars) {
        data[x] <- scale(data[[x]])[, 1]
      }
    data
  }

lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
lm_zx  <- lm(dv ~ iv*mod + v1 + cat1, transform0(dat, c("iv")))
lm_zw  <- lm(dv ~ iv*mod + v1 + cat1, transform0(dat, c("mod")))
lm_zy  <- lm(dv ~ iv*mod + v1 + cat1, transform0(dat, c("dv")))
lm_zxzw  <- lm(dv ~ iv*mod + v1 + cat1, transform0(dat, c("iv", "mod")))
lm_zxzy  <- lm(dv ~ iv*mod + v1 + cat1, transform0(dat, c("iv", "dv")))
lm_zyzw  <- lm(dv ~ iv*mod + v1 + cat1, transform0(dat, c("dv", "mod")))
lm_zall  <- lm(dv ~ iv*mod + v1 + cat1, transform0(dat, c("iv", "dv", "mod")))

dat_tmp <- dat
dat_tmp$iv <- scale(dat$iv, scale = FALSE, center = TRUE)[, 1]
dat_tmp$mod <- scale(dat$mod, scale = sd(dat$mod), center = FALSE)[, 1]
lm_cxsw  <- lm(dv ~ iv*mod + v1 + cat1, dat_tmp)


stdmod_x <- std_selected(lm_raw, to_scale = ~ iv,  to_center = ~ iv)
stdmod_y <- std_selected(lm_raw, to_scale = ~ dv,  to_center = ~ dv)
stdmod_w <- std_selected(lm_raw, to_scale = ~ mod, to_center = ~ mod)
stdmod_xw <- std_selected(lm_raw, to_scale = ~ mod + iv, to_center = ~ iv + mod)
stdmod_yw <- std_selected(lm_raw, to_scale = ~ mod + dv, to_center = ~ dv + mod)
stdmod_xy <- std_selected(lm_raw, to_scale = ~ dv + iv,  to_center = ~ iv + dv)
stdmod_xyw <- std_selected(lm_raw, to_scale = ~ dv + iv + mod,  to_center = ~ mod + iv + dv)
stdmod_cxsw <- std_selected(lm_raw, to_scale = ~ mod, to_center = ~ iv)

test_that("Standardize x", {
    expect_equal(
        coef(stdmod_x), coef(lm_zx),
        ignore_attr = TRUE
      )
  })

test_that("Standardize y", {
    expect_equal(
        coef(stdmod_y), coef(lm_zy),
        ignore_attr = TRUE
      )
  })

test_that("Standardize w", {
    expect_equal(
        coef(stdmod_w), coef(lm_zw),
        ignore_attr = TRUE
      )
  })

test_that("Standardize xy", {
    expect_equal(
        coef(stdmod_xy), coef(lm_zxzy),
        ignore_attr = TRUE
      )
  })

test_that("Standardize xw", {
    expect_equal(
        coef(stdmod_xw), coef(lm_zxzw),
        ignore_attr = TRUE
      )
  })

test_that("Standardize yw", {
    expect_equal(
        coef(stdmod_yw), coef(lm_zyzw),
        ignore_attr = TRUE
      )
  })

test_that("Standardize x, y, and w", {
    expect_equal(
        coef(stdmod_xyw), coef(lm_zall),
        ignore_attr = TRUE
      )
  })

test_that("Center x and scale w", {
    expect_equal(
        coef(stdmod_cxsw), coef(lm_cxsw),
        ignore_attr = TRUE
      )
  })


stdmod_nb_x <- std_selected_boot(lm_raw, to_scale = ~ iv,  to_center = ~ iv, do_boot = FALSE)
stdmod_nb_y <- std_selected_boot(lm_raw, to_scale = ~ dv,  to_center = ~ dv, do_boot = FALSE)
stdmod_nb_w <- std_selected_boot(lm_raw, to_scale = ~ mod, to_center = ~ mod, do_boot = FALSE)
stdmod_nb_xw <- std_selected_boot(lm_raw, to_scale = ~ mod + iv, to_center = ~ iv + mod, do_boot = FALSE)
stdmod_nb_yw <- std_selected_boot(lm_raw, to_scale = ~ mod + dv, to_center = ~ dv + mod, do_boot = FALSE)
stdmod_nb_xy <- std_selected_boot(lm_raw, to_scale = ~ dv + iv,  to_center = ~ iv + dv, do_boot = FALSE)
stdmod_nb_xyw <- std_selected_boot(lm_raw, to_scale = ~ dv + iv + mod,  to_center = ~ mod + iv + dv, do_boot = FALSE)
stdmod_nb_cxsw <- std_selected_boot(lm_raw, to_scale = ~ mod, to_center = ~ iv, do_boot = FALSE)

test_that("std_selected_boot with do_boot = FALSE", {
    expect_equal(
        coef(stdmod_nb_x), coef(stdmod_x),
        ignore_attr = TRUE
      )
    expect_equal(
        coef(stdmod_nb_y), coef(stdmod_y),
        ignore_attr = TRUE
      )
    expect_equal(
        coef(stdmod_nb_w), coef(stdmod_w),
        ignore_attr = TRUE
      )
    expect_equal(
        coef(stdmod_nb_xw), coef(stdmod_xw),
        ignore_attr = TRUE
      )
    expect_equal(
        coef(stdmod_nb_yw), coef(stdmod_yw),
        ignore_attr = TRUE
      )
    expect_equal(
        coef(stdmod_nb_xy), coef(stdmod_xy),
        ignore_attr = TRUE
      )
    expect_equal(
        coef(stdmod_nb_xyw), coef(stdmod_xyw),
        ignore_attr = TRUE
      )
    expect_equal(
        coef(stdmod_nb_cxsw), coef(stdmod_cxsw),
        ignore_attr = TRUE
      )
  })

library(testthat)
library(stdmod)

# context("Check to_standardize")

dat <- test_x_1_w_1_v_1_cat1_n_500

lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)

stdmod_x <- std_selected(lm_raw, to_scale = ~ iv,  to_center = ~ iv)
stdmod_x2 <- std_selected(lm_raw, to_standardize = ~ iv)
stdmod_y <- std_selected(lm_raw, to_scale = ~ dv,  to_center = ~ dv)
stdmod_y2 <- std_selected(lm_raw, to_standardize = ~ dv)
stdmod_w <- std_selected(lm_raw, to_scale = ~ mod, to_center = ~ mod)
stdmod_w2 <- std_selected(lm_raw, to_standardize = ~ mod)
stdmod_xw <- std_selected(lm_raw, to_scale = ~ mod + iv, to_center = ~ iv + mod)
stdmod_xw2 <- std_selected(lm_raw, to_standardize = ~ mod + iv, to_center = ~ iv + mod)
stdmod_yw <- std_selected(lm_raw, to_scale = ~ mod + dv, to_center = ~ dv + mod)
stdmod_yw2 <- std_selected(lm_raw, to_standardize = ~ mod + dv)
stdmod_xy <- std_selected(lm_raw, to_scale = ~ dv + iv,  to_center = ~ iv + dv)
stdmod_xy2 <- std_selected(lm_raw, to_standardize = ~ dv + iv,  to_center = ~ iv, to_scale = ~ dv)
stdmod_xyw <- std_selected(lm_raw, to_scale = ~ dv + iv + mod,  to_center = ~ mod + iv + dv)
stdmod_xyw2 <- std_selected(lm_raw, to_standardize = ~ dv + iv + mod)
stdmod_cxsw2 <- std_selected(lm_raw, to_scale = ~ mod, to_center = ~ iv, to_standardize = ~ v1)
stdmod_cxsw3 <- std_selected(lm_raw, to_scale = ~ mod + v1, to_center = ~ v1 + iv)

test_that("Standardize x", {
    expect_equal(
        coef(stdmod_x), coef(stdmod_x2),
        ignore_attr = TRUE
      )
  })

test_that("Standardize y", {
    expect_equal(
        coef(stdmod_y), coef(stdmod_y2),
        ignore_attr = TRUE
      )
  })

test_that("Standardize w", {
    expect_equal(
        coef(stdmod_w), coef(stdmod_w2),
        ignore_attr = TRUE
      )
  })

test_that("Standardize xy", {
    expect_equal(
        coef(stdmod_xy), coef(stdmod_xy2),
        ignore_attr = TRUE
      )
  })

test_that("Standardize xw", {
    expect_equal(
        coef(stdmod_xw), coef(stdmod_xw2),
        ignore_attr = TRUE
      )
  })

test_that("Standardize yw", {
    expect_equal(
        coef(stdmod_yw), coef(stdmod_yw2),
        ignore_attr = TRUE
      )
  })

test_that("Standardize x, y, and w", {
    expect_equal(
        coef(stdmod_xyw), coef(stdmod_xyw2),
        ignore_attr = TRUE
      )
  })

test_that("Center x and scale w", {
    expect_equal(
        coef(stdmod_cxsw2), coef(stdmod_cxsw3),
        ignore_attr = TRUE
      )
  })

# Test std_selected_boot with do_boot = FALSE

stdmod_nb_x <- std_selected_boot(lm_raw, to_scale = ~ iv,  to_center = ~ iv, do_boot = FALSE)
stdmod_nb_x2 <- std_selected_boot(lm_raw, to_standardize = ~ iv, do_boot = FALSE)
stdmod_nb_y <- std_selected_boot(lm_raw, to_scale = ~ dv,  to_center = ~ dv, do_boot = FALSE)
stdmod_nb_y2 <- std_selected_boot(lm_raw, to_standardize = ~ dv, do_boot = FALSE)
stdmod_nb_w <- std_selected_boot(lm_raw, to_scale = ~ mod, to_center = ~ mod, do_boot = FALSE)
stdmod_nb_w2 <- std_selected_boot(lm_raw, to_standardize = ~ mod, do_boot = FALSE)
stdmod_nb_xw <- std_selected_boot(lm_raw, to_scale = ~ mod + iv, to_center = ~ iv + mod, do_boot = FALSE)
stdmod_nb_xw2 <- std_selected_boot(lm_raw, to_standardize = ~ mod + iv, do_boot = FALSE)
stdmod_nb_yw <- std_selected_boot(lm_raw, to_scale = ~ mod + dv, to_center = ~ dv + mod, do_boot = FALSE)
stdmod_nb_yw2 <- std_selected_boot(lm_raw, to_standardize = ~ mod + dv, do_boot = FALSE)
stdmod_nb_xy <- std_selected_boot(lm_raw, to_scale = ~ dv + iv,  to_center = ~ iv + dv, do_boot = FALSE)
stdmod_nb_xy2 <- std_selected_boot(lm_raw, to_standardize = ~ dv + iv, do_boot = FALSE)
stdmod_nb_xyw <- std_selected_boot(lm_raw, to_scale = ~ dv + iv + mod,  to_center = ~ mod + iv + dv, do_boot = FALSE)
stdmod_nb_xyw2 <- std_selected_boot(lm_raw, to_scale = ~ dv + iv,  to_center = ~ iv + dv, to_standardize = ~ mod, do_boot = FALSE)
stdmod_nb_cxsw2 <- std_selected_boot(lm_raw, to_scale = ~ mod, to_center = ~ iv, to_standardize = ~ v1, do_boot = FALSE)
stdmod_nb_cxsw3 <- std_selected_boot(lm_raw, to_scale = ~ mod + v1, to_center = ~ v1 + iv, do_boot = FALSE)

test_that("std_selected_boot with do_boot = FALSE", {
    expect_equal(
        coef(stdmod_nb_x), coef(stdmod_nb_x2),
        ignore_attr = TRUE
      )
    expect_equal(
        coef(stdmod_nb_y), coef(stdmod_nb_y2),
        ignore_attr = TRUE
      )
    expect_equal(
        coef(stdmod_nb_w), coef(stdmod_nb_w2),
        ignore_attr = TRUE
      )
    expect_equal(
        coef(stdmod_nb_xw), coef(stdmod_nb_xw2),
        ignore_attr = TRUE
      )
    expect_equal(
        coef(stdmod_nb_yw), coef(stdmod_nb_yw2),
        ignore_attr = TRUE
      )
    expect_equal(
        coef(stdmod_nb_xy), coef(stdmod_nb_xy2),
        ignore_attr = TRUE
      )
    expect_equal(
        coef(stdmod_nb_xyw), coef(stdmod_nb_xyw2),
        ignore_attr = TRUE
      )
    expect_equal(
        coef(stdmod_nb_cxsw2), coef(stdmod_nb_cxsw3),
        ignore_attr = TRUE
      )
  })

library(testthat)
library(stdmod)

set.seed(64252)
dat <- test_x_1_w_1_v_1_cat1_n_500

lm0_out <- lm(dv ~ iv + mod + v1 + cat1, dat)
lm1_out <- lm(dv ~ iv * mod + v1 + cat1, dat)
lm2_out <- lm(dv ~ iv * cat1 + mod + v1, dat)
lm3_out <- lm(dv ~ mod * cat1 + iv + v1, dat)
lm4_out <- lm(dv ~ mod * cat1 + iv*v1, dat)
lm5_out <- lm(dv ~ v1, dat)
lm0_std <- std_selected(lm0_out, to_standardize = ~ .)
lm1_std <- std_selected(lm1_out, to_standardize = ~ .)
lm2_std <- std_selected(lm2_out, to_standardize = ~ .)
lm3_std <- std_selected(lm3_out, to_standardize = ~ .)
lm4_std <- std_selected(lm4_out, to_standardize = ~ .)
lm5_std <- std_selected(lm5_out, to_standardize = ~ .)
lm0_std_boot <- std_selected_boot(lm0_out, to_standardize = ~ .)
lm1_std_boot <- std_selected_boot(lm1_out, to_standardize = ~ .)
lm2_std_boot <- std_selected_boot(lm2_out, to_standardize = ~ .)
lm3_std_boot <- std_selected_boot(lm3_out, to_standardize = ~ .)
lm4_std_boot <- std_selected_boot(lm4_out, to_standardize = ~ .)
lm5_std_boot <- std_selected_boot(lm5_out, to_standardize = ~ .)

test_that("summary.std_select with Rsq", {
    expect_false(any(grepl("adding this term",
                     capture.output(print(summary(lm0_std))))))
    expect_true(any(grepl("adding this term",
                    capture.output(print(summary(lm1_std))))))
    expect_true(any(grepl("adding this term",
                    capture.output(print(summary(lm2_std))))))
    expect_true(any(grepl("adding this term",
                    capture.output(print(summary(lm3_std))))))
    expect_false(any(grepl("adding this term",
                     capture.output(print(summary(lm4_std))))))
    expect_false(any(grepl("adding this term",
                     capture.output(print(summary(lm5_std))))))
    expect_false(any(grepl("adding this term",
                     capture.output(print(summary(lm0_std_boot))))))
    expect_true(any(grepl("adding this term",
                    capture.output(print(summary(lm1_std_boot))))))
    expect_true(any(grepl("adding this term",
                    capture.output(print(summary(lm2_std_boot))))))
    expect_true(any(grepl("adding this term",
                    capture.output(print(summary(lm3_std_boot))))))
    expect_false(any(grepl("adding this term",
                     capture.output(print(summary(lm4_std_boot))))))
    expect_false(any(grepl("adding this term",
                     capture.output(print(summary(lm5_std_boot))))))
  })

library(testthat)
library(stdmod)

dat <- test_x_1_w_1_v_1_cat1_n_500

lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)


stdmod_x <- std_selected(lm_raw, to_scale = ~ iv,  to_center = ~ iv)

stdmod_x1 <- update(stdmod_x, ~ . - cat1)

test_that("Drop a term", {
    expect_false(
        "cat1" %in% attr(terms(stdmod_x1), "term.labels")
      )
  })

dat2 <- dat[-c(1:10), ]
lm_raw2 <- lm(dv ~ iv*mod + v1 + cat1, dat2)
stdmod_x2 <- std_selected(lm_raw2, to_scale = ~ iv,  to_center = ~ iv)
stdmod_x2b <- update(stdmod_x, data = dat2)
stdmod_x2b
stdmod_x2

test_that("Change dataset", {
    expect_equal(
        coef(stdmod_x2), coef(stdmod_x2b),
        ignore_attr = TRUE
      )
  })

stdmod_x_bt <- std_selected_boot(lm_raw, to_scale = ~ iv,  to_center = ~ iv, nboot = 100)
set.seed(1234)
stdmod_x_bt2 <- update(stdmod_x_bt, data = dat2)
dat2 <- dat[-c(1:10), ]
lm_raw2 <- lm(dv ~ iv*mod + v1 + cat1, dat2)
set.seed(1234)
stdmod_x_bt2b <- std_selected_boot(lm_raw2, to_scale = ~ iv,  to_center = ~ iv, nboot = 100)

test_that("Change dataset", {
    expect_equal(
        stdmod_x_bt2$boot_ci, stdmod_x_bt2b$boot_ci,
        ignore_attr = TRUE
      )
  })

#' @title Sample Dataset: A Path Model With A Moderator
#'
#' @description For testing the handling of warnings
#' in [stdmod_lavaan()]. Generated from the following
#' model. `dv` has about 88% missing. A warning on missing
#' data will be raised in some bootstrap samples.
#'
#'```
#'mod <-
#'"
#'med ~ iv + mod + iv:mod + cov1
#'dv ~ med + cov2
#'"
#'```
#'
#' @format A data frame with 500 rows and 6 variables:
#' \describe{
#'   \item{dv}{Dependent variable, continuous}
#'   \item{iv}{Independent variable, continuous}
#'   \item{med}{Mediator, continuous}
#'   \item{mod}{Moderator, continuous}
#'   \item{cov1}{Covariate, continuous}
#'   \item{cov2}{Covariate, continuous}
#' }
#'
"test_mod3_miss"

#' @title Conditional Effects
#'
#' @description Compute the conditional effects in a moderated regression model.
#'
#' @details [cond_effect()] uses the centering approach to find the conditional
#' effect of the focal variable. For each level of the moderator, the value for
#' this level is subtracted from the moderator scores, and the model is
#' fitted to the modified data.
#' The coefficient of the focal variable is then the conditional effect of the
#' focal variable when the moderator's score is equal this value.
#'
#' @return
#' [cond_effect()] returns a data-frame-like object of the conditional effects.
#' The class is
#' `cond_effect` and the print method will print additional information of
#' the conditional effects. Additional information is stored in the
#' following attributes:
#'
#' - `call`: The original call.
#'
#' - `output`: The `output` object, such as the output from `lm()`.
#'
#' - `x`, `y`, and `w`: The three variables used to compute the conditional
#'                      effects: focal variable (`x`), outcome variable (`y`),
#'                      and the moderator (`w`).
#'
#'  - `w_method`: The method used to determine the values of the moderator
#'                at the selected levels.
#'
#'  - `w_percentiles` The percentiles to use if `w_method` = `"percentile"`.
#'
#'  - `w_sd_to_percentiles`: If not equal to `NA`, this is a scalar, the
#'                           number of standard deviation from the mean used to
#'                           determine the percentiles for the "low" and "high"
#'                           levels of the moderator.
#'  - `w_from_mean_in_sd`: The number of SD above or below the mean, for
#'                          determining the "low" and "high" levels of the
#'                          moderator if `w_method` is `"sd"`.
#'  - `w_empirical_percentiles`: The actual percentile levels in the dataset
#'                               for the selected
#'                               levels of the moderator. A numeric vector.
#'  - `w_empirical_z`: The actual distance from the mean, in SD, of each
#'                     selected level of the moderator. A numeric vector.
#'  - `y_standardized`, `x_standardized`, and `w_standardized`: Each of them
#'                     is a logical scalar, indicating whether the outcome
#'                     variable, focal variable, and moderator are standardized.
#'
#' @param output The output from [stats::lm()]. It can also accept the output
#'               from
#'               [std_selected()] or [std_selected_boot()].
#' @param x      The focal variable (independent variable), that is, the
#'               variable with its effect on the outcome variable (dependent)
#'              being moderated. It must be a numeric variable.
#' @param w      The moderator. Both numeric variables and categorical variables
#'               (character or factor) are supported.
#' @param w_method How to define "low", "medium", and "high" for the moderator
#'                  levels.
#'                  Default is in terms of mean and
#'                  standard deviation (SD) of the moderator, `"sd"`:
#'                  "low", "medium", and "high" are one SD below mean,
#'                  mean, and one SD above mean, respectively.
#'                  If equal to
#'                  `"percentile"`, then percentiles of the moderator in
#'                  the
#'                  dataset are used: "low", "medium", and "high" are
#'                  16th, 50th (median), and 84th percentiles, respectively.
#'                  Ignored if `w` is categorical.
#' @param w_percentiles If `w_method` is `"percentile"`, then this
#'                      argument
#'                      specifies the three percentiles to be used,
#'                      divided by 100.
#'                        It must be a
#'                      vector of two numbers. The default is
#'                      `c(.16, .50, .84)`,
#'                      the 16th, 50th, and 84th percentiles,
#'                      which corresponds approximately
#'                      to one SD below and above mean in a
#'                      normal distribution, respectively.
#'                      Ignored if `w` is categorical.
#' @param w_sd_to_percentiles If `w_method` is `"percentile"` and
#'                            this argument is
#'                            set to a number, this number will be
#'                            used to
#'                            to determine the percentiles to be used.
#'                            The
#'                            lower percentile is the percentile in a
#'                            normal
#'                            distribution
#'                            that is `w_sd_to_percentiles` SD below
#'                            the mean.
#'                            The upper percentile is the percentile in
#'                            a normal
#'                            distribution that is `w_sd_to_percentiles`
#'                            SD
#'                            above the mean. Therefore, if
#'                            `w_sd_to_percentiles` is set to 1, then the
#'                            lower
#'                            and upper percentiles are 16th and 84th,
#'                            respectively. Default is `NA`.
#' @param w_from_mean_in_sd How many SD from mean is used to define
#'                          "low" and
#'                          "high" for the moderator. Default is 1.
#'                          Ignored if `w` is categorical.
#'
#' @param w_values The values of `w` to
#' be used. Default is `NULL`. If a
#' numeric vector is supplied, these
#' values will be used to compute the
#' conditional effects. Other arguments
#' on generating levels are ignored.
#' Note that, if `w` has been standardized
#' or centered, these values are for
#' the standardized or centered `w`.
#' The values will always be sorted.
#' This argument is ignored if `w` is
#' categorical.
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#'
#' @examples
#'
#' # Load a sample data set
#'
#' dat <- test_x_1_w_1_v_1_cat1_n_500
#'
#' # Do a moderated regression by lm
#' lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
#' summary(lm_raw)
#' cond_effect(lm_raw, x = iv, w = mod)
#'
#' lm_std <- std_selected(lm_raw, to_standardize = ~ iv + mod)
#' cond_effect(lm_std, x = iv, w = mod)
#'
#' # Categorical moderator
#' lm_cat <- lm(dv ~ iv*cat1 + v1, dat)
#' summary(lm_cat)
#' cond_effect(lm_cat, x = iv, w = cat1)
#'
#' @export
#' @order 1

cond_effect <- function(output,
                      x = NULL,
                      w = NULL,
                      w_method = c("sd", "percentile"),
                      w_percentiles = c(.16, .50, .84),
                      w_sd_to_percentiles = NA,
                      w_from_mean_in_sd = 1,
                      w_values = NULL
                      ) {
    mf0 <- stats::model.frame(output)
    w_method <- match.arg(w_method)
    x0 <- deparse(substitute(x))
    if (inherits(tryCatch(x00 <- as.character(x), error = function(e) e),
                 "error")) {
        x <- x0
      } else {
        x <- x00
      }
    w0 <- deparse(substitute(w))
    if (inherits(tryCatch(w00 <- as.character(w), error = function(e) e),
                 "error")) {
        w <- w0
      } else {
        w <- w00
      }
    y <- colnames(stats::model.frame(output))[
                    attr(stats::terms(output), "response")
                  ]
    x_numeric <- is.numeric(mf0[, x])
    w_numeric <- is.numeric(mf0[, w])
    if (!x_numeric) {
        stop("x variable must be a numeric variable.")
      }
    output0 <- output
    if (inherits(output, "std_selected")) {
        tmp <- class(output0)
        class(output0) <- tmp[!(tmp %in% "std_selected")]
      }
    if (w_numeric) {
        if (is.numeric(w_values)) {
            w_levels <- sort(w_values, decreasing = TRUE)
            w_levels_labels <- as.character(w_levels)
          } else {
            w_levels <- gen_levels(mf0[, w],
                                  method = w_method,
                                  from_mean_in_sd = w_from_mean_in_sd,
                                  levels = c(-1, 0, 1),
                                  sd_levels = c(-1, 0, 1),
                                  sd_to_percentiles = w_sd_to_percentiles,
                                  percentiles = w_percentiles)
            w_levels <- sort(w_levels, decreasing = TRUE)
            w_levels_labels <- c("High", "Medium", "Low")
          }
      } else {
        w_levels <- levels(as.factor(mf0[, w]))
        w_levels_labels <- w_levels
      }
    if (w_numeric) {
        out_all <- lapply(w_levels, cond_numeric_i,
                          output = output0,
                          w = w)
      } else {
        out_all <- lapply(w_levels, cond_categorical_i,
                          output = output0,
                          w = w)
      }
    coef_all <- lapply(out_all, function(xi) summary(xi)$coefficients[x, ])
    out0 <- do.call(rbind, coef_all)
    out0 <- data.frame(Level = w_levels_labels,
                       w = w_levels,
                       out0, check.names = FALSE)
    colnames(out0)[2] <- w
    colnames(out0)[3] <- "x's Effect"
    mycall <- match.call()
    attr(out0, "call") <- mycall
    attr(out0, "output") <- output
    attr(out0, "x") <- x
    attr(out0, "y") <- y
    attr(out0, "w") <- w
    attr(out0, "w_method") <- w_method
    attr(out0, "w_percentiles") <- w_percentiles
    attr(out0, "w_sd_to_percentiles") <- w_sd_to_percentiles
    attr(out0, "w_from_mean_in_sd") <- w_from_mean_in_sd
    if (w_numeric) {
        wecdf <- stats::ecdf(mf0[, w])
        w_pq <- wecdf(w_levels)
        w_mean <- mean(mf0[, w], na.rm = TRUE)
        w_sd  <- stats::sd(mf0[, w], na.rm = TRUE)
        w_z <- (w_levels - w_mean) / w_sd
      } else {
        w_pq <- rep(NA, length(w_levels))
        w_z <- rep(NA, length(w_levels))
      }
    attr(out0, "w_empirical_percentiles") <- w_pq
    attr(out0, "w_empirical_z") <- w_z
    attr(out0, "y_standardized") <- is_standardized(mf0[, y])
    attr(out0, "x_standardized") <- is_standardized(mf0[, x])
    attr(out0, "w_standardized") <- is_standardized(mf0[, w])
    attr(out0, "out_all") <- out_all
    class(out0) <- c("cond_effect", class(out0))
    out0
  }

cond_numeric_i <- function(w_i, output, w) {
    mf_i <- stats::model.frame(output)
    mf_i[, w] <- mf_i[, w] - w_i
    out <- stats::update(output, data = mf_i)
    out
  }

cond_categorical_i <- function(w_i, output, w) {
    mf_i <- stats::model.frame(output)
    mf_i[, w] <- stats::relevel(as.factor(mf_i[, w]),
                                ref = w_i)
    out <- stats::update(output, data = mf_i)
    out
  }

library(testthat)
library(stdmod)

dat <- sleep_emo_con
lm_out <- lm(sleep_duration ~ age + gender + emotional_stability*conscientiousness, dat)
lm_std <- std_selected(lm_out,
                      to_center = ~ .,
                      to_scale = ~ .)
out_ustd <- cond_effect(lm_out, x = "emotional_stability", w = "conscientiousness",
                        w_from_mean_in_sd = 1.25)
out_std <- cond_effect(lm_std, x = "emotional_stability", w = "conscientiousness")

w_mean <- mean(dat$conscientiousness)
w_sd <- sd(dat$conscientiousness)
dat$w_lo <- dat$conscientiousness - (w_mean - 1.25 * w_sd)
dat$w_me <- dat$conscientiousness - w_mean
dat$w_hi <- dat$conscientiousness - (w_mean + 1.25 * w_sd)
lm_out_lo <-  lm(sleep_duration ~ age + gender + emotional_stability*w_lo, dat)
lm_out_me <-  lm(sleep_duration ~ age + gender + emotional_stability*w_me, dat)
lm_out_hi <-  lm(sleep_duration ~ age + gender + emotional_stability*w_hi, dat)
out_ustd_check <- c(coef(lm_out_hi)["emotional_stability"],
                    coef(lm_out_me)["emotional_stability"],
                    coef(lm_out_lo)["emotional_stability"])
out_ustd[, 3]

dat_std <- data.frame(scale(dat[, 2:5]), gender = dat$gender)
dat_std$w_lo <- dat_std$conscientiousness - (-1)
dat_std$w_me <- dat_std$conscientiousness
dat_std$w_hi <- dat_std$conscientiousness - (1)
lm_std_lo <-  lm(sleep_duration ~ age + gender + emotional_stability*w_lo, dat_std)
lm_std_me <-  lm(sleep_duration ~ age + gender + emotional_stability*w_me, dat_std)
lm_std_hi <-  lm(sleep_duration ~ age + gender + emotional_stability*w_hi, dat_std)
out_std_check <- c(coef(lm_std_hi)["emotional_stability"],
                    coef(lm_std_me)["emotional_stability"],
                    coef(lm_std_lo)["emotional_stability"])
out_std[, 3]

test_that("Check ustd, mean +/- a SD", {
    expect_equal(
        out_ustd_check, out_ustd[, 3],
        ignore_attr = TRUE
      )
  })

test_that("Check std, mean +/- a SD", {
    expect_equal(
        out_std_check, out_std[, 3],
        ignore_attr = TRUE
      )
  })

lm_std2 <- std_selected(lm_out,
                        to_center = ~ age,
                        to_scale = ~ .,
                        to_standardize = ~ .)
out_std2 <- cond_effect(lm_std2, x = "emotional_stability", w = "conscientiousness")

test_that("cond_effect, to_standardize", {
    expect_equal(
        coef(out_std),
        coef(out_std2),
        ignore_attr = TRUE
      )
  })

out_std <- cond_effect(lm_std, x = "emotional_stability", w = "conscientiousness")


out_ustd_p <- cond_effect(lm_out, x = "emotional_stability", w = "conscientiousness",
                          w_method = "percentile", w_percentiles = c(.20, .40, .90))
out_std_p <- cond_effect(lm_std, x = "emotional_stability", w = "conscientiousness",
                          w_method = "percentile", w_sd_to_percentiles = 1.25)

dat$w_p1 <- dat$conscientiousness - quantile(dat$conscientiousness, .20)
dat$w_p2 <- dat$conscientiousness - quantile(dat$conscientiousness, .40)
dat$w_p3 <- dat$conscientiousness - quantile(dat$conscientiousness, .90)
lm_out_p1 <-  lm(sleep_duration ~ age + gender + emotional_stability*w_p1, dat)
lm_out_p2 <-  lm(sleep_duration ~ age + gender + emotional_stability*w_p2, dat)
lm_out_p3 <-  lm(sleep_duration ~ age + gender + emotional_stability*w_p3, dat)
out_ustd_p_check <- c(coef(lm_out_p3)["emotional_stability"],
                    coef(lm_out_p2)["emotional_stability"],
                    coef(lm_out_p1)["emotional_stability"])
out_ustd_p[, 3]

dat_std <- data.frame(scale(dat[, 2:5]), gender = dat$gender)
dat_std$w_p1 <- dat_std$conscientiousness - quantile(dat_std$conscientiousness, stats::pnorm(-1.25))
dat_std$w_p2 <- dat_std$conscientiousness - quantile(dat_std$conscientiousness, stats::pnorm(0))
dat_std$w_p3 <- dat_std$conscientiousness - quantile(dat_std$conscientiousness, stats::pnorm(1.25))
lm_std_p1 <-  lm(sleep_duration ~ age + gender + emotional_stability*w_p1, dat_std)
lm_std_p2 <-  lm(sleep_duration ~ age + gender + emotional_stability*w_p2, dat_std)
lm_std_p3 <-  lm(sleep_duration ~ age + gender + emotional_stability*w_p3, dat_std)
out_std_p_check <- c(coef(lm_std_p3)["emotional_stability"],
                    coef(lm_std_p2)["emotional_stability"],
                    coef(lm_std_p1)["emotional_stability"])
out_std_p[, 3]


test_that("Check ustd, percentile", {
    expect_equal(
        out_ustd_p[, 3], out_ustd_p_check,
        ignore_attr = TRUE
      )
  })

test_that("Check ustd, percentile", {
    expect_equal(
        out_std_p[, 3], out_std_p_check,
        ignore_attr = TRUE
      )
  })

# Categorical Moderator


dat <- sleep_emo_con
set.seed(4807859)
dat$city <- sample(c("Alpha", "Beta", "Gamma"), nrow(dat), replace = TRUE)
lm_out_wcat3 <- lm(sleep_duration ~ age + city*emotional_stability + conscientiousness, dat)
lm_std_wcat3 <- std_selected(lm_out_wcat3,
                      to_center = ~ .,
                      to_scale = ~ .)
dat_std <- data.frame(scale(dat[, 2:5]), gender = dat$gender, city = dat$city)
lm_std_wcat3_check <- lm(sleep_duration ~ age + city*emotional_stability + conscientiousness,
                   dat_std)
identical(coef(lm_std_wcat3), coef(lm_std_wcat3_check))

out_ustd_wcat3 <- cond_effect(lm_out_wcat3, x = "emotional_stability", w = "city")
out_std_wcat3 <- cond_effect(lm_std_wcat3, x = "emotional_stability", w = "city")

dat_tmp <- dat
dat_tmp$city <- relevel(factor(dat_tmp$city), ref = "Alpha")
lm_out_Alpha <-  lm(sleep_duration ~ age + city*emotional_stability + conscientiousness, dat_tmp)
dat_tmp$city <- relevel(factor(dat_tmp$city), ref = "Beta")
lm_out_Beta <-  lm(sleep_duration ~ age + city*emotional_stability + conscientiousness, dat_tmp)
dat_tmp$city <- relevel(factor(dat_tmp$city), ref = "Gamma")
lm_out_Gamma <-  lm(sleep_duration ~ age + city*emotional_stability + conscientiousness, dat_tmp)
out_ustd_wcat3_check <- c(coef(lm_out_Alpha)["emotional_stability"],
                         coef(lm_out_Beta)["emotional_stability"],
                         coef(lm_out_Gamma)["emotional_stability"])
out_ustd_wcat3[, 3]


dat_tmp <- dat
dat_tmp_std <- data.frame(scale(dat_tmp[, 2:5]), age = dat$gender, city = dat$city)
dat_tmp_std$city <- relevel(factor(dat_tmp_std$city), ref = "Alpha")
lm_std_Alpha <-  lm(sleep_duration ~ age + city*emotional_stability + conscientiousness, dat_tmp_std)
dat_tmp_std$city <- relevel(factor(dat$city), ref = "Beta")
lm_std_Beta <-  lm(sleep_duration ~ age + city*emotional_stability + conscientiousness, dat_tmp_std)
dat_tmp_std$city <- relevel(factor(dat$city), ref = "Gamma")
lm_std_Gamma <-  lm(sleep_duration ~ age + city*emotional_stability + conscientiousness, dat_tmp_std)
out_std_wcat3_check <- c(coef(lm_std_Alpha)["emotional_stability"],
                         coef(lm_std_Beta)["emotional_stability"],
                         coef(lm_std_Gamma)["emotional_stability"])
out_std_wcat3[, 3]

test_that("Check ustd, 3 categories", {
    expect_equal(
        out_ustd_wcat3[, 3], out_ustd_wcat3_check,
        ignore_attr = TRUE
      )
  })

test_that("Check std, 3 categories", {
    expect_equal(
        out_std_wcat3[, 3], out_std_wcat3_check,
        ignore_attr = TRUE
      )
  })

# Test cond_effect_boot with do_boot = FALSE

out_nb_ustd <- cond_effect_boot(lm_out, x = "emotional_stability", w = "conscientiousness",
                        w_from_mean_in_sd = 1.25, do_boot = FALSE)
out_nb_std <- cond_effect_boot(lm_std, x = "emotional_stability", w = "conscientiousness", do_boot = FALSE)
out_nb_ustd_p <- cond_effect_boot(lm_out, x = "emotional_stability", w = "conscientiousness",
                          w_method = "percentile", w_percentiles = c(.20, .40, .90), do_boot = FALSE)
out_nb_std_p <- cond_effect_boot(lm_std, x = "emotional_stability", w = "conscientiousness",
                          w_method = "percentile", w_sd_to_percentiles = 1.25, do_boot = FALSE)
out_nb_ustd_wcat3 <- cond_effect_boot(lm_out_wcat3, x = "emotional_stability", w = "city", do_boot = FALSE)
out_nb_std_wcat3 <- cond_effect_boot(lm_std_wcat3, x = "emotional_stability", w = "city", do_boot = FALSE)


test_that("cond_effect_boot with do_boot = FALSE", {
    expect_equal(
        out_nb_ustd, out_ustd,
        ignore_attr = TRUE
      )
    expect_equal(
        out_nb_std, out_std,
        ignore_attr = TRUE
      )
    expect_equal(
        out_nb_ustd_p, out_ustd_p,
        ignore_attr = TRUE
      )
    expect_equal(
        out_nb_std_p, out_std_p,
        ignore_attr = TRUE
      )
    expect_equal(
        out_nb_ustd_wcat3, out_ustd_wcat3,
        ignore_attr = TRUE
      )
    expect_equal(
        out_nb_std_wcat3, out_std_wcat3,
        ignore_attr = TRUE
      )
  })

library(testthat)
library(stdmod)

dat <- sleep_emo_con
lm_out <- lm(sleep_duration ~ age + gender + emotional_stability*conscientiousness, dat)
lm_std <- std_selected(lm_out,
                      to_center = ~ .,
                      to_scale = ~ .)
out_ustd <- cond_effect(lm_out, x = "emotional_stability", w = "conscientiousness",
                        w_from_mean_in_sd = 1.25)
out_std <- cond_effect(lm_std, x = "emotional_stability", w = "conscientiousness")

out_ustd_p <- cond_effect(lm_out, x = "emotional_stability", w = "conscientiousness",
                          w_method = "percentile", w_percentiles = c(.20, .40, .90))
out_std_p <- cond_effect(lm_std, x = "emotional_stability", w = "conscientiousness",
                          w_method = "percentile", w_sd_to_percentiles = 1.25)


test_that("confint.cond_effect", {
    expect_equal(
        confint(out_ustd),
        t(sapply(attr(out_ustd, "out_all"), function(x) {confint(x)["emotional_stability", ]})),
        ignore_attr = TRUE
      )
    expect_equal(
        confint(out_std, level = .90),
        t(sapply(attr(out_std, "out_all"), function(x) {confint(x, level = .90)["emotional_stability", ]})),
        ignore_attr = TRUE
      )
    expect_equal(
        confint(out_ustd_p, level = .80),
        t(sapply(attr(out_ustd_p, "out_all"), function(x) {confint(x, level = .80)["emotional_stability", ]})),
        ignore_attr = TRUE
      )
    expect_equal(
        confint(out_std_p, level = .91),
        t(sapply(attr(out_std_p, "out_all"), function(x) {confint(x, level = .91)["emotional_stability", ]})),
        ignore_attr = TRUE
      )
  })

test_that("coef.cond_effect", {
    expect_equal(
        coef(out_ustd),
        as.data.frame(out_ustd)[, "x's Effect"],
        ignore_attr = TRUE
      )
    expect_equal(
        coef(out_std),
        as.data.frame(out_std)[, "x's Effect"],
        ignore_attr = TRUE
      )
    expect_equal(
        coef(out_ustd_p),
        as.data.frame(out_ustd_p)[, "x's Effect"],
        ignore_attr = TRUE
      )
    expect_equal(
        coef(out_std_p, level = .91),
        as.data.frame(out_std_p)[, "x's Effect"],
        ignore_attr = TRUE
      )
  })


# Categorical Moderator


dat <- sleep_emo_con
set.seed(4807859)
dat$city <- sample(c("Alpha", "Beta", "Gamma"), nrow(dat), replace = TRUE)
lm_out_wcat3 <- lm(sleep_duration ~ age + city*emotional_stability + conscientiousness, dat)
lm_std_wcat3 <- std_selected(lm_out_wcat3,
                      to_center = ~ .,
                      to_scale = ~ .)
out_ustd_wcat3 <- cond_effect(lm_out_wcat3, x = "emotional_stability", w = "city")
out_std_wcat3 <- cond_effect(lm_std_wcat3, x = "emotional_stability", w = "city")

test_that("confint.cond_effect, categorical", {
    expect_equal(
        confint(out_ustd_wcat3),
        t(sapply(attr(out_ustd_wcat3, "out_all"), function(x) {confint(x)["emotional_stability", ]})),
        ignore_attr = TRUE
      )
    expect_equal(
        confint(out_std_wcat3, level = .90),
        t(sapply(attr(out_std_wcat3, "out_all"), function(x) {confint(x, level = .90)["emotional_stability", ]})),
        ignore_attr = TRUE
      )
    expect_equal(
        confint(out_ustd_wcat3, level = .80),
        t(sapply(attr(out_ustd_wcat3, "out_all"), function(x) {confint(x, level = .80)["emotional_stability", ]})),
        ignore_attr = TRUE
      )
    expect_equal(
        confint(out_std_wcat3, level = .91),
        t(sapply(attr(out_std_wcat3, "out_all"), function(x) {confint(x, level = .91)["emotional_stability", ]})),
        ignore_attr = TRUE
      )
  })

test_that("coef.cond_effect, categorical", {
    expect_equal(
        coef(out_ustd_wcat3),
        as.data.frame(out_ustd_wcat3)[, "x's Effect"],
        ignore_attr = TRUE
      )
    expect_equal(
        coef(out_std_wcat3),
        as.data.frame(out_std_wcat3)[, "x's Effect"],
        ignore_attr = TRUE
      )
    expect_equal(
        coef(out_ustd_wcat3),
        as.data.frame(out_ustd_wcat3)[, "x's Effect"],
        ignore_attr = TRUE
      )
    expect_equal(
        coef(out_std_wcat3),
        as.data.frame(out_std_wcat3)[, "x's Effect"],
        ignore_attr = TRUE
      )
  })

library(testthat)
library(stdmod)

dat <- sleep_emo_con
lm_out <- lm(sleep_duration ~ age + gender + emotional_stability*conscientiousness, dat)
lm_std <- std_selected(lm_out,
                      to_center = ~ .,
                      to_scale = ~ .)
out_ustd <- cond_effect(lm_out, x = "emotional_stability", w = "conscientiousness",
                        w_values = c(20, 3, 54, -4))
out_std <- cond_effect(lm_std, x = "emotional_stability", w = "conscientiousness",
                       w_values = c(-1.5, 23))

dat$w_a <- dat$conscientiousness - 54
dat$w_b <- dat$conscientiousness - 20
dat$w_c <- dat$conscientiousness - 3
dat$w_d <- dat$conscientiousness - -4
lm_out_a <-  lm(sleep_duration ~ age + gender + emotional_stability*w_a, dat)
lm_out_b <-  lm(sleep_duration ~ age + gender + emotional_stability*w_b, dat)
lm_out_c <-  lm(sleep_duration ~ age + gender + emotional_stability*w_c, dat)
lm_out_d <-  lm(sleep_duration ~ age + gender + emotional_stability*w_d, dat)
out_ustd_check <- c(coef(lm_out_a)["emotional_stability"],
                    coef(lm_out_b)["emotional_stability"],
                    coef(lm_out_c)["emotional_stability"],
                    coef(lm_out_d)["emotional_stability"])
out_ustd[, 3]

dat_std <- data.frame(scale(dat[, 2:5]), gender = dat$gender)
dat_std$w_a <- dat_std$conscientiousness - 23
dat_std$w_b <- dat_std$conscientiousness - (-1.5)
lm_std_a <-  lm(sleep_duration ~ age + gender + emotional_stability*w_a, dat_std)
lm_std_b <-  lm(sleep_duration ~ age + gender + emotional_stability*w_b, dat_std)
out_std_check <- c(coef(lm_std_a)["emotional_stability"],
                   coef(lm_std_b)["emotional_stability"])
out_std[, 3]

test_that("Check ustd", {
    expect_equal(
        out_ustd_check, out_ustd[, 3],
        ignore_attr = TRUE
      )
  })

test_that("Check std", {
    expect_equal(
        out_std_check, out_std[, 3],
        ignore_attr = TRUE
      )
  })

# Test cond_effect_boot with do_boot = FALSE

out_nb_ustd <- cond_effect_boot(lm_out, x = "emotional_stability", w = "conscientiousness",
                                w_values = c(20, 3, 54, -4), do_boot = FALSE)
out_nb_std <- cond_effect_boot(lm_std, x = "emotional_stability", w = "conscientiousness",,
                               w_values = c(-1.5, 23), do_boot = FALSE)

test_that("cond_effect_boot with do_boot = FALSE", {
    expect_equal(
        out_nb_ustd, out_ustd,
        ignore_attr = TRUE
      )
    expect_equal(
        out_nb_std, out_std,
        ignore_attr = TRUE
      )
  })

library(testthat)
library(stdmod)
library(boot)

dat <- sleep_emo_con
lm_out <- lm(sleep_duration ~ age + gender + emotional_stability*conscientiousness, dat)
lm_std <- std_selected(lm_out,
                      to_center = ~ .,
                      to_scale = ~ .)
out_ustd <- cond_effect(lm_out, x = "emotional_stability", w = "conscientiousness",
                        w_from_mean_in_sd = 1.25)
out_std <- cond_effect(lm_std, x = "emotional_stability", w = "conscientiousness")

set.seed(875415)
boot_out_ustd <- cond_effect_boot(lm_out, x = "emotional_stability", w = "conscientiousness",
                        w_from_mean_in_sd = 1.25,
                        nboot = 50)


set.seed(875415)
boot_out_std <- cond_effect_boot(lm_std, x = "emotional_stability", w = "conscientiousness",
                              nboot = 50)


test_that("confint.cond_effect", {
    expect_equal(
        confint(boot_out_ustd),
        attr(boot_out_ustd, "boot_ci"),
        ignore_attr = TRUE
      )
    expect_true(
        all(confint(boot_out_ustd, level = .90) %*% matrix(c(-1, 1), 2, 1) <
            attr(boot_out_ustd, "boot_ci") %*% matrix(c(-1, 1), 2, 1))
      )
    expect_equal(
        confint(boot_out_std),
        attr(boot_out_std, "boot_ci"),
        ignore_attr = TRUE
      )
    expect_true(
        all(confint(boot_out_std, level = .90) %*% matrix(c(-1, 1), 2, 1) <
            attr(boot_out_std, "boot_ci") %*% matrix(c(-1, 1), 2, 1))
      )
  })

test_that("coef.cond_effect", {
    expect_equal(
        coef(boot_out_ustd),
        as.data.frame(out_ustd)[, "x's Effect"],
        ignore_attr = TRUE
      )
    expect_equal(
        coef(boot_out_std),
        as.data.frame(boot_out_std)[, "x's Effect"],
        ignore_attr = TRUE
      )
  })

#' @title Sample Dataset: One IV, One Moderator, Two Covariates
#'
#' @description The independent variable and the moderator are
#'  associated. For demonstrating the use of tumble graph.
#'
#' @format A data frame with 500 rows and 5 variables:
#' \describe{
#'   \item{dv}{Dependent variable, continuous}
#'   \item{iv}{Independent variable, continuous}
#'   \item{mod}{Moderator variable, continuous}
#'   \item{v1}{Covariate, continuous}
#'   \item{cat1}{Covariate, categorical (string) with three values, "gp1",
#'                 "gp2", and "gp3"}
#' }
#'
"test_x_1_w_1_v_1_cat1_xw_cov_n_500"

#' @title Sample Dataset: One IV, One 3-Category Moderator, Two Covariates
#'
#' @description The independent variable and the categorical
#'  moderator are
#'  associated. For demonstrating the use of tumble graph.
#'
#' @format A data frame with 500 rows and 5 variables:
#' \describe{
#'   \item{dv}{Dependent variable, continuous}
#'   \item{iv}{Independent variable, continuous}
#'   \item{mod}{Moderator variable, categorical (string) with three categories,
#'              "City Alpha", "City Gamma", and "City Beta"}
#'   \item{v1}{Covariate, continuous}
#'   \item{cat1}{Covariate, categorical (string) with three values, "gp1",
#'               "gp2", and "gp3"}
#' }
#'
"test_x_1_w_1_v_1_cat1_xw_cov_wcat3_n_500"

#' @title Standardized Moderation Effect in a 'stdmod_lavaan' Class Object
#'
#' @description Return the estimate of the standardized
#'              moderation effect
#'              in the output of [stdmod_lavaan()].
#'
#' @details It just extracts and returns the element `stdmod`.
#'
#' @return
#'  A scalar: The estimate of the standardized moderation effect.
#'
#' @param object The output of [stdmod_lavaan()].
#' @param ...  Optional arguments. Ignored by the function.
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#'
#' @examples
#'
#' # Load a test data of 500 cases
#' dat <- test_mod1
#' library(lavaan)
#'
#' mod <-
#' "
#' med ~ iv + mod + iv:mod + cov1
#' dv ~ med + cov2
#' "
#' fit <- sem(mod, dat)
#' coef(fit)
#'
#' # Compute the standardized moderation effect
#' out_noboot <- stdmod_lavaan(fit = fit,
#'                             x = "iv",
#'                             y = "med",
#'                             w = "mod",
#'                             x_w = "iv:mod")
#' coef(out_noboot)
#'
#' # Compute the standardized moderation effect and
#' # its confidence interval based on nonparametric bootstrapping
#' # Fit the model with bootstrap confidence intervals
#' # At least 2000 bootstrap samples should be used
#' # in real research. 50 is used here only for
#' # illustration.
#' fit <- sem(mod, dat, se = "boot", bootstrap = 50,
#'            iseed = 89574)
#' out_boot <- stdmod_lavaan(fit = fit,
#'                           x = "iv",
#'                           y = "med",
#'                           w = "mod",
#'                           x_w = "iv:mod",
#'                           boot_ci = TRUE)
#' coef(out_boot)
#'
#' @export


coef.stdmod_lavaan <- function(object, ...) {
    object$stdmod
  }

#' @title Confidence Intervals for a 'stdmod_lavaan' Class Object
#'
#' @description Return the confidence interval of the standardized
#'              moderation effect
#'              in the output of [stdmod_lavaan()].
#'
#' @details If bootstrapping is used to form the confidence interval by
#' [stdmod_lavaan()],
#' users can request the percentile confidence interval of
#' using the stored bootstrap estimate.
#'
#' @return
#'  A one-row matrix of the confidence intervals.
#'
#' @param object The output of [stdmod_lavaan()].
#' @param parm Ignored. Always return the bootstrap confidence interval
#'              of the standardized moderation effect.
#' @param level The level of confidence, default is .95, returning the
#'               95% confidence interval.
#' @param ...  Additional arguments. Ignored by the function.
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#'
#' @examples
#'
#' # Load a test data of 500 cases
#' dat <- test_mod1
#' library(lavaan)
#'
#' mod <-
#' "
#' med ~ iv + mod + iv:mod + cov1
#' dv ~ med + cov2
#' "
#' fit <- sem(mod, dat)
#' coef(fit)
#'
#' # Compute the standardized moderation effect and
#' # its confidence interval based on nonparametric bootstrapping
#' # Fit the model with bootstrap confidence intervals
#' # At least 2000 bootstrap samples should be used
#' # in real research. 50 is used here only for
#' # illustration.
#' fit <- sem(mod, dat, se = "boot", bootstrap = 50,
#'            iseed = 89574)
#' out_boot <- stdmod_lavaan(fit = fit,
#'                           x = "iv",
#'                           y = "med",
#'                           w = "mod",
#'                           x_w = "iv:mod",
#'                           boot_ci = TRUE)
#' confint(out_boot)
#'
#' @export


confint.stdmod_lavaan <- function(object, parm, level = .95, ...) {
    if (isTRUE(object$call$boot_ci)) {
        out0 <- boot::boot.ci(object$boot_out,
                            type = "perc",
                            conf = level)$percent[4:5]
      } else {
        warning("Bootstrapping output not in the object.")
        out0 <- c(NA, NA)
      }
    # Borrowed from stats::confint()
    probs <- c((1 - level) / 2, 1 - (1 - level) / 2)
    cnames <- paste(format(100 * probs,
                           trim = TRUE,
                           scientific = FALSE,
                           digits = 2), "%")
    rnames <- paste0(object$call$y, "~", object$call$x_w)
    out <- array(data = out0,
                 dim = c(1, 2),
                 dimnames = list(rnames, cnames))
    out
  }

#' @title Confidence Intervals for a 'cond_effect' Class Object
#'
#' @description Return the confidence intervals of estimates
#'              conditional effect in the output of
#'              [cond_effect()] or [cond_effect_boot()].
#'
#' @details If bootstrapping is used to form the confidence interval by
#' [cond_effect_boot()],
#' users can request the percentile confidence intervals of
#' the bootstrap estimates. This method does not do the bootstrapping itself.
#'
#' @return
#'  A matrix of the confidence intervals.
#'
#' @param object The output of [cond_effect()] or [cond_effect_boot()].
#' @param parm Ignored by this function. The confidence intervals for all
#'             available levels will be returned.
#' @param level The level of confidence. For the confidence intervals returned
#'             by [lm()], default is .95, i.e., 95%. For the bootstrap
#'             percentile confidence
#'             intervals, default is the level used in calling
#'             [cond_effect_boot()].
#' @param type The type of the confidence intervals. If est to `"lm"`,
#'            returns the confidence interval given by the [confint()] method
#'            of [lm()].
#'            If set to `"boot"`, the bootstrap percentile confidence intervals
#'            are returned.
#'             Default is `"boot"` if bootstrap estimates are stored in
#'             `object`, and `"lm"` if bootstrap estimates are not stored.
#' @param ...  Additional arguments. Ignored.
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#'
#' @examples
#'
#' # Load a sample data set
#'
#' dat <- test_x_1_w_1_v_1_cat1_n_500
#'
#' # Do a moderated regression by lm
#' lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
#' summary(lm_raw)
#' out <- cond_effect(lm_raw, x = iv, w = mod)
#' print(out, t_ci = TRUE)
#' confint(out)
#'
#' lm_std <- std_selected(lm_raw, to_center = ~ iv + mod, to_scale = ~ iv + mod)
#' # Alternative: use to_standardize as a shortcut
#' # lm_std <- std_selected(lm_raw, to_standardize = ~ iv + mod)
#' out <- cond_effect(lm_std, x = iv, w = mod)
#' print(out, t_ci = TRUE)
#' confint(out)
#'
#' # Categorical moderator
#' lm_cat <- lm(dv ~ iv*cat1 + v1, dat)
#' summary(lm_cat)
#' out <- cond_effect(lm_cat, x = iv, w = cat1)
#' print(out, t_ci = TRUE)
#' confint(out)
#'
#' @export


confint.cond_effect <- function(object, parm, level = .95, type, ...) {
    if (!missing(type)) {
        if (!(type %in% c("lm", "boot"))) {
            stop("type must be either lm or boot.")
          }
      } else {
        if (is.null(attr(object, "cond_effect_boot_call"))) {
            type <- "lm"
          } else {
            type <- "boot"
          }
      }
    if (type == "boot") {
        if (is.null(attr(object, "boot_est"))) {
            stop("Bootstrap estimates not available in the object.")
          }
        # if ((level != attr(object, "conf")) && is.null(object$boot_out)) {
        #     stop(paste("level is different form conf in",
        #                "std_selected_boot() but full_output is FALSE."))
        #   }
        if (isTRUE(level == attr(object, "conf"))) {
            out <- attr(object,"boot_ci")
            out_all <- attr(object, "out_all")
            tmp <- stats::confint(out_all[[1]], level = level)
            colnames(out) <- colnames(tmp)
            rownames(out) <- object$Level
            return(out)
          } else {
            boot_est <- attr(object, "boot_est")
            # To be replaced by coef():
            est_org <- as.data.frame(object)[, "x's Effect"]
            boot_tmp <- list(t0 = est_org,
                              t = boot_est,
                              R = attr(object, "nboot"))
            # Adapted from boot
            boot_ci <- sapply(seq_along(est_org), function(x) {
                                  if (all(abs(boot_est[, x] -
                                              mean(boot_est[, x], na.rm = TRUE)) <
                                              1e-8) ||
                                      all(is.na(boot_est[, x]))) {
                                      return(c(NA, NA))
                                    }
                                  boot::boot.ci(boot_tmp,
                                        index = x,
                                        type = "perc",
                                        conf = level)$percent[4:5]
                                })
            out <- t(boot_ci)
            out_all <- attr(object, "out_all")
            tmp <- stats::confint(out_all[[1]], level = level)
            colnames(out) <- colnames(tmp)
            rownames(out) <- object$Level
            return(out)
          }
      }
    if (type == "lm") {
        iv <- attr(object, "x")
        out_all <- attr(object, "out_all")
        ci_all <- lapply(out_all, stats::confint, level = level)
        out <- t(sapply(ci_all, function(y) y[iv, ]))
        rownames(out) <- object$Level
        return(out)
      }
    return(NULL)
  }

#' @title Standardized Moderation Effect and Its Bootstrap CI in 'lavaan'
#'
#' @description Compute the standardized moderation effect in a structural
#'              equation model fitted by [lavaan::lavaan()] or its wrappers and
#'              form the nonparametric bootstrap confidence interval.
#'
#' @details
#' ## Important Notes
#'
#' Starting from Version 0.2.7.5, of [stdmod_lavaan()] adopts an approach
#' to bootstrapping different from that in the previous
#' versions (0.2.7.4 and before),
#' yielding bootstrapping results different from those in
#' previous versions (for reasons explained later).
#'
#' To reproduce results from the older version of this function,
#' set `use_old_version` to `TRUE`.
#'
#' ## How it works
#'
#' [stdmod_lavaan()] accepts a [lavaan::lavaan-class] object, the
#' structural equation model output returned
#' by [lavaan::lavaan()] and its wrappers (e.g, [lavaan::sem()]) and computes
#' the standardized moderation effect using the formula in the appendix of
#' Cheung, Cheung, Lau, Hui, and Vong (2022).
#'
#' The standard deviations of the focal variable (the variable with its effect
#' on the outcome variable being moderated), moderator, and outcome
#' variable (dependent variable) are computed from the implied
#' covariance matrix returned by
#' [lavaan::lavInspect()]. Therefore, models fitted to data sets with missing
#' data (e.g., with `missing = "fiml"`) are also supported.
#'
#' Partial standardization can also be requested. For example, standardization
#' can be requested for only the focal variable and the outcome variable.
#'
#' There are two ways to request nonparametric bootstrap
#' confidence interval. First, the model is fitted with `se = "boot"`
#' or `se = "bootstrap"` in `lavaan`. The stored bootstrap
#' estimates will then be retrieved automatically to compute the
#' standardized moderation effect. This is the most efficient approach
#' if the bootstrap confidence intervals are also needed for
#' other parameters in the model. Bootstrapping needs to be
#' done only once.
#'
#' Second, bootstrap estimates can be generated by [manymome::do_boot()].
#' The output is then supplied through the argument `boot_out`.
#' Bootstrapping also only needs to be done once. This approach
#' is appropriate when bootstrapping confidence intervals are
#' not needed for other model parameters, or another type
#' of confidence interval is needed when fitting the model.
#' Please refer to the help page of [manymome::do_boot()]
#' on how to use this function.
#'
#' In both approaches, the standard deviations are also computed
#' in each bootstrap samples. This ensures that the sampling
#' variability of the standard deviations is also taken into
#' account in computing the bootstrap confidence interval of
#' the standardized moderation effect.
#'
#' ## Note on the differences between the current version (Version 0.2.7.5 or later) and previous versions (0.2.7.4 and before)
#'
#' In older versions, [stdmod_lavaan()] does not allow for
#' partial standardization. Moreover,
#' it uses [boot::boot()] to do the bootstrapping. Even with
#' the same seed, the results from [boot::boot()] are not
#' identical to those of `lavaan` with `se = "boot"`
#' because they differ in the way the indices
#' of resampling are generated. Both approaches are correct,
#' They just use the generated random numbers differently.
#' To have results consistent with those from `lavaan`,
#' the current version of [stdmod_lavaan()] adopts a
#' resampling algorithm identical to that of `lavaan`.
#' Last, in older versions, [stdmod_lavaan()] does
#' bootstrapping every time it is called. This is
#' inefficient.
#'
#' The bootstrapping results in the current version are not
#' identical to those in older versions due to the use
#' of different resampling algorithms, To reproduce
#' previous results, set `use_old_version` to `TRUE`
#'
#' @return
#' A list of class `stdmod_lavaan` with these elements:
#'
#'  - `stdmod`: The standardized moderation effect.
#'
#'  - `ci`: The nonparametric bootstrap confidence interval. `NA` if
#'            confidence interval not requested.
#'
#'  - `boot_out`: The raw output from [boot::boot()]. `NA` if
#'            confidence interval not requested.
#'
#'  - `fit`: The original fit object.
#'
#' @param fit The SEM output by [lavaan::lavaan()] or its wrappers.
#' @param x The name of the focal variable in the model, the variable with
#'          its effect on the outcome variable being moderated.
#' @param y The name of the outcome variable (dependent variable) in the model.
#' @param w The name of the moderator in the model.
#' @param x_w The name of the product term (x * w) in the model. It can be
#'            the variable generated by the colon operator, e.g., `"x:w"`,
#'            which is only in the model and not in the original data set.
#' @param standardized_x If `TRUE`, the default, `x` is standardized when
#' computing the standardized moderation effect.
#' @param standardized_y If `TRUE`, the default, `y` is standardized when
#' computing the standardized moderation effect.
#' @param standardized_w If `TRUE`, the default, `w` is standardized when
#' computing the standardized moderation effect.
#' @param boot_ci Boolean. Whether nonparametric bootstrapping will be
#'                conducted. Default is `FALSE`.
#' @param boot_out If set to the output of [manymome::do_boot()], the stored
#' bootstrap estimates will be retrieved to form the bootstrap confidence
#' interval. If set, bootstrap estimates stored in `fit`, if any, will not
#' be used. Default is `NULL`.
#' @param R (Not used in the current version. Used when `use_old_version`
#' is set to `TRUE`.)
#' The number of nonparametric bootstrapping samples. Default is 100.
#' Set this to at least 2000 in actual use.
#' @param conf The level of confidence. Default is .95, i.e., 95%.
#' @param use_old_version If set to `TRUE`, it will use the bootstrapping
#' method used in 0.2.7.4 or before. Included only for reproducing
#' previous results if necessary. Default is `FALSE`.
#' @param ... (Not used in the current version. Used when `use_old_version`
#' is set to `TRUE`.) Optional arguments to be passed to [boot::boot()]. Parallel
#'            processing can be used by adding the appropriate arguments in
#'            [boot::boot()].
#'
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#'
#' @references
#' Cheung, S. F., Cheung, S.-H., Lau, E. Y. Y., Hui, C. H., & Vong, W. N.
#' (2022) Improving an old way to measure moderation effect in standardized
#' units. *Health Psychology*, *41*(7), 502-505.
#' \doi{10.1037/hea0001188}
#'
#' @examples
#'
#' #Load a test data of 500 cases
#'
#' dat <- test_mod1
#' library(lavaan)
#' mod <-
#' "
#' med ~ iv + mod + iv:mod + cov1
#' dv ~ med + cov2
#' "
#'
#' fit <- sem(mod, dat)
#'
#' # Compute the standardized moderation effect
#' out_noboot <- stdmod_lavaan(fit = fit,
#'                             x = "iv",
#'                             y = "med",
#'                             w = "mod",
#'                             x_w = "iv:mod")
#' out_noboot
#'
#' # Compute the standardized moderation effect and
#' # its percentile confidence interval using
#' # nonparametric bootstrapping
#' # Fit the model with bootstrap confidence intervals
#' # At least 2000 bootstrap samples should be used
#' # in real research. 50 is used here only for
#' # illustration.
#' fit <- sem(mod, dat, se = "boot", bootstrap = 50,
#'            iseed = 89574)
#' out_boot <- stdmod_lavaan(fit = fit,
#'                           x = "iv",
#'                           y = "med",
#'                           w = "mod",
#'                           x_w = "iv:mod",
#'                           boot_ci = TRUE)
#' out_boot
#'
#' @export

stdmod_lavaan <- function(fit,
                          x,
                          y,
                          w,
                          x_w,
                          standardized_x = TRUE,
                          standardized_y = TRUE,
                          standardized_w = TRUE,
                          boot_ci = FALSE,
                          boot_out = NULL,
                          R = 100,
                          conf = 0.95,
                          use_old_version = FALSE,
                          ...) {
    if (use_old_version) {
        out <- (stdmod_lavaan_old(
                  fit = fit,
                  x = x,
                  y = y,
                  w = w,
                  x_w = x_w,
                  boot_ci = boot_ci,
                  R = R,
                  conf = conf,
                  ...
                ))
      } else {
        out <- (stdmod_lavaan2(
                  fit = fit,
                  x = x,
                  y = y,
                  w = w,
                  x_w = x_w,
                  standardized_x = standardized_x,
                  standardized_y = standardized_y,
                  standardized_w = standardized_w,
                  boot_ci = boot_ci,
                  conf = conf,
                  boot_out = boot_out
                ))
      }
    out$call <- match.call()
    out$old_version <- use_old_version
    return(out)
  }

#' @noRd
stdmod_lavaan_old <- function(fit,
                          x,
                          y,
                          w,
                          x_w,
                          boot_ci = FALSE,
                          R = 100,
                          conf = 0.95, ...) {
    if (!requireNamespace("lavaan", quietly = TRUE)) {
        stop(paste("lavaan needs to be installed to run this function."))
      }
    boot_i <- boot_i_gen(fit = fit, x = x,
                                    y = y,
                                    w = w,
                                    x_w = x_w)
    dat_org <- lavaan::lavInspect(fit, "data")
    if (!boot_ci) {
        stdmod <- stdmod_from_fit(fit = fit,
                                  x = x,
                                  y = y,
                                  w = w,
                                  x_w = x_w)
        boot_out <- NA
        stdmod_ci <- NA
      } else {
        boot_out <- boot::boot(dat_org, boot_i, R = R, ...)
        stdmod <- boot_out$t0
        if (any(is.na(boot_out$t))) {
            tmp <- paste0("Fit not safe in at least one bootstrap ",
                          "samples, either with an error or with ",
                          "a warning message. Please check the original ",
                          "fit object, or set 'warn' to FALSE ",
                          "in lavaan if the warning can be ignored.")
            warning(tmp)
            cat(paste0("\nNumber of valid bootstrap estimates: ",
                           sum(!is.na(boot_out$t)), "\n"))
          }
        stdmod_ci <- boot::boot.ci(boot_out,
                                   type = "perc",
                                   conf = conf)$percent[4:5]
        names(stdmod_ci) <- c("lower_bound", "upper_bound")
      }
    out <- list(stdmod = stdmod,
                ci     = stdmod_ci,
                boot_out = boot_out)
    out$call <- match.call()
    out$fit <- fit
    class(out) <- c("stdmod_lavaan", class(out))
    return(out)
  }

#' @noRd

boot_i_gen <- function(fit, x, y, w, x_w) {
  force(fit)
  force(x)
  force(y)
  force(w)
  force(x_w)
  function(dat, i = NULL) {
      if (is.null(i)) {
          dat_i <- dat
        } else {
          dat_i <- dat[i, ]
        }
      fit_test <- tryCatch(fit_i <- lavaan::update(fit,
                                       data = dat_i,
                                       se = "none",
                                       h1 = FALSE,
                                       baseline = FALSE,
                                       test = "standard"
                                       ),
                            error = function(e) e,
                            warning = function(e) e)
      if (inherits(fit_test, "error")) {
          print(fit_test)
          return(NA)
        }
      if (inherits(fit_test, "warning")) {
          # options_old <- options("warn" = -1)
          warning(fit_test)
          # options("warn" = options_old$warn)
          return(NA)
        }
      if (!inherits(fit_i, "lavaan")) {
          warning("Something's wrong. Fit failed.")
          return(NA)
        } else {
          x_w_std_i <- stdmod_from_fit(fit = fit_i,
                                       x = x,
                                       y = y,
                                       w = w,
                                       x_w = x_w)
          return(x_w_std_i)
        }
    }
  }

#' @noRd

stdmod_from_fit <- function(fit, x, y, w, x_w) {
    fit_cov_implied <- lavaan::lavInspect(fit, "implied")
    x_sd <- sqrt(diag(fit_cov_implied$cov)[x])
    w_sd <- sqrt(diag(fit_cov_implied$cov)[w])
    y_sd <- sqrt(diag(fit_cov_implied$cov)[y])
    x_w_i <- lavaan::coef(fit)[paste0(y, "~", x_w)]
    x_w_std_i <- x_w_i * x_sd * w_sd / y_sd
    return(x_w_std_i)
  }

#' @noRd

stdmod_lavaan2 <- function(fit,
                           x = NULL,
                           y = NULL,
                           w = NULL,
                           x_w = NULL,
                           standardized_x = TRUE,
                           standardized_y = TRUE,
                           standardized_w = TRUE,
                           boot_ci = FALSE,
                           conf = 0.95,
                           boot_out = NULL) {
    if (lavaan::lavInspect(fit, "ngroups") > 1) {
        stop("Multiple-group models are not supported.")
      }
    tmp <- try(lavaan::lavInspect(fit, "boot"),
               silent = TRUE)
    if (boot_ci && inherits(tmp, "try-error") &&
        is.null(boot_out)) {
        stop("Bootstrap confidence interval ",
             "requested but either bootstrapping was ",
             "not done when fitting the model or ",
             "boot_out was not set.")
      }
    stdmod0 <- stdmod_from_fit2(fit = fit,
                               x = x,
                               y = y,
                               w = w,
                               x_w = x_w,
                               standardized_x = standardized_x,
                               standardized_y = standardized_y,
                               standardized_w = standardized_w)
    if (boot_ci) {
        if (is.null(boot_out)) {
            boot_out <- manymome::fit2boot_out(fit)
          }
        est_all <- lapply(boot_out, function(x) x$est)
        implied_cov_all <- lapply(boot_out, function(x) x$implied$cov)
        stdmod_all <- mapply(stdmod_from_fit2,
                             est = est_all,
                             implied_cov = implied_cov_all,
                             MoreArgs = list(x = x,
                                             y = y,
                                             w = w,
                                             x_w = x_w,
                                             standardized_x = standardized_x,
                                             standardized_y = standardized_y,
                                             standardized_w = standardized_w))
        boot_tmp <- list(t0 = stdmod0,
                         t = matrix(stdmod_all, ncol = 1),
                         R = length(stdmod_all))
        stdmod_ci <- boot::boot.ci(boot_tmp,
                            type = "perc",
                            conf = conf)$percent[4:5]
        names(stdmod_ci) <- c("lower_bound", "upper_bound")
      } else {
        stdmod_ci <- NA
        boot_tmp <- NA
      }
    out <- list(stdmod = stdmod0,
                ci     = stdmod_ci,
                boot_out = boot_tmp)
    out$call <- match.call()
    out$fit <- fit
    class(out) <- c("stdmod_lavaan", class(out))
    return(out)
  }

#' @noRd

stdmod_from_fit2 <- function(fit,
                             est,
                             implied_cov,
                             x,
                             y,
                             w,
                             x_w,
                             standardized_x = TRUE,
                             standardized_y = TRUE,
                             standardized_w = TRUE,
                             full = FALSE
                             ) {
    if (!missing(fit)) {
        est <- lavaan::parameterEstimates(fit)
        implied_cov <- lavaan::lavInspect(fit, "implied")$cov
      }
    sd_x <- ifelse(standardized_x,
                   sqrt(implied_cov[x, x]),
                   1)
    sd_y <- ifelse(standardized_y,
                   sqrt(implied_cov[y, y]),
                   1)
    sd_w <- ifelse(standardized_w,
                   sqrt(implied_cov[w, w]),
                   1)
    est_x_w <- est[(est$lhs == y) &
                   (est$rhs == x_w) &
                   (est$op == "~"), "est"]
    out <- est_x_w * sd_x * sd_w / sd_y
    if (full) {
        return(c(out, est_x_w, sd_x, sd_w, sd_y))
      } else {
        return(out)
      }
  }

library(testthat)
library(stdmod)

# context("Check standardized moderation effect given a lavaan output")

test_that("stdmod_lavaan", {
  # skip_if(!interactive(),
  #         message = "stdmod_lavaan not tested if not interactive")

  dat <- test_x_1_w_1_v_2_n_500

  transform0 <- function(data, vars) {
      for (x in vars) {
          data[x] <- scale(data[[x]])[, 1]
        }
      data
    }

  # Results based on stdmod
  lm_raw <- lm(dv ~ iv*mod + v1 + v2, dat)
  lm_zall  <- lm(dv ~ iv*mod + v1 + v2, transform0(dat, c("iv", "mod", "dv")))
  stdmod_xyw <- stdmod(lm_raw, x = iv, y = dv, w = mod,
                      x_rescale = TRUE, y_rescale = TRUE, w_rescale = TRUE)
  stdmod_xyw
  coef(lm_zall)["iv:mod"]

  # Results based on stdmod_lavaan

  library(lavaan)
  # Scale down some variables to facilitate optimization in lavaan.
  dat$dv <- (dat$dv - 6500)/1100
  dat$mod <- (dat$mod - 95)/5
  dat$iv <- (dat$iv - 10)
  dat$iv_mod <- dat$iv * dat$mod

  mod <-
  "
  dv ~ iv + mod + mod + v1 + v2 + iv_mod
  "
  # Use likelihood = "wishart" to replicate results in lm
  fit <- sem(mod, dat, likelihood = "wishart")
  coef(fit)
  fit_cov_implied <- lavInspect(fit, "implied")
  iv_sd <- sqrt(diag(fit_cov_implied$cov)["iv"])
  mod_sd <- sqrt(diag(fit_cov_implied$cov)["mod"])
  dv_sd <- sqrt(diag(fit_cov_implied$cov)["dv"])
  iv_mod <- coef(fit)["dv~iv_mod"]
  iv_mod_std <- iv_mod * iv_sd * mod_sd / dv_sd
  iv_mod_std

  out_noboot <- stdmod_lavaan(fit = fit, x = "iv",
                                  y = "dv",
                                  w = "mod",
                                  x_w = "iv_mod",
                                  use_old_version = TRUE)
  out_noboot


  set.seed(6589107)
  stdmod_xyw_boot <- stdmod_boot(lm_raw, x = iv, y = dv, w = mod,
                                    x_rescale = TRUE, y_rescale = TRUE, w_rescale = TRUE,
                                    nboot = 100)
  stdmod_xyw_boot$ci
  set.seed(6589107)
  system.time(out_boot <- stdmod_lavaan(fit = fit, x = "iv",
                                  y = "dv",
                                  w = "mod",
                                  x_w = "iv_mod",
                            boot_ci = TRUE, R = 100,
                            use_old_version = TRUE))
  out_boot$ci
  round(stdmod_xyw_boot$ci, 5)
  round(out_boot$ci, 5)
  expect_equal(
      stdmod_xyw_boot$ci, out_boot$ci,
      ignore_attr = TRUE
    )
  expect_equal(
      stdmod_xyw, iv_mod_std,
      ignore_attr = TRUE
    )
  expect_equal(
      stdmod_xyw, out_boot$stdmod,
      ignore_attr = TRUE
    )
  })

library(testthat)
library(stdmod)

# context("Test methods for stdmod_lavaan")

dat <- test_mod1

# Results based on stdmod_lavaan

library(lavaan)
mod <-
"
med ~ iv + mod + iv:mod + cov1
dv ~ med + cov2
"
# Use likelihood = "wishart" to replicate results in lm
fit <- sem(mod, dat, likelihood = "wishart")
out_noboot <- stdmod_lavaan(fit = fit, x = "iv",
                            y = "med",
                            w = "mod",
                            x_w = "iv:mod",
                            use_old_version = TRUE)
out_noboot$ci
out_noboot$stdmod
coef(out_noboot)

test_that("confint.stdmod_lavaan: No boot ci", {
  expect_warning(
      tmp <- confint(out_noboot)
    )
  expect_true(
      all(is.na(tmp))
    )
  })

test_that("coef.stdmod_lavaan: No boot ci", {
  expect_equal(
      coef(out_noboot),
      out_noboot$stdmod,
      ignore_attr = TRUE
    )
  })


set.seed(6589107)
system.time(out_boot <- stdmod_lavaan(fit = fit,
                                      x = "iv",
                                      y = "med",
                                      w = "mod",
                                      x_w = "iv:mod",
                                      boot_ci = TRUE,
                                      R = 100,
                                      use_old_version = TRUE))
out_boot$ci
confint(out_boot)
out_boot$stdmod
coef(out_boot)

test_that("confint.stdmod_lavaan: Has boot ci", {
  expect_equal(
      as.vector(confint(out_boot)),
      out_boot$ci,
      ignore_attr = TRUE
    )
  })

test_that("coef.stdmod_lavaan: Has boot ci", {
  expect_equal(
      coef(out_boot),
      out_boot$stdmod,
      ignore_attr = TRUE
    )
  })


set.seed(6589107)
system.time(out_boot <- stdmod_lavaan(fit = fit,
                                      x = "iv",
                                      y = "med",
                                      w = "mod",
                                      x_w = "iv:mod",
                                      boot_ci = TRUE,
                                      R = 100,
                                      conf = .90,
                                      use_old_version = TRUE))
out_boot$ci
confint(out_boot, level = .90)

test_that("confint.stdmod_lavaan: Has boot ci, level .90", {
  expect_equal(
      as.vector(confint(out_boot, level = .90)),
      out_boot$ci,
      ignore_attr = TRUE
    )
  })

skip("Disabled")

# Not reliable to hard core failures in convergence because
# lavaan may change the optimization process.

library(testthat)
library(stdmod)

# context("Check stdmod_lavaan when warning occurs")

dat <- test_mod3_miss

# Results based on stdmod_lavaan

library(lavaan)
mod <-
"
med ~ iv + mod + iv:mod + cov1
dv ~ med + cov2
"
# No need to test no boot case.
# Users should know fit the warning message before calling stdmod_lavaan
# suppressWarnings(fit <- sem(mod, dat[-c(485:500), ], missing = "fiml.x"))
# test_that("No boot, warning on run", {
#     expect_warning(
#         out_noboot <- stdmod_lavaan(fit = fit, x = "iv",
#                                     y = "med",
#                                     w = "mod",
#                                     x_w = "iv:mod")
#       )
#   })

fit <- sem(mod, dat, missing = "fiml.x")
test_that("Boot", {
    expect_warning({
        set.seed(6589107)
        system.time(out_boot <- stdmod_lavaan(fit = fit,
                                              x = "iv",
                                              y = "med",
                                              w = "mod",
                                              x_w = "iv:mod",
                                              boot_ci = TRUE,
                                              R = 100,
                                              use_old_version = TRUE))
      })
    expect_equal(
        sum(is.na(out_boot$boot_out$t)),
        9,
        ignore_attr = TRUE
      )
    print_out <- capture.output(print(out_boot))
    expect_equal(
        as.numeric(strsplit(print_out[grepl("valid results",
                                            print_out)],
                                            ":")[[1]][2]),
        91,
        ignore_attr = TRUE
      )
  })

skip_on_cran()
library(testthat)
library(stdmod)

test_that("stdmod_lavaan2", {
  # skip_if(!interactive(),
  #         message = "stdmod_lavaan not tested if not interactive")

  dat <- test_x_1_w_1_v_2_n_500

  transform0 <- function(data, vars) {
      for (x in vars) {
          data[x] <- scale(data[[x]])[, 1]
        }
      data
    }

  # Results based on stdmod
  lm_raw <- lm(dv ~ iv*mod + v1 + v2, dat)
  lm_zall  <- lm(dv ~ iv*mod + v1 + v2, transform0(dat, c("iv", "mod", "dv")))
  stdmod_xyw <- stdmod(lm_raw, x = iv, y = dv, w = mod,
                      x_rescale = TRUE, y_rescale = TRUE, w_rescale = TRUE)
  stdmod_xyw
  coef(lm_zall)["iv:mod"]

  # Results based on stdmod_lavaan

  library(lavaan)
  # Scale down some variables to facilitate optimization in lavaan.
  dat$dv <- (dat$dv - 6500)/1100
  dat$mod <- (dat$mod - 95)/5
  dat$iv <- (dat$iv - 10)
  dat$iv_mod <- dat$iv * dat$mod

  mod <-
  "
  dv ~ iv + mod + mod + v1 + v2 + iv_mod
  "
  # Use likelihood = "wishart" to replicate results in lm
  # iseed results are different from set.seed() results
  # Results are not expected to be identical
  # because boot::boot() and lavaan::lavaan()
  # draw samples differently.
  suppressWarnings(fit <- sem(mod, dat, likelihood = "wishart", fixed.x = FALSE,
             se = "boot",
             bootstrap = 150,
             iseed = 82526))
  fit_noboot <- sem(mod, dat, likelihood = "wishart", fixed.x = FALSE)
  fit_boot_out <- manymome::do_boot(fit, R = 150,
                                    seed = 82526,
                                    parallel = FALSE)

  out_noboot <- stdmod_lavaan2(fit = fit, x = "iv",
                                  y = "dv",
                                  w = "mod",
                                  x_w = "iv_mod")
  out_noboot
  set.seed(82526)
  stdmod_xyw_boot <- stdmod_boot(lm_raw, x = iv, y = dv, w = mod,
                                    x_rescale = TRUE, y_rescale = TRUE, w_rescale = TRUE,
                                    nboot = 150)
  stdmod_xyw_boot$ci
  out_boot <- stdmod_lavaan2(fit = fit, x = "iv",
                                  y = "dv",
                                  w = "mod",
                                  x_w = "iv_mod",
                            boot_ci = TRUE)
  out_boot$ci
  out_boot_user <- stdmod_lavaan2(fit = fit_noboot, x = "iv",
                                  y = "dv",
                                  w = "mod",
                                  x_w = "iv_mod",
                            boot_ci = TRUE,
                            boot_out = fit_boot_out)
  out_boot_user$ci
  expect_equal(
      out_boot$ci, out_boot_user$ci,
      ignore_attr = TRUE
    )
  expect_equal(
      round(stdmod_xyw_boot$ci, 2),
      round(out_boot$ci, 2),
      ignore_attr = TRUE
    )
  expect_equal(
      stdmod_xyw, out_boot$stdmod,
      ignore_attr = TRUE
    )
  })

#' @title Summary Method for a 'std_selected' Class Object
#'
#' @description Summarize the results of [std_selected()] or
#'             [std_selected_boot()].
#'
#' @return
#'  An object of class `summary.std_selected`, with
#'  bootstrap confidence intervals added if present in the object.
#'  The object is a list. Its main element `coefficients` is similar to
#'  the
#'  coefficient table in the [summary()] printout of [lm()].
#'  This object is for printing summary information of the results
#'  from [std_selected()] or [std_selected_boot()].
#'
#' @param object The output of [std_selected()] or [std_selected_boot()].
#' @param ...  Additional arguments. Ignored by this function.
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#' @examples
#'
#' # Load a sample data set
#'
#' dat <- test_x_1_w_1_v_1_cat1_n_500
#'
#' # Do a moderated regression by lm
#' lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
#' summary(lm_raw)
#'
#' # Standardize all variables except for categorical variables.
#' # Interaction terms are formed after standardization.
#' lm_std <- std_selected(lm_raw, to_scale = ~ .,
#'                                to_center = ~ .)
#' summary(lm_std)
#'
#' # With bootstrapping
#' # nboot = 100 just for illustration. nboot >= 2000 should be used in read
#' # research.
#' lm_std_boot <- std_selected_boot(lm_raw, to_scale = ~ .,
#'                                          to_center = ~ .,
#'                                          nboot = 100)
#' summary(lm_std_boot)
#'
#' @export
#' @importFrom stats anova

summary.std_selected <- function(object, ...) {
    out <- stats::summary.lm(object, ...)
    out$scaled_terms <- object$scaled_terms
    out$centered_terms <- object$centered_terms
    out$scaled_by <- object$scaled_by
    out$centered_by <- object$centered_by
    out$nboot <- object$nboot
    out$std_selected_call <- object$std_selected_call
    out$std_selected_boot_call <- object$std_selected_boot_call
    if (!is.null(object$boot_ci)) {
      out$coefficients <- cbind(out$coefficients[, 1, drop = FALSE],
                                object$boot_ci,
                                out$coefficients[, -1])
      }
    out$highest_order <- tryCatch(highest_order(object),
                                  error = function(e) NA)
    if (!is.na(out$highest_order)) {
        lm_out <- eval(object$lm_out_call,
                       envir = parent.frame())
        lm_call0 <- stats::update(lm_out,
                                  paste0("~ .-", out$highest_order),
                                  evaluate = FALSE)
        lm_out0 <- eval(lm_call0,
                        envir = parent.frame())
        names(lm_out0)
        anova_out <- anova(lm_out0, lm_out)
        rsq_change <- summary(lm_out)$r.squared - summary(lm_out0)$r.squared
        anova_out1 <- cbind(R.sq.change = c(NA, rsq_change), anova_out)
        class(anova_out1) <- class(anova_out)
        attr(anova_out1, "heading") <- attr(anova_out, "heading")
        out$f_highest <- anova_out1
      } else {
        out$f_highest <- NA
      }
    class(out) <- c("summary.std_selected", class(out))
    out
  }

#' @noRd
# Adapted from lmhelprs

highest_order <- function(lm_out) {
    terms_x <- stats::terms(lm_out)
    labels_x <- labels(terms_x)
    order_x <- attr(terms_x, "order")
    order_max <- which.max(order_x)
    order_min <- which.min(order_x)
    max_n <- sum(order_x == max(order_x))
    if ((order_max == order_min) ||
        (max_n != 1)) {
        stop("No unique highest order term.")
      }
    labels_x[order_max]
  }

skip_on_cran()

library(testthat)
library(stdmod)
library(ggplot2)

# Manual check

lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        graph_type = "tumble",
        w_method = "percentile",
        w_percentiles = c(.16, .50, .84))
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        graph_type = "tumble",
        w_method = "percentile",
        w_percentiles = c(.16, .30, .50, .70, .84))
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        w_method = "percentile",
        w_percentiles = c(.16, .30, .50, .40, .84))

#' Similar to sweep, but will skip columns that are not numeric.
#' It sweep only on the columns.
#' @param x The data frame.
#' @param values A named vector with length equal to the number of columns
#'        in data.
#' @param FUN The function to be applied.
#' @noRd

sweep2 <- function(x, values, FUN = "-") {
    FUN <- match.fun(FUN)
    k <- ncol(x)
    n <- nrow(x)
    fct <- function(x, values, FUN) {
        if(is.numeric(x)) {
            out <- FUN(x, values)
          } else {
            out <- x
          }
        out
      }
    out <- mapply(fct, x, values, MoreArgs = list(FUN = FUN),
                  SIMPLIFY = FALSE)
    data.frame(out, stringsAsFactors = FALSE)
  }

library(testthat)
library(stdmod)

# context("Check sweep2")

set.seed(864901374)
n <- 100
dat <- data.frame(x1 = rnorm(n, 10, 5),
                  x2 = sample(c("A", "B", "C"), n, replace = TRUE),
                  x3 = runif(n, 2, 5))
dat2 <- data.frame(x1 = dat$x1 - 10,
                   x2 = dat$x2,
                   x3 = dat$x3/2)

dat2_check1 <- sweep2(dat, c(10, 0, 0), "-")
dat2_check1 <- sweep2(dat2_check1, c( 1, 0, 2), "/")

test_that("Subtract a value then rescale by a factor", {
    expect_identical(
        dat2, dat2_check1
      )
  })

#' @keywords internal
#' @aliases stdmod-package
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
## usethis namespace: end
NULL

#' @title Print a 'stdmod_lavaan' Class Object
#'
#' @description Print the output of [stdmod_lavaan()].
#'
#' @return
#'  `x` is returned invisibly.
#'
#' @param x The output of [stdmod_lavaan()].
#' @param conf If nonparametric bootstrapping has been conducted by
#'              [stdmod_lavaan()],
#'             this is the level of confidence in proportion (.95 denotes 95%),
#'             of the confidence interval. Default is .95.
#' @param nd The number of digits to be printed.
#' @param ...  Optional arguments. Ignored.
#'
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#' @examples
#'
#' # Load a test data of 500 cases
#'
#' dat <- test_mod1
#' library(lavaan)
#'
#' mod <-
#' "
#' med ~ iv + mod + iv:mod + cov1
#' dv ~ med + cov2
#' "
#' fit <- sem(mod, dat)
#' coef(fit)
#'
#' # Compute the standardized moderation effect
#' out_noboot <- stdmod_lavaan(fit = fit,
#'                             x = "iv",
#'                             y = "med",
#'                             w = "mod",
#'                             x_w = "iv:mod")
#' out_noboot
#'
#' # Compute the standardized moderation effect and
#' # its percentile confidence interval based on nonparametric bootstrapping
#' # Fit the model with bootstrap confidence intervals
#' # At least 2000 bootstrap samples should be used
#' # in real research. 50 is used here only for
#' # illustration.
#' fit <- sem(mod, dat, se = "boot", bootstrap = 50,
#'            iseed = 89574)
#' out_boot <- stdmod_lavaan(fit = fit,
#'                           x = "iv",
#'                           y = "med",
#'                           w = "mod",
#'                           x_w = "iv:mod",
#'                           boot_ci = TRUE)
#'
#' out_boot
#'
#'
#' @export

print.stdmod_lavaan <- function(x,
                                conf = .95,
                                nd = 3,
                                ...) {
    if (!requireNamespace("lavaan", quietly = TRUE)) {
        stop(paste("lavaan needs to be installed to run this function."))
      }
    if (isTRUE(x$call$boot_ci)) {
        has_bootci <- TRUE
      } else {
        has_bootci <- FALSE
      }
    cat("\nCall:\n")
    print(x$call)
    roles <- c("Focal Variable", "Moderator", "Outcome Variable", "Product Term")
    c_x <- x$call$x
    c_w <- x$call$w
    c_y <- x$call$y
    c_x_w <- x$call$x_w
    vars <- c(c_x, c_w, c_y, c_x_w)
    tmpdf <- data.frame(Variable = vars)
    rownames(tmpdf) <- roles
    cat("\n")
    print(tmpdf)
    fit <- x$fit
    ptable <- lavaan::parameterEstimates(fit, level = conf)
    mod_org <- ptable[(ptable$lhs == c_y) &
                      (ptable$op == "~") &
                      (ptable$rhs == c_x_w), ]
    mod_table <- rbind(mod_org, mod_org)
    rownames(mod_table) <- c("Original", "Standardized")
    mod_table[2, ] <- NA
    mod_table[2, 1:3] <- mod_table[1, 1:3]
    mod_table[2, "est"] <- x$stdmod
    if (has_bootci) {
        stdmod_ci_new <- boot::boot.ci(x$boot_out,
                                        type = "perc",
                                        conf = conf)$percent[4:5]
        mod_table[2, c("ci.lower", "ci.upper")] <- stdmod_ci_new
      }
    cat("\n")
    print(mod_table, nd = nd)
    if (has_bootci) {
        nboot_real <- sum(!is.na(x$boot_out$t))
        tmp <- paste0("Confidence interval of standardized ",
                      "moderation effect:\n",
                      "- Level of confidence: ",
                         round(conf, 2) * 100, "%\n",
                      "- Bootstrapping Method: Nonparametric\n",
                      "- Type: Percentile\n",
                      "- Number of bootstrap samples requests: ",
                      x$call$R, "\n",
                      "- Number of bootstrap samples with valid results: ",
                      nboot_real)
        cat("\n", tmp, "\n", sep = "")
      }
    if (has_bootci) {
        if (isTRUE(x$old_version) || is.null(x$old_version)) {
            cat(strwrap(paste0("NOTE: Bootstrapping conducted by the old version method (0.2.7.4 or before). ",
                              "The method in the newer version is recommended.")), sep = "\n")
          } else {
            cat("\n")
            cat(strwrap(paste0("NOTE: Bootstrapping conducted by the method in 0.2.7.5 or later. ",
                              "To use the method in the older versions for reproducing previous results, ",
                              "set 'use_old_version' to 'TRUE'.")), sep = "\n")

          }
      }
    invisible(x)
  }

#' @title Confidence Intervals for a 'std_selected' Class Object
#'
#' @description Return the confidence intervals of estimates
#'              in the output of [std_selected()] or [std_selected_boot()].
#'
#' @details If bootstrapping is used to form the confidence interval by
#' [std_selected_boot()],
#' users can request the percentile confidence intervals of
#' the bootstrap estimates. This method does not do the bootstrapping itself.
#'
#' @return
#'  A matrix of the confidence intervals.
#'
#' @param object The output of [std_selected()] or [std_selected_boot()].
#' @param parm The parameters (coefficients) for which confidence intervals
#'            should be returned.
#'            If missing, the confidence intervals of all parameters will be
#'            returned.
#' @param level The level of confidence. For the confidence intervals returned
#'             by [lm()], default is .95, i.e., 95%. For the bootstrap
#'             percentile confidence
#'             intervals, default is the level used in calling
#'             [std_selected_boot()]. If a level different from that in the
#'             original
#'             call is specified, `full_output` needs to be set in the call
#'             to [std_selected_boot()] such that the original bootstrapping
#'             output
#'             is stored.
#' @param type The type of the confidence intervals. If est to `"lm"`,
#'            returns the confidence interval given by the [confint()] method
#'            of [lm()].
#'            If set to `"boot"`, the bootstrap percentile confidence intervals
#'            are returned.
#'             Default is `"boot"` if bootstrap estimates are stored in
#'             `object`, and `"lm"` if bootstrap estimates are not stored.
#' @param ...  Arguments to be passed to [summary.lm()].
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#'
#' @examples
#'
#' # Load a sample data set
#'
#' dat <- test_x_1_w_1_v_1_cat1_n_500
#'
#' # Do a moderated regression by lm
#' lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
#' summary(lm_raw)
#'
#' # Standardize all variables except for categorical variables.
#' # Interaction terms are formed after standardization.
#' lm_std <- std_selected(lm_raw, to_center = ~ .,
#'                                to_scale = ~ .)
#' # Alternative: use to_standardize as a shortcut
#' # lm_std <- std_selected(lm_raw, to_standardize = ~ .)
#' summary(lm_std)
#'
#' confint(lm_std)
#'
#' # Use to_standardize as a shortcut
#' lm_std2 <- std_selected(lm_raw, to_standardize = ~ .)
#' # The results are the same
#' confint(lm_std)
#' confint(lm_std2)
#' all.equal(confint(lm_std), confint(lm_std2))
#'
#' # With bootstrapping
#' # nboot = 100 just for illustration. nboot >= 2000 should be used in read
#' # research.
#' set.seed(89572)
#' lm_std_boot <- std_selected_boot(lm_raw, to_scale = ~ .,
#'                                          to_center = ~ .,
#'                                          nboot = 100)
#' summary(lm_std_boot)
#'
#' # Bootstrap percentile intervals, default when bootstrap was conduced
#'
#' confint(lm_std_boot)
#'
#' # Force OLS confidence intervals
#'
#' confint(lm_std_boot, type = "lm")
#'
#' # Use to_standardize as a shortcut
#' set.seed(89572)
#' lm_std_boot2 <- std_selected_boot(lm_raw, to_standardize = ~ .,
#'                                           nboot = 100)
#' # The results are the same
#' confint(lm_std_boot)
#' confint(lm_std_boot2)
#' all.equal(confint(lm_std_boot), confint(lm_std_boot2))
#'
#' @export


confint.std_selected <- function(object, parm, level = .95, type, ...) {
    if (!missing(type)) {
        if (!(type %in% c("lm", "boot"))) {
            stop("type must be either lm or boot.")
          }
      } else {
        if (is.null(object$boot_est)) {
            type <- "lm"
          } else {
            type <- "boot"
          }
      }
    if (type == "boot") {
        if (is.null(object$boot_est)) {
            stop("Bootstrap estimates not available in the object.")
          }
        if ((level != object$conf) && is.null(object$boot_out)) {
            stop(paste("level is different form conf in",
                       "std_selected_boot() but full_output is FALSE."))
          }
        if (level == object$conf) {
            out <- object$boot_ci
          } else {
            p <- nrow(object$boot_ci)
            out <- t(sapply(seq_len(p), function(x) {
                        boot::boot.ci(object$boot_out, conf = level,
                                      type = "perc", index = x)$percent[4:5]
                      }))
            rownames(out) <- names(object$boot_out$t0)
          }
        tmp <- object
        class(tmp) <- "lm"
        colnames(out) <- colnames(stats::confint(tmp, level = level))
        if (!missing(parm)) {
            out <- out[rownames(out) %in% parm, , drop = FALSE]
          }
        return(out)
      }
    if (type == "lm") {
        object$boot_est <- NULL
        NextMethod()
      }
  }

plot_df_meansd_w_numeric <- function(output,
                           x,
                           w,
                           x_levels,
                           w_levels,
                           x_levels_labels,
                           w_levels_labels,
                           other_numeric_on = "mean",
                           other_categorical_on = "reference"
                          ) {
    tmp_lvl <- expand.grid(w_level = w_levels, x_level = x_levels)
    tmp_lbl <- expand.grid(w_label = w_levels_labels, x_label = x_levels_labels)
    mf1 <- stats::model.frame(output)
    mf2 <- data.frame(lapply(mf1, sum_col,
                             other_numeric_on = other_numeric_on,
                             other_categorical_on = other_categorical_on))
    mf2 <- do.call(rbind, replicate(nrow(tmp_lvl), mf2, simplify = FALSE))
    mf2 <- data.frame(x_level = as.character(tmp_lbl$x_label),
                      w_level = as.character(tmp_lbl$w_label),
                      mf2)
    mf2[, x] <- tmp_lvl$x_level
    mf2[, w] <- tmp_lvl$w_level
    mf2
  }


plot_df_meansd_w_categorical <- function(output,
                           x,
                           w,
                           x_levels,
                           w_levels,
                           x_levels_labels,
                           w_levels_labels,
                           other_numeric_on = "mean",
                           other_categorical_on = "reference"
                          ) {
    tmp_lvl <- expand.grid(w_level = w_levels, x_level = x_levels)
    tmp_lbl <- expand.grid(w_label = w_levels, x_label = x_levels_labels)
    mf1 <- stats::model.frame(output)
    mf2 <- data.frame(lapply(mf1, sum_col,
                             other_numeric_on = other_numeric_on,
                             other_categorical_on = other_categorical_on))
    mf2 <- do.call(rbind, replicate(nrow(tmp_lvl), mf2, simplify = FALSE))
    mf2 <- data.frame(x_level = as.character(tmp_lbl$x_label),
                      w_level = as.character(tmp_lbl$w_label),
                      mf2)
    mf2[, x] <- tmp_lvl$x_level
    mf2[, w] <- tmp_lvl$w_level
    mf2
  }

gen_levels <- function(x,
                       method, ...) {
    out <- switch(method,
              sd = gen_levels_sd(x = x, ...),
              percentile = gen_levels_percentiles(x = x, ...)
            )
    return(out)
  }

gen_levels_sd <- function(x,
                          from_mean_in_sd = 1,
                          levels = c(-1, 0, 1),
                          ...) {
    x_mean <- mean(x, na.rm = TRUE)
    x_sd <- stats::sd(x, na.rm = TRUE)
    out <-  x_mean + levels * from_mean_in_sd * x_sd
    out
  }

gen_levels_percentiles <- function(x,
                          sd_to_percentiles = NA,
                          sd_levels = c(-1, 0, 1),
                          percentiles = c(.16, .50, .84),
                          ...) {
    if (is.numeric(sd_to_percentiles)) {
        percentiles <- sapply(sd_levels * sd_to_percentiles, stats::pnorm)
      }
    out <- stats::quantile(x, percentiles, na.rm = TRUE)
    out <- unname(out)
    out
  }

sum_col <- function(x,
                    other_numeric_on = c("mean", "median"),
                    other_categorical_on = c("reference", "modal")
  ) {
    if (missing(other_numeric_on)) other_numeric_on <- "mean"
    if (missing(other_categorical_on)) other_categorical_on <- "reference"
    if (is.numeric(x)) {
        out <- switch(other_numeric_on,
                mean = mean(x, na.rm = TRUE),
                median = stats::median(x, na.rm = TRUE))
        return(out)
      }
    if (is.character(x)) {
        xf <- as.factor(x)
      } else {
        xf <- x
      }
    if (is.factor(xf)) {
        out <- switch(other_categorical_on,
                reference = as.ordered(xf)[1],
                modal = names(which(table(xf) == max(table(xf)))))
        return(out)
      }
  }

is_standardized <- function(x) {
    if (!is.numeric(x)) return(FALSE)
    isTRUE(all.equal(stats::sd(x, na.rm = TRUE), 1)) &&
      isTRUE(all.equal(mean(x, na.rm = TRUE), 0))
  }

find_bs <- function(mf, x, w, w_levels) {
    b_all <- sapply(w_levels, find_b, mf = mf, x = x, w = w)
    b_all
  }

find_b <- function(w_i, mf, x, w) {
    mf_i <- mf[mf[, w] == w_i, ]
    b_i <- (mf_i[mf_i$x_level == "High", "predicted"] -
            mf_i[mf_i$x_level == "Low", "predicted"]) /
          (mf_i[mf_i$x_level == "High", x] -
            mf_i[mf_i$x_level == "Low", x])
    b_i
  }

#' @title Print the Summary of a 'std_selected' Class Object
#'
#' @description Print the summary generated by [summary()] on the output
#'  of [std_selected()] or [std_selected_boot()].
#'
#' @return
#'  `x` is returned invisibly.
#'
#' @param x The output of [summary()].
#' @param ...  Arguments to be passed to [summary()].
#'
#' @param est_digits The number of digits
#' after the decimal to be displayed for
#' the coefficient estimates, their
#' standard errors, and bootstrap
#' confidence intervals (if present). Note
#' that the values will be rounded to
#' this number of digits before printing.
#' If all digits at this position are
#' zero for all values, the values may
#' be displayed with fewer digits.
#' Note that the coefficient table is
#' printed by [stats::printCoefmat()].
#' If some numbers are vary large, the
#' number of digits after the decimal
#' may be smaller than `est_digits` due
#' to a limit on the column width.
#' This value also determines the number
#' of digits for displayed R-squared
#' if `default_style` is `FALSE`.
#' Default if 4.
#'
#' @param t_digits The number of digits
#' after the decimal to be displayed
#' for the *t* statistic (in the column
#' `"t value"`). This value also
#' determines the number of digits for
#' the *F* statistic for the R-squared
#' if `default_style` is `FALSE`.
#' Default is 4.
#'
#' @param pvalue_less_than If a *p*-value
#' is less than this value, it will be
#' displayed with `"<(this value)".`
#' For example, if `pvalue_less_than`
#' is .001, the default, *p*-values less
#' than .001 will be displayed as
#' `<.001`. This value also determines
#' the printout of the *p*-value of
#' the *F* statistic if `default_style`
#' is `FALSE`. (This argument does what
#' `eps.Pvalue` does in
#' [stats::printCoefmat()].)
#'
#' @param default_style Logical. If
#' `FALSE`, the default, R-squared
#' and *F* statistic will be displayed
#' in a more readable style. If `TRUE`,
#' then the default style in the
#' printout of the `summary` of
#' [lm()] output will be used.
#'
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#' @examples
#'
#' # Load a sample data set
#'
#' dat <- test_x_1_w_1_v_1_cat1_n_500
#'
#' # Do a moderated regression by lm
#' lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
#'
#' # Standardize all variables except for categorical variables.
#' # Interaction terms are formed after standardization.
#' lm_std <- std_selected(lm_raw, to_scale = ~ .,
#'                                to_center = ~ .)
#' summary(lm_std)
#'
#' # With bootstrapping
#' # nboot = 100 just for illustration. nboot >= 2000 should be used in read
#' # research.
#' lm_std_boot <- std_selected_boot(lm_raw, to_scale = ~ .,
#'                                          to_center = ~ .,
#'                                          nboot = 100)
#' summary(lm_std_boot)
#'
#' @export

print.summary.std_selected <- function(x, ...,
                                       est_digits = 4,
                                       t_digits = 4,
                                       pvalue_less_than = .001,
                                       default_style = FALSE) {
  if (!is.null(x$std_selected_boot_call)) {
      cat("\nCall to std_selected_boot():\n")
      print(x$std_selected_boot_call)
    } else {
      cat("\nCall to std_selected():\n")
      print(x$std_selected_call)
    }
  scaled_or_centered <- any(c(!is.null(x$centered_terms), !is.null(x$scaled_terms)))
  if (!is.null(x$scaled_terms)) {
      scaled <- TRUE
    } else {
      scaled <- TRUE
    }
  opt_width <- 0.9 * getOption("width")
  cat("\n")
  tmp <- character(0)
  if (scaled_or_centered) {
      tmp <- c(tmp,
               strwrap("Selected variable(s) are centered by mean and/or scaled by SD"))
      if (!is.null(x$centered_terms)) {
          tmp <- c(tmp,
                   strwrap(paste(c("- Variable(s) centered:", x$centered_terms),
                                   collapse = " "),
                           exdent = 2))
        }
      if (!is.null(x$scaled_terms)) {
          tmp <- c(tmp,
                   strwrap(paste(c("- Variable(s) scaled:", x$scaled_terms),
                                   collapse = " "),
                           exdent = 2))
        }
    } else {
      tmp <- c(tmp,
               strwrap("No variables are centered by mean or scaled by SD by std_selected()."))
    }
  cat(tmp, sep = "\n")
  cat("\n")
  dat_sc <- format_dat_sc(x)
  print(dat_sc)
  tmp <- character(0)
  tmp <- c(tmp, "Note:")
  tmp <- c(tmp,
           strwrap("- Categorical variables will not be centered or scaled even if requested.",
                    exdent = 2))
  if (!is.null(x$nboot)) {
      tmp <- c(tmp,
               strwrap("- Nonparametric bootstrapping 95% confidence intervals computed.",
                       exdent = 2))
      tmp <- c(tmp,
               strwrap(paste0("- The number of bootstrap samples is ", x$nboot, "."),
                       exdent = 2))
    }
  cat("\n")
  cat(tmp, sep = "\n")
  x_rsq <- x$r.squared
  x_rsq_adj <- x$adj.r.squared
  x_fstatistic <- x$fstatistic
  x$coefficients[, "Estimate"] <- round(x$coefficients[, "Estimate"], est_digits)
  x$coefficients[, "Std. Error"] <- round(x$coefficients[, "Std. Error"], est_digits)
  if (!is.null(x$nboot)) {
      x$coefficients[, "CI Lower"] <- round(x$coefficients[, "CI Lower"], est_digits)
      x$coefficients[, "CI Upper"] <- round(x$coefficients[, "CI Upper"], est_digits)
    }
  x$coefficients[, "t value"] <- round(x$coefficients[, "t value"], t_digits)
  if (!default_style) {
      x$fstatistic <- NULL
    }
  NextMethod(eps.Pvalue = pvalue_less_than,
             dig.tst = t_digits)
  if (!default_style) {
      cat(format_rsq(rsq = x_rsq,
                       rsq_adj = x_rsq_adj,
                       digits = est_digits), sep = "\n")
      print_fstatistic(x_fstatistic,
                       f_digits = t_digits,
                       p_digits = ceiling(-log10(pvalue_less_than)))
      cat("\n")
    }
  if (!is.na(x$highest_order) && !identical(x$f_highest, NA)) {
      rsq_highest <- formatC(x$f_highest[2, "R.sq.change"],
                             digits = est_digits,
                             format = "f")
      cat("= Test the highest order term =",
          paste0("The highest order term             : ", x$highest_order),
          paste0("R-squared increase adding this term: ", rsq_highest),
          sep = "\n")
      print_fstatistic_change(x$f_highest,
                              f_digits = t_digits,
                              p_digits = ceiling(-log10(pvalue_less_than)))
      cat("\n")
    }
  tmp <- character(0)
  if (scaled_or_centered) {
      tmp1 <- paste("- Estimates and their statistics are based on the data after",
                     "mean-centering, scaling, or standardization.", collapse = " ")
      tmp <- c(tmp,
               strwrap(tmp1, exdent = 2))
    }
  if (scaled && is.null(x$nboot)) {
      tmp1 <- paste("- One or more variables are scaled by SD or",
                     "standardized. OLS standard errors and",
                     "confidence intervals may be biased for their",
                     "coefficients.",
                     "Please use `std_selected_boot()`.", collapse = " ")
      tmp <- c(tmp,
               strwrap(tmp1, exdent = 2))
    }
  if (!is.null(x$nboot)) {
      tmp <- c(tmp,
               strwrap("- [CI Lower, CI Upper] are bootstrap percentile confidence intervals.",
                       exdent = 2))
      tmp <- c(tmp,
               strwrap("- Std. Error are not bootstrap SEs.", exdent = 2))
    }
  if (length(tmp) > 0) {
      cat("Note:\n")
      cat(tmp, sep = "\n")
      cat("\n")
    }
  invisible(x)
}

format_dat_sc <- function(x) {
  dat_sc <- data.frame(centered_by = x$centered_by,
                      scaled_by   = x$scaled_by)
  nonnumeric <- attr(stats::terms(x), "dataClasses") != "numeric"
  dat_sc[nonnumeric, ] <- NA
  centered <- dat_sc$centered_by != 0
  scaled <- dat_sc$scaled_by != 1
  centered[is.na(centered)] <- FALSE
  scaled[is.na(scaled)] <- FALSE
  dat_sc$Note <- ""
  tmpfct <- function(xc, xs) {
      if (xc && xs) {
        return("Standardized (mean = 0, SD = 1)")
      }
      if (xc) return("Centered (mean = 0)")
      if (xs) return("Scaled (SD = 1)")
      return("")
    }
  dat_sc$Note <- format(mapply(tmpfct, centered, scaled))
  dat_sc[is.na(dat_sc$centered_by) &
        is.na(dat_sc$scaled_by), "Note"] <- "Nonnumeric"
  dat_sc$Note <- format(dat_sc$Note)
  dat_sc
}

#' @noRd

print_fstatistic <- function(fstatistic,
                             f_digits = 4,
                             p_digits = 3) {
     f <- fstatistic["value"]
     df1 <- fstatistic["numdf"]
     df2 <- fstatistic["dendf"]
     f_txt <- paste0("F(",
                     df1, ", ", df2, ") = ",
                     round(f, f_digits))
     p <- stats::pf(f, df1, df2, lower.tail = FALSE)
     p_txt <- format_pvalue(p,
                            eps = 10^(-p_digits))
     if (!grepl("^<", p_txt)) {
        p_txt <- paste0("= ", p_txt)
       }
     cat("ANOVA test of R-squared  : ",
         f_txt, ", p ", p_txt, "\n", sep = "")
  }

#' @noRd

print_fstatistic_change <- function(fstatistic,
                                    f_digits = 4,
                                    p_digits = 3) {
     f <- fstatistic[2, "F"]
     df1 <- fstatistic[2, "Df"]
     df2 <- fstatistic[2, "Res.Df"]
     f_txt <- paste0("F(",
                     df1, ", ", df2, ") = ",
                     round(f, f_digits))
     p <- fstatistic[2, "Pr(>F)"]
     p_txt <- format_pvalue(p,
                            eps = 10^(-p_digits))
     if (!grepl("^<", p_txt)) {
        p_txt <- paste0("= ", p_txt)
       }
     cat("F test of R-squared increase       : ",
         f_txt, ", p ", p_txt, "\n", sep = "")
  }

#' @noRd

format_pvalue <- function(p,
                          eps = 1e-3) {
    p_digits <- ceiling(-log10(eps))
    if (p < eps) {
        return(paste0("< ",
               formatC(eps,
                       digits = p_digits,
                       format = "f")))
      } else {
        return(formatC(p,
                       digits = p_digits,
                       format = "f"))
      }
  }

#' @noRd

format_rsq <- function(rsq, rsq_adj,
                       digits = 4) {
    x1 <- c("R-squared",
            "Adjusted R-squared")
    x2 <- formatC(c(rsq, rsq_adj),
                  digits = digits,
                  format = "f")
    x1max <- max(nchar(x1))
    i <- which(nchar(x1) != x1max)
    x1[i] <- paste0(x1[i],
                    paste0(rep(" ", x1max - nchar(x1[1])),
                           collapse = ""))
    paste0(x1, "       : ", x2)
  }

#' @title Standardize Variables in a Regression Model
#'
#' @description Standardize, mean center, or scale by standard deviation
#'              selected variables in a regression model and refit the model
#'
#' @details
#' [std_selected()] was originally developed to compute the standardized
#' moderation effect and the standardized coefficients for other predictors
#' given an [lm()] output (Cheung, Cheung, Lau, Hui, & Vong, 2022).
#' It has been extended such that users can specify
#' which variables in a regression model are to be mean-centered and/or
#' rescaled by
#' their standard deviations. If the model has one or more interaction terms,
#' they will be formed after the transformation, yielding the correct
#' standardized solution for a moderated regression model. Moreover,
#' categorical predictors will be automatically skipped in mean-centering
#' and rescaling.
#'
#' Standardization is conducted when a variable is mean-centered and then
#' rescaled by its standard deviation. Therefore, if the goal is to get the
#' standardized solution of a moderated regression, users
#' just instruct the function to standardize all non-categorical variables
#' in the regression model.
#'
#' @return
#' The updated [lm()] output, with the class `std_selected` added. It will be
#' treated as a usual [lm()] object by most functions. These are the major
#' additional element in the list:
#'
#'  - `scaled_terms`: If not `NULL`, a character vector of the variables scaled.
#'
#'  - `centered_terms`: If not `NULL`, a character vector of the variables mean-centered.
#'
#'  - `scaled_by`: A numeric vector of the scaling factors for all the variables in
#'                 the model. The value is 1 for terms not scaled.
#'
#'  - `centered_by`: A numeric vector of the numbers used for centering for all
#'                  the variables in the model. The value is 0 for
#'                   terms not centered.
#'
#'  - `std_selected_call`: The original call.
#'
#'  - `lm_out_call`: The call in `lm_out`.
#'
#' @param lm_out The output from [lm()]
#' @param to_scale The terms to be rescaled by standard deviation,
#'       specified by a formula as in [lm()]. For example, if the terms to be scaled
#'       are `x1` and `x3`, use `~ x1 + x3`. No need to specify the
#'       interaction
#'       term. To scale the outcome variable, list it on the *right hand side*
#'       as a predictor.
#'       Specify only the original variables. If `NULL`, then no terms
#'       will be rescaled by their standard deviations. Variables that are not
#'       numeric will be ignored. Default is `NULL`.
#' @param to_center The terms to be mean centered, specified by a formula
#'        as in [lm()]. For example, if the terms to be centered
#'        is `x1` and `x3`, use `~ x1 + x3`. No need to specify the
#'        interaction term. To center  the outcome variable, list it on the
#'        *right hand side*
#'        as a predictor.
#'        Specify only the original variables. If `NULL`, then no term
#'        will be centered. Default is `NULL`.
#'
#' @param to_standardize The terms to be standardized, specified by a formula
#'        as in [lm()]. For example, if the terms to be standardized
#'        is `x1` and `x3`, use `~ x1 + x3`. No need to specify the
#'        interaction term. To standardize the outcome variable, list it on the
#'        *right hand side*
#'        as a predictor.
#'        Specify only the original variables.
#'        This is a shortcut to `to_center` and `to_scale`. Listing a variable
#'        in `to_standardize` is equivalent to listing this variable in
#'        both `to_center` and `to_scale`.
#'        Default is `NULL`.
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#' @references
#' Cheung, S. F., Cheung, S.-H., Lau, E. Y. Y., Hui, C. H., & Vong, W. N.
#' (2022) Improving an old way to measure moderation effect in standardized
#' units. *Health Psychology*, *41*(7), 502-505.
#' \doi{10.1037/hea0001188}
#'
#' @examples
#'
#' # Load a sample data set
#'
#' dat <- test_x_1_w_1_v_1_cat1_n_500
#' head(dat)
#'
#' # Do a moderated regression by lm
#' lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
#' summary(lm_raw)
#'
#' # Mean center mod only
#' lm_cw <- std_selected(lm_raw, to_center = ~ mod)
#' summary(lm_cw)
#'
#' # Mean center mod and iv
#' lm_cwx <- std_selected(lm_raw, to_center = ~ mod + iv)
#' summary(lm_cwx)
#'
#' # Standardize both mod and iv
#' lm_stdwx <- std_selected(lm_raw, to_scale = ~ mod + iv,
#'                                to_center = ~ mod + iv)
#' summary(lm_stdwx)
#'
#' # Standardize all variables except for categorical variables.
#' # Interaction terms are formed after standardization.
#' lm_std <- std_selected(lm_raw, to_scale = ~ .,
#'                                to_center = ~ .)
#' summary(lm_std)
#'
#' # Use to_standardize as a shortcut
#' lm_stdwx2 <- std_selected(lm_raw, to_standardize = ~ mod + iv)
#' # The results are the same
#' coef(lm_stdwx)
#' coef(lm_stdwx2)
#' all.equal(coef(lm_stdwx), coef(lm_stdwx2))
#'
#'
#' @export
#' @describeIn std_selected The base function to center or
#'             scale selected variables in a regression model
#' @order 1

std_selected <- function(lm_out,
                         to_scale = NULL,
                         to_center = NULL,
                         to_standardize = NULL) {
    if (missing(lm_out)) {
        stop("The arguments lm_out cannot be empty.")
      }

    # Collect the terms.
    # Get the data frame.
    # Do the scaling and centering.
    # Update the results.
    # Return the results.

    # Get the data frame

    dat <- lm_out$model
    k <- ncol(dat)

    # Collect the terms

    if (is.null(to_scale)) {
#        scale_terms <- colnames(lm_out$model)
        scale_terms <- NULL
      } else {
        scale_f <- stats::as.formula(to_scale)
        scale_terms  <- attr(stats::terms(scale_f, data = dat),  "term.labels")
      }

    if (is.null(to_center)) {
#        center_terms <- colnames(lm_out$model)
        center_terms <- NULL
      } else {
        center_f <- stats::as.formula(to_center)
        center_terms <- attr(stats::terms(center_f, data = dat), "term.labels")
      }

    if (is.null(to_standardize)) {
      } else {
        std_f <- stats::as.formula(to_standardize)
        std_terms <- attr(stats::terms(std_f, data = dat), "term.labels")
        scale_terms <- unique(c(scale_terms, std_terms))
        center_terms <- unique(c(center_terms, std_terms))
      }


    # Check if the terms are valid

    varnames <- colnames(dat)

    if (!all(scale_terms %in% varnames)) {
        stop("Not all terms in to_scale is in the data frame.")
      }

    if (!all(center_terms %in% varnames)) {
        stop("Not all terms in to_center is in the data frame.")
      }

    # Prepare for Centering

#    var_means <- colMeans(dat)
    var_means <- suppressWarnings(sapply(dat, mean))

    var_a <- rep(0, k)
    names(var_a) <- colnames(dat)

    var_a[center_terms] <- var_means[center_terms]

    # Prepare for Scaling

#    var_sds <- apply(dat, 2, sd)
    var_sds <- suppressWarnings(sapply(dat, sd2))

    var_b <- rep(1, k)
    names(var_b) <- colnames(dat)

    var_b[scale_terms] <- var_sds[scale_terms]

    # Do centering and scaling

    # dat_mod <- sweep(dat,     2, var_a, FUN = "-")
    # dat_mod <- sweep(dat_mod, 2, var_b, FUN = "/")

    dat_mod <- sweep2(dat, var_a, FUN = "-")
    dat_mod <- sweep2(dat_mod, var_b, FUN = "/")

    # Do the regression

    lm_out_mod <- stats::update(lm_out, data = dat_mod)

    class(lm_out_mod) <- c("std_selected", class(lm_out))

    lm_out_mod$scaled_terms   <- scale_terms
    lm_out_mod$centered_terms <- center_terms
    lm_out_mod$scaled_by   <- var_b
    lm_out_mod$centered_by <- var_a
    lm_out_mod$std_selected_call <- match.call()
    lm_out_mod$lm_out_call <- stats::getCall(lm_out)
    lm_out_mod

  }

skip_on_cran()
# False alarm in R-devel

library(testthat)
library(stdmod)
library(ggplot2)

dat <- sleep_emo_con
lm_out <- lm(sleep_duration ~ age + gender + emotional_stability*conscientiousness, dat)
lm_std <- std_selected(lm_out,
                      to_center = ~ .,
                      to_scale = ~ .)
dat_std <- data.frame(scale(dat[, 2:5]), gender = dat$gender)
lm_std_check <- lm(sleep_duration ~ age + gender + emotional_stability*conscientiousness,
                   dat_std)
identical(coef(lm_std), coef(lm_std_check))

p0 <- plotmod(output = lm_out,
        x = emotional_stability,
        w = conscientiousness,
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration",
        graph_type = "tumble")
p0
p0 + coord_fixed(ratio = (1/.3)*sd(dat$emotional_stability)/sd(dat$sleep_duration))

p0_check <- structure(list(x = c(1.7704117227118, 2.195380906087), xend = c(3.27637190301919,
3.69434882364273), y = c(6.27828864766516, 7.0325784780084),
    yend = c(6.90278158628822, 7.05128943568469)), class = "data.frame", row.names = c(NA,
-2L))

test_that("Check plotmod_tumble lm", {
  expect_equal(layer_data(p0, 2)[, colnames(p0_check)],
               p0_check,
               ignore_attr = TRUE)
  })

p0p <- plotmod(output = lm_out,
        x = emotional_stability,
        w = conscientiousness,
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration",
        w_method = "percentile",
        w_percentiles = c(.10, .90),
        graph_type = "tumble")
p0p

p0p_check <- structure(list(x = c(1.76424143533983, 2.22284685095342), xend = c(3.28509189799351,
3.7306415211396), y = c(6.12936696085203, 7.1029925961128), yend = c(6.87917205410562,
7.04687146756592)), class = "data.frame", row.names = c(NA, -2L
))

test_that("Check plotmod_tumble lm, percentiles", {
  expect_equal(layer_data(p0p, 2)[, colnames(p0p_check)],
               p0p_check,
               ignore_attr = TRUE)
  })

p1 <- plotmod(output = lm_std,
        x = emotional_stability,
        w = conscientiousness,
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration",
        graph_type = "tumble")
p1
p1 + coord_fixed(ratio = (1/.3))

p1_check <- structure(list(x = c(-1.23569602969199, -0.678696388002916),
    xend = c(0.738139518022633, 1.2859745237813), y = c(-0.351520649673297,
    0.180858188680334), yend = c(0.0892473573312492, 0.19406440930199
    )), class = "data.frame", row.names = c(NA, -2L))

test_that("Check plotmod_tumble lm, std", {
  expect_equal(layer_data(p1, 2)[, colnames(p1_check)],
               p1_check,
               ignore_attr = TRUE)
  })


p1p <- plotmod(output = lm_std,
        x = emotional_stability,
        w = conscientiousness,
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration",
        w_method = "percentile",
        w_percentiles = c(.10, .90),
        graph_type = "tumble")
p1p

p1p_check <- structure(list(x = c(-1.24378331693482, -0.642697256659458),
    xend = c(0.749568662219976, 1.33354272470621), y = c(-0.456629788683114,
    0.230556573693766), yend = c(0.0725837158345487, 0.190946201100927
    )), class = "data.frame", row.names = c(NA, -2L))

test_that("Check plotmod_tumble lm, std, percentiles", {
  expect_equal(layer_data(p1p, 2)[, colnames(p1p_check)],
               p1p_check,
               ignore_attr = TRUE)
  })


# Categorical Moderator

dat <- sleep_emo_con
lm_out_wcat <- lm(sleep_duration ~ age + gender*emotional_stability + conscientiousness, dat)
summary(lm_out_wcat)
lm_std_wcat <- std_selected(lm_out_wcat,
                      to_center = ~ .,
                      to_scale = ~ .)
summary(lm_std_wcat)
dat_std <- data.frame(scale(dat[, 2:5]), gender = dat$gender)
lm_std_wcat_check <- lm(sleep_duration ~ age + gender*emotional_stability + conscientiousness,
                   dat_std)
identical(coef(lm_std_wcat), coef(lm_std_wcat_check))

p0_wcat <- plotmod(output = lm_out_wcat,
        x = emotional_stability,
        w = gender,
        x_label = "Emotional Stability",
        w_label = "Gender",
        y_label = "Sleep Duration",
        graph_type = "tumble")
p0_wcat

p0_wcat_check <- structure(list(x = c(1.92857619767485, 2.05891627554369), xend = c(3.45889368184323,
3.5575543126916), y = c(6.61654530818674, 6.59512147218901),
    yend = c(7.02064514249012, 6.54487024901611)), class = "data.frame", row.names = c(NA,
-2L))

test_that("Check plotmod_tumble lm, std, percentiles", {
  expect_equal(layer_data(p0_wcat, 2)[, colnames(p0_wcat_check)],
               p0_wcat_check,
               ignore_attr = TRUE)
  })


p1_wcat <- plotmod(output = lm_std_wcat,
        x = emotional_stability,
        w = gender,
        x_label = "Emotional Stability",
        w_label = "Gender",
        y_label = "Sleep Duration",
        graph_type = "tumble")

p1_wcat

p1_wcat_check <- structure(list(x = c(-1.02839263140163, -0.857558181491475),
    xend = c(0.977367606511294, 1.10668036183842), y = c(-0.112778616107886,
    -0.12789959011911), yend = c(0.172435623446994, -0.163366975124769
    )), class = "data.frame", row.names = c(NA, -2L))

test_that("Check plotmod_tumble lm, std, percentiles", {
  expect_equal(layer_data(p1_wcat, 2)[, colnames(p1_wcat_check)],
               p1_wcat_check,
               ignore_attr = TRUE)
  })


# Categorical Moderator: 3 Groups

dat <- sleep_emo_con
set.seed(4807859)
dat$city <- sample(c("Alpha", "Beta", "Gamma"), nrow(dat), replace = TRUE)
lm_out_wcat3 <- lm(sleep_duration ~ age + city*emotional_stability + conscientiousness, dat)
summary(lm_out_wcat3)
lm_std_wcat3 <- std_selected(lm_out_wcat3,
                      to_center = ~ .,
                      to_scale = ~ .)
summary(lm_std_wcat3)
dat_std <- data.frame(scale(dat[, 2:5]), gender = dat$gender, city = dat$city)
lm_std_wcat3_check <- lm(sleep_duration ~ age + city*emotional_stability + conscientiousness,
                   dat_std)
identical(coef(lm_std_wcat3), coef(lm_std_wcat3_check))

p0_wcat3 <- plotmod(output = lm_out_wcat3,
        x = emotional_stability,
        w = "city",
        x_label = "Emotional Stability",
        w_label = "City",
        y_label = "Sleep Duration",
        graph_type = "tumble")
p0_wcat3


p0_wcat_check <- structure(list(x = c(1.96606187963417, 1.89124587472203, 1.99826986566196
), xend = c(3.43643812036583, 3.41858071487335, 3.57179001457756
), y = c(6.94671722231592, 6.50466744365998, 6.40981328362209
), yend = c(7.08575348944406, 6.81942773846245, 6.91994451844113
)), class = "data.frame", row.names = c(NA, -3L))

test_that("Check plotmod_tumble lm, std, percentiles", {
  expect_equal(layer_data(p0_wcat3, 2)[, colnames(p0_wcat_check)],
               p0_wcat_check,
               ignore_attr = TRUE)
  })


p1_wcat3 <- plotmod(output = lm_std_wcat3,
        x = emotional_stability,
        w = "city",
        x_label = "Emotional Stability",
        w_label = "City",
        y_label = "Sleep Duration",
        graph_type = "tumble")
p1_wcat3

p1_wcat3_check <- structure(list(x = c(-0.979260806703231, -1.07732082977996, -0.937046365330035
), xend = c(0.947935496640418, 0.924530109026632, 1.1253388461706
), y = c(0.120257191919775, -0.191742172499917, -0.258690373977058
), yend = c(0.218389187613023, 0.0304160926201823, 0.101360979330034
)), class = "data.frame", row.names = c(NA, -3L))

test_that("Check plotmod_tumble lm, std, percentiles", {
  expect_equal(layer_data(p1_wcat3, 2)[, colnames(p1_wcat3_check)],
               p1_wcat3_check,
               ignore_attr = TRUE)
  })

skip_on_cran()
skip_if_not_installed("visreg")
# False alarm in R-devel

library(testthat)
library(stdmod)
library(ggplot2)

dat <- sleep_emo_con
lm_out <- lm(sleep_duration ~ age + gender + emotional_stability*conscientiousness, dat)
lm_std <- std_selected(lm_out,
                      to_center = ~ .,
                      to_scale = ~ .)
dat_std <- data.frame(scale(dat[, 2:5]), gender = dat$gender)
lm_std_check <- lm(sleep_duration ~ age + gender + emotional_stability*conscientiousness,
                   dat_std)
identical(coef(lm_std), coef(lm_std_check))

p0 <- plotmod(output = lm_out,
        x = emotional_stability,
        w = conscientiousness,
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration")
p0
p0 + coord_fixed(ratio = (1/.3)*sd(dat$emotional_stability)/sd(dat$sleep_duration))

p0_check <- structure(list(x = c(1.95023867728441, 1.95023867728441), xend = c(3.47616132271559,
3.47616132271559), y = c(6.35285945290502, 7.02951847531317),
    yend = c(6.98563044473979, 7.04856589701016)), class = "data.frame", row.names = c(NA,
-2L))

test_that("Check plotmod lm", {
  expect_equal(layer_data(p0, 2)[, colnames(p0_check)],
               p0_check,
               ignore_attr = TRUE)
  })

p0ylim <- layer_scales(p0)$y$range$range
p0xlim <- layer_scales(p0)$x$range$range
wlo <- mean(dat$conscientiousness) - sd(dat$conscientiousness)
whi <- mean(dat$conscientiousness) + sd(dat$conscientiousness)

visreg::visreg(lm_out, "emotional_stability", "conscientiousness",
            breaks = c(wlo, whi), overlay = TRUE,
            partial = FALSE,
            band = FALSE,
            rug = FALSE,
            gg = TRUE) +
            xlim(p0xlim[1], p0xlim[2]) + ylim(p0ylim[1], p0ylim[2]) +
               theme(legend.position = "top",
                     plot.caption = element_text(hjust = .5),
                     plot.title = element_text(hjust = .5),
                     plot.subtitle = element_text(hjust = .5))
p0

p0p <- plotmod(output = lm_out,
        x = emotional_stability,
        w = conscientiousness,
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration",
        w_method = "percentile",
        w_percentiles = c(.10, .90))

p0p_check <- structure(list(x = c(1.95023867728441, 1.95023867728441), xend = c(3.47616132271559,
3.47616132271559), y = c(6.22106675827842, 7.11313925511082),
    yend = c(6.97337252373737, 7.05634339117116)), class = "data.frame", row.names = c(NA,
-2L))

test_that("Check plotmod lm, percentiles", {
  expect_equal(layer_data(p0p, 2)[, colnames(p0p_check)],
               p0p_check,
               ignore_attr = TRUE)
  })

p0pylim <- layer_scales(p0p)$y$range$range
p0pxlim <- layer_scales(p0p)$x$range$range
visreg::visreg(lm_out, "emotional_stability", "conscientiousness",
            overlay = TRUE,
            partial = FALSE,
            band = FALSE,
            rug = FALSE,
            gg = TRUE) +
            xlim(p0pxlim[1], p0pxlim[2]) + ylim(p0pylim[1], p0pylim[2]) +
               theme(legend.position = "top",
                     plot.caption = element_text(hjust = .5),
                     plot.title = element_text(hjust = .5),
                     plot.subtitle = element_text(hjust = .5))
p0p

# library(sjPlot)
# p <- plot_model(lm_out,
#            type = "pred", terms = c("emotional_stability",
#                                     "conscientiousness"),
#            mdrt.values = "meansd",
#            ci.lvl = NA)
# p + xlim(p0xlim[1], p0xlim[2]) + ylim(p0ylim[1], p0ylim[2]) +
#                theme(legend.position = "top",
#                      plot.caption = element_text(hjust = .5),
#                      plot.title = element_text(hjust = .5),
#                      plot.subtitle = element_text(hjust = .5))

# library(interactions)
# interact_plot(model = lm_out,
#               pred = emotional_stability,
#               modx = conscientiousness) +
#               xlim(p0xlim[1], p0xlim[2]) + ylim(p0ylim[1], p0ylim[2]) +
#                theme(legend.position = "top",
#                      plot.caption = element_text(hjust = .5),
#                      plot.title = element_text(hjust = .5),
#                      plot.subtitle = element_text(hjust = .5))

p1 <- plotmod(output = lm_std,
        x = emotional_stability,
        w = conscientiousness,
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration")
p1
p1 + coord_fixed(ratio = (1/.3))

p1_check <- structure(list(x = c(-1, -1), xend = c(1, 1), y = c(-0.298888468949575,
0.178698434412524), yend = c(0.147722199821573, 0.192142131820008
)), class = "data.frame", row.names = c(NA, -2L))

test_that("Check plotmod lm, std", {
  expect_equal(layer_data(p1, 2)[, colnames(p1_check)],
               p1_check,
               ignore_attr = TRUE)
  })


p1ylim <- layer_scales(p1)$y$range$range
p1xlim <- layer_scales(p1)$x$range$range

visreg::visreg(lm_std_check, "emotional_stability", "conscientiousness",
            breaks = c(-1, 1), overlay = TRUE,
            partial = FALSE,
            band = FALSE,
            rug = FALSE,
            gg = TRUE) +
            xlim(p1xlim[1], p1xlim[2]) + ylim(p1ylim[1], p1ylim[2]) +
               theme(legend.position = "top",
                     plot.caption = element_text(hjust = .5),
                     plot.title = element_text(hjust = .5),
                     plot.subtitle = element_text(hjust = .5))
p1


p1p <- plotmod(output = lm_std,
        x = emotional_stability,
        w = conscientiousness,
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration",
        w_method = "percentile",
        w_percentiles = c(.10, .90))
p1p

p1p_check <- structure(list(x = c(-1, -1), xend = c(1, 1), y = c(-0.391907940837296,
0.237718100090162), yend = c(0.139070541690466, 0.197631498294497
)), class = "data.frame", row.names = c(NA, -2L))

test_that("Check plotmod lm, std, percentiles", {
  expect_equal(layer_data(p1p, 2)[, colnames(p1p_check)],
               p1p_check,
               ignore_attr = TRUE)
  })



p1pylim <- layer_scales(p1p)$y$range$range
p1pxlim <- layer_scales(p1p)$x$range$range

visreg::visreg(lm_std_check, "emotional_stability", "conscientiousness",
            overlay = TRUE,
            partial = FALSE,
            band = FALSE,
            rug = FALSE,
            gg = TRUE) +
            xlim(p1pxlim[1], p1pxlim[2]) + ylim(p1pylim[1], p1pylim[2]) +
               theme(legend.position = "top",
                     plot.caption = element_text(hjust = .5),
                     plot.title = element_text(hjust = .5),
                     plot.subtitle = element_text(hjust = .5))
p1p


# p <- plot_model(lm_std,
#            type = "pred", terms = c("emotional_stability",
#                                     "conscientiousness"),
#            mdrt.values = "meansd",
#            ci.lvl = NA)
# p + xlim(p1xlim[1], p1xlim[2]) + ylim(p1ylim[1], p1ylim[2]) +
#                theme(legend.position = "top",
#                      plot.caption = element_text(hjust = .5),
#                      plot.title = element_text(hjust = .5),
#                      plot.subtitle = element_text(hjust = .5))

# interact_plot(model = lm_std,
#               pred = emotional_stability,
#               modx = conscientiousness) +
#               xlim(p1xlim[1], p1xlim[2]) + ylim(p1ylim[1], p1ylim[2]) +
#                theme(legend.position = "top",
#                      plot.caption = element_text(hjust = .5),
#                      plot.title = element_text(hjust = .5),
#                      plot.subtitle = element_text(hjust = .5))
# p1

# Categorical Moderator

dat <- sleep_emo_con
lm_out_wcat <- lm(sleep_duration ~ age + gender*emotional_stability + conscientiousness, dat)
summary(lm_out_wcat)
lm_std_wcat <- std_selected(lm_out_wcat,
                      to_center = ~ .,
                      to_scale = ~ .)
summary(lm_std_wcat)
dat_std <- data.frame(scale(dat[, 2:5]), gender = dat$gender)
lm_std_wcat_check <- lm(sleep_duration ~ age + gender*emotional_stability + conscientiousness,
                   dat_std)
identical(coef(lm_std_wcat), coef(lm_std_wcat_check))

p0_wcat <- plotmod(output = lm_out_wcat,
        x = emotional_stability,
        w = gender,
        x_label = "Emotional Stability",
        w_label = "Gender",
        y_label = "Sleep Duration")
p0_wcat

p0_wcat_check <- structure(list(x = c(1.95023867728441, 1.95023867728441), xend = c(3.47616132271559,
3.47616132271559), y = c(6.62226556199772, 6.59876556910144),
    yend = c(7.02520488312402, 6.54759945860682)), class = "data.frame", row.names = c(NA,
-2L))

test_that("Check plotmod lm, std, percentiles", {
  expect_equal(layer_data(p0_wcat, 2)[, colnames(p0_wcat_check)],
               p0_wcat_check,
               ignore_attr = TRUE)
  })



p0wcatylim <- layer_scales(p0_wcat)$y$range$range
p0wcatxlim <- layer_scales(p0_wcat)$x$range$range

p1_wcat <- plotmod(output = lm_std_wcat,
        x = emotional_stability,
        w = gender,
        x_label = "Emotional Stability",
        w_label = "Gender",
        y_label = "Sleep Duration")
p1_wcat

p1wcatylim <- layer_scales(p1_wcat)$y$range$range
p1wcatxlim <- layer_scales(p1_wcat)$x$range$range

visreg::visreg(lm_out_wcat, "emotional_stability", "gender",
            overlay = TRUE,
            partial = FALSE,
            band = FALSE,
            rug = FALSE,
            gg = TRUE) +
            xlim(p0wcatxlim[1], p0wcatxlim[2]) + ylim(p0wcatylim[1], p0wcatylim[2]) +
               theme(legend.position = "top",
                     plot.caption = element_text(hjust = .5),
                     plot.title = element_text(hjust = .5),
                     plot.subtitle = element_text(hjust = .5))
p0_wcat

# library(sjPlot)
# p <- plot_model(lm_out_wcat,
#            type = "pred", terms = c("emotional_stability",
#                                     "gender"),
#            mdrt.values = "meansd",
#            ci.lvl = NA)
# p +  xlim(p0wcatxlim[1], p0wcatxlim[2]) + ylim(p0wcatylim[1], p0wcatylim[2])  +
#                theme(legend.position = "top",
#                      plot.caption = element_text(hjust = .5),
#                      plot.title = element_text(hjust = .5),
#                      plot.subtitle = element_text(hjust = .5))
# p0_wcat

# interact_plot(model = lm_out_wcat,
#               pred = emotional_stability,
#               modx = gender) +
#               xlim(p0wcatxlim[1], p0wcatxlim[2]) + ylim(p0wcatylim[1], p0wcatylim[2]) +
#                theme(legend.position = "top",
#                      plot.caption = element_text(hjust = .5),
#                      plot.title = element_text(hjust = .5),
#                      plot.subtitle = element_text(hjust = .5))
# p0_wcat

visreg::visreg(lm_std_wcat_check, "emotional_stability", "gender",
            overlay = TRUE,
            partial = FALSE,
            band = FALSE,
            rug = FALSE,
            gg = TRUE) +
           xlim(p1wcatxlim[1], p1wcatxlim[2]) + ylim(p1wcatylim[1], p1wcatylim[2]) +
               theme(legend.position = "top",
                     plot.caption = element_text(hjust = .5),
                     plot.title = element_text(hjust = .5),
                     plot.subtitle = element_text(hjust = .5))
p1_wcat

# p <- plot_model(lm_std_wcat,
#            type = "pred", terms = c("emotional_stability",
#                                     "gender"),
#            ci.lvl = NA)
# p + xlim(p1wcatxlim[1], p1wcatxlim[2]) + ylim(p1wcatylim[1], p1wcatylim[2]) +
#                theme(legend.position = "top",
#                      plot.caption = element_text(hjust = .5),
#                      plot.title = element_text(hjust = .5),
#                      plot.subtitle = element_text(hjust = .5))
# p1_wcat

# interact_plot(model = lm_std_wcat,
#               pred = emotional_stability,
#               modx = gender) +
#                xlim(p1wcatxlim[1], p1wcatxlim[2]) + ylim(p1wcatylim[1], p1wcatylim[2])  +
#                theme(legend.position = "top",
#                      plot.caption = element_text(hjust = .5),
#                      plot.title = element_text(hjust = .5),
#                      plot.subtitle = element_text(hjust = .5))
# p1_wcat


# Categorical Moderator: 3 Groups

dat <- sleep_emo_con
set.seed(4807859)
dat$city <- sample(c("Alpha", "Beta", "Gamma"), nrow(dat), replace = TRUE)
lm_out_wcat3 <- lm(sleep_duration ~ age + city*emotional_stability + conscientiousness, dat)
summary(lm_out_wcat3)
lm_std_wcat3 <- std_selected(lm_out_wcat3,
                      to_center = ~ .,
                      to_scale = ~ .)
summary(lm_std_wcat3)
dat_std <- data.frame(scale(dat[, 2:5]), gender = dat$gender, city = dat$city)
lm_std_wcat3_check <- lm(sleep_duration ~ age + city*emotional_stability + conscientiousness,
                   dat_std)
identical(coef(lm_std_wcat3), coef(lm_std_wcat3_check))

p0_wcat3 <- plotmod(output = lm_out_wcat3,
        x = emotional_stability,
        w = "city",
        x_label = "Emotional Stability",
        w_label = "City",
        y_label = "Sleep Duration")
p0_wcat3


p0_wcat_check <- structure(list(x = c(1.95023867728441, 1.95023867728441, 1.95023867728441
), xend = c(3.47616132271559, 3.47616132271559, 3.47616132271559
), y = c(6.94522100731477, 6.51682495584239, 6.39424169437123
), yend = c(7.08950964764363, 6.83129421896357, 6.88894193948001
)), class = "data.frame", row.names = c(NA, -3L))

test_that("Check plotmod lm, std, percentiles", {
  expect_equal(layer_data(p0_wcat3, 2)[, colnames(p0_wcat_check)],
               p0_wcat_check,
               ignore_attr = TRUE)
  })


p0wcat3ylim <- layer_scales(p0_wcat3)$y$range$range
p0wcat3xlim <- layer_scales(p0_wcat3)$x$range$range

p1_wcat3 <- plotmod(output = lm_std_wcat3,
        x = emotional_stability,
        w = "city",
        x_label = "Emotional Stability",
        w_label = "City",
        y_label = "Sleep Duration")
p1_wcat3

p1_wcat3_check <- structure(list(x = c(-1, -1, -1), xend = c(1, 1, 1), y = c(0.119201161237405,
-0.183161383057221, -0.269680823885312), yend = c(0.221040289428956,
0.0387914714870233, 0.0794793148698632)), class = "data.frame", row.names = c(NA,
-3L))

test_that("Check plotmod lm, std, percentiles", {
  expect_equal(layer_data(p1_wcat3, 2)[, colnames(p1_wcat3_check)],
               p1_wcat3_check,
               ignore_attr = TRUE)
  })


p1wcat3ylim <- layer_scales(p1_wcat3)$y$range$range
p1wcat3xlim <- layer_scales(p1_wcat3)$x$range$range

visreg::visreg(lm_out_wcat3, "emotional_stability", "city",
            overlay = TRUE,
            partial = FALSE,
            band = FALSE,
            rug = FALSE,
            gg = TRUE) +
            xlim(p0wcat3xlim[1], p0wcat3xlim[2]) + ylim(p0wcat3ylim[1], p0wcat3ylim[2]) +
               theme(legend.position = "top",
                     plot.caption = element_text(hjust = .5),
                     plot.title = element_text(hjust = .5),
                     plot.subtitle = element_text(hjust = .5))
p0_wcat3

visreg::visreg(lm_std_wcat3_check, "emotional_stability", "city",
            overlay = TRUE,
            partial = FALSE,
            band = FALSE,
            rug = FALSE,
            gg = TRUE) +
           xlim(p1wcat3xlim[1], p1wcat3xlim[2]) + ylim(p1wcat3ylim[1], p1wcat3ylim[2]) +
               theme(legend.position = "top",
                     plot.caption = element_text(hjust = .5),
                     plot.title = element_text(hjust = .5),
                     plot.subtitle = element_text(hjust = .5))
p1_wcat3

skip_on_cran()
skip_if_not_installed("visreg")
# False alarm in R-devel

library(testthat)
library(stdmod)
library(ggplot2)

dat <- sleep_emo_con
lm_out <- lm(sleep_duration ~ age + gender + emotional_stability*conscientiousness, dat)
lm_std <- std_selected(lm_out,
                      to_center = ~ .,
                      to_scale = ~ .)
dat_std <- data.frame(scale(dat[, 2:5]), gender = dat$gender)
lm_std_check <- lm(sleep_duration ~ age + gender + emotional_stability*conscientiousness,
                   dat_std)
identical(coef(lm_std), coef(lm_std_check))

p0 <- plotmod(output = lm_out,
        x = emotional_stability,
        w = conscientiousness,
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration",
        w_values = c(9, -3, 2))
p0

p0_check <- structure(list(x = c(1.95023867728441, 1.95023867728441, 1.95023867728441
), xend = c(3.47616132271559, 3.47616132271559, 3.47616132271559
), y = c(9.84511127666008, 5.94229410301828, 3.154567550417),
    yend = c(7.31044167268714, 6.94744412766431, 6.68816016693373
    )), class = "data.frame", row.names = c(NA, -3L))

test_that("Check plotmod lm", {
  expect_equal(layer_data(p0, 2)[, colnames(p0_check)],
               p0_check,
               ignore_attr = TRUE)
  })

p1 <- plotmod(output = lm_std,
        x = emotional_stability,
        w = conscientiousness,
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration",
        w_values = c(2.5, -1.5, 2.0))
p1

p1_check <- structure(list(x = c(-1, -1, -1), xend = c(1, 1, 1), y = c(0.536888611934098,
0.417491886093573, -0.4182851947901), yend = c(0.225457080818834,
0.214352097819225, 0.136617216821964)), class = "data.frame", row.names = c(NA,
-3L))

test_that("Check plotmod lm, std", {
  expect_equal(layer_data(p1, 2)[, colnames(p1_check)],
               p1_check,
               ignore_attr = TRUE)
  })

#' @title Moderation Effect Plot
#'
#' @description Plot the moderation effect in a regression model
#'
#' @details This function generate a basic `ggplot2` graph
#'          typically found in psychology manuscripts. It tries to
#'          check whether one or more variables are standardized, and
#'          report this in the plot if required.
#'
#' This function only has features for typical plots of moderation effects.
#' It is not intended to be a flexible tool for a fine control on the plots.
#'
#' @return
#'  A `ggplot2` graph. Plotted if not assigned to a name. It can
#'  be further modified like a usual `ggplot2` graph.
#'
#' @param output The output
#'                  of [stats::lm()], [std_selected()], or
#'                  [std_selected_boot()].
#'
#' @param x The name of the focal variable (x-axis) in `output``. It
#'          can be the name of the variable, with or without quotes.
#'          Currently only numeric variables are supported.
#' @param w The name of the moderator in `output`. It
#'          can be the name of the variable, with or without quotes.
#' @param x_label The label for the X-axis. Default is the value of `x`.
#' @param w_label The label for the legend for the lines.
#'                Default is the value of`w`.
#' @param y_label The label for the Y-axis. Default is the
#'                name of the response variable in the model.
#' @param title The title of the graph. If not supplied, it will be
#'               generated from the variable
#'               names or labels (in `x_label`, `y_label`,
#'               and `w_label`). If `""`, no title will be printed.
#'               This can be used when the plot is for manuscript
#'               submission and figures are required to have no
#'               titles.
#' @param digits Number of decimal places to print. Default is 3.
#' @param x_from_mean_in_sd How many SD from mean is used to define
#'                          "low" and
#'                          "high" for the focal variable.
#'                          Default is 1.
#' @param w_from_mean_in_sd How many SD from mean is used to define
#'                          "low" and
#'                          "high" for the moderator. Default is 1.
#'                          Ignored if `w` is categorical.
#' @param w_method How to define "high" and "low" for the moderator levels.
#'                  Default is in terms of the
#'                  standard deviation of the moderator, `"sd"`.
#'                  If equal to
#'                  `"percentile"`, then the percentiles of the moderator in
#'                  the
#'                  dataset are used.
#'                  Ignored if `w` is categorical.
#' @param w_percentiles If `w_method` is `"percentile"`, then this
#'                      argument
#'                      specifies the two percentiles to be used,
#'                      divided by 100.
#'                        It must be a
#'                      vector of two numbers. The default is
#'                      `c(.16, .84)`,
#'                      the 16th and 84th percentiles,
#'                      which corresponds approximately
#'                      to one SD below and above mean for a
#'                      normal distribution, respectively.
#'                      Ignored if `w` is categorical.
#' @param x_method How to define "high" and "low" for the focal
#'                  variable levels.
#'                  Default is in terms of the
#'                  standard deviation of the focal variable, `"sd"`.
#'                  If equal to
#'                  `"percentile"`, then the percentiles of the
#'                  focal variable in the
#'                  dataset is used.
#' @param x_percentiles If `x_method` is `"percentile"`, then this
#'                      argument
#'                      specifies the two percentiles to be used,
#'                      divided by 100.
#'                        It must be a
#'                      vector of two numbers. The default is
#'                      `c(.16, .84)`,
#'                      the 16th and 84th percentiles,
#'                      which corresponds approximately
#'                      to one SD below and above mean for a
#'                      normal distribution, respectively.
#' @param w_sd_to_percentiles If `w_method` is `"percentile"` and
#'                            this argument is
#'                            set to a number, this number will be
#'                            used
#'                            to determine the percentiles to be used.
#'                            The
#'                            lower percentile is the percentile in a
#'                            normal
#'                            distribution
#'                            that is `w_sd_to_percentiles` SD below
#'                            the mean.
#'                            The upper percentile is the percentile in
#'                            a normal
#'                            distribution that is `w_sd_to_percentiles`
#'                            SD
#'                            above the mean. Therefore, if
#'                            `w_sd_to_percentiles` is set to 1, then the
#'                            lower
#'                            and upper percentiles are 16th and 84th,
#'                            respectively. Default is `NA`.
#' @param x_sd_to_percentiles If `x_method` is `"percentile"` and this
#'                            argument is
#'                            set to a number, this number will be used
#'                            to determine the percentiles to be used. The
#'                            lower percentile is the percentile in a
#'                            normal
#'                            distribution
#'                            that is `x_sd_to_percentiles` SD below the mean.
#'                            The upper percentile is the percentile in a normal
#'                            distribution that is `x_sd_to_percentiles` SD
#'                            above the mean. Therefore, if
#'                            `x_sd_to_percentiles` is set to 1, then the lower
#'                            and upper percentiles are 16th and 84th,
#'                            respectively. Default is `NA`.
#'
#' @param w_values The values of `w` to
#' be used. Default is `NULL`. If a
#' numeric vector is supplied, these
#' values will be used to compute the
#' conditional effects. Other arguments
#' on generating levels are ignored.
#' Note that, if `w` has been standardized
#' or centered, these values are for
#' the standardized or centered `w`.
#' The values will always be sorted.
#' This argument is ignored if `w` is
#' categorical.
#'
#' @param note_standardized If `TRUE`, will check whether a variable has SD
#'                          nearly equal to one. If yes, will report this in the
#'                          plot.
#'                          Default is `TRUE`.
#' @param no_title If `TRUE`, title will be suppressed. Default is `FALSE`.
#' @param line_width The width of the lines as used in
#'                   [ggplot2::geom_segment()].
#'                   Default is 1.
#' @param point_size The size of the points as used in [ggplot2::geom_point()].
#'                    Default is 5.
#' @param graph_type If `"default"`, the typical line-graph with equal end-points
#'                   will be plotted. If `"tubmle"`, then the tumble graph
#'                   proposed by Bodner (2016) will be plotted. Default is
#'                   `"default"`.
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#' @references
#'
#' Bodner, T. E. (2016). Tumble graphs: Avoiding misleading end point
#' extrapolation when graphing interactions from a moderated multiple
#' regression analysis. *Journal of Educational and Behavioral
#' Statistics, 41*(6), 593-604. \doi{10.3102/1076998616657080}
#'
#' @examples
#'
#' # Do a moderated regression by lm
#' lm_out <- lm(sleep_duration ~ age + gender + emotional_stability*conscientiousness, sleep_emo_con)
#' plotmod(lm_out,
#'         x = emotional_stability,
#'         w = conscientiousness,
#'         x_label = "Emotional Stability",
#'         w_label = "Conscientiousness",
#'         y_label = "Sleep Duration")
#'
#' # Standardize all variables except for categorical variables
#' # Alternative: use to_standardize as a shortcut
#' # lm_std <- std_selected(lm_out,
#' #                        to_standardize = ~ .)
#' lm_std <- std_selected(lm_out,
#'                        to_scale = ~ .,
#'                        to_center = ~ .)
#' plotmod(lm_std,
#'         x = emotional_stability,
#'         w = conscientiousness,
#'         x_label = "Emotional Stability",
#'         w_label = "Conscientiousness",
#'         y_label = "Sleep Duration")
#'
#' # Tumble Graph
#' plotmod(lm_std,
#'         x = emotional_stability,
#'         w = conscientiousness,
#'         x_label = "Emotional Stability",
#'         w_label = "Conscientiousness",
#'         y_label = "Sleep Duration",
#'         graph_type = "tumble")
#'
#' @importFrom rlang .data
#' @export

plotmod <- function(output, x, w,
                            x_label,
                            w_label,
                            y_label,
                            title,
                            digits = 3,
                            x_from_mean_in_sd = 1,
                            w_from_mean_in_sd = 1,
                            w_method = c("sd", "percentile"),
                            w_percentiles = c(.16, .84),
                            x_method = c("sd", "percentile"),
                            x_percentiles = c(.16, .84),
                            w_sd_to_percentiles = NA,
                            x_sd_to_percentiles= NA,
                            w_values = NULL,
                            note_standardized = TRUE,
                            no_title = FALSE,
                            line_width = 1,
                            point_size = 5,
                            graph_type = c("default", "tumble")
                    ) {
    w_method <- match.arg(w_method)
    x_method <- match.arg(x_method)
    graph_type <- match.arg(graph_type)
    x0 <- deparse(substitute(x))
    if (inherits(tryCatch(x00 <- as.character(x), error = function(e) e),
                 "error")) {
        x <- x0
      } else {
        x <- x00
      }
    w0 <- deparse(substitute(w))
    if (inherits(tryCatch(w00 <- as.character(w), error = function(e) e),
                 "error")) {
        w <- w0
      } else {
        w <- w00
      }
    y <- colnames(stats::model.frame(output))[
                    attr(stats::terms(output), "response")
                  ]
    mf0 <- stats::model.frame(output)
    x_numeric <- is.numeric(mf0[, x])
    w_numeric <- is.numeric(mf0[, w])
    if (!x_numeric) {
        stop("x variable must be a numeric variable.")
      }
    if (w_numeric) {
        if (is.numeric(w_values)) {
            w_levels <- sort(w_values, decreasing = TRUE)
            w_levels_labels <- as.character(w_levels)
            w_method <- "user"
          } else {
            w_levels <- gen_levels(mf0[, w],
                                  method = w_method,
                                  from_mean_in_sd = w_from_mean_in_sd,
                                  levels = c(-1, 1),
                                  sd_levels = c(-1, 1),
                                  sd_to_percentiles = w_sd_to_percentiles,
                                  percentiles = w_percentiles)
            tmp <- length(w_levels)
            if (tmp == 2) {
              w_levels_labels <- c("Low", "High")
            }
            if (tmp == 3) {
              w_levels_labels <- c("Low", "Medium", "High")
            }
            if (tmp > 3) {
              if (w_method == "percentile") {
                  w_levels_labels <- paste0(formatC(w_percentiles * 100,
                                                    digits = 0,
                                                    format = "f"),
                                            "%")
                } else {
                  w_levels_labels <- formatC(w_levels,
                                            digits = 2,
                                            format = "f")
                }
            }
          }
      } else {
        w_lo <- NA
        w_hi <- NA
        w_levels <- levels(as.factor(mf0[, w]))
      }
    if (graph_type == "default") {
        x_levels <- gen_levels(mf0[, x],
                              method = x_method,
                              from_mean_in_sd = x_from_mean_in_sd,
                              levels = c(-1, 1),
                              sd_levels = c(-1, 1),
                              sd_to_percentiles = x_sd_to_percentiles,
                              percentiles = x_percentiles)
      }
    if (graph_type == "tumble") {
        x_subsets <- lapply(w_levels, x_for_w,
                            mf = mf0, x = x, w = w, w_numeric = w_numeric)
        x_levels_list <- lapply(x_subsets,
                                gen_levels,
                                method = x_method,
                                from_mean_in_sd = x_from_mean_in_sd,
                                levels = c(-1, 1),
                                sd_levels = c(-1, 1),
                                sd_to_percentiles = x_sd_to_percentiles,
                                percentiles = x_percentiles)
      }
    if (w_numeric) {
        if (graph_type == "default") {
            mf2 <- plot_df_meansd_w_numeric(output = output,
                                            x = x,
                                            w = w,
                                            x_levels,
                                            w_levels,
                                            x_levels_labels = c("Low", "High"),
                                            w_levels_labels = w_levels_labels,
                                            other_numeric_on = "mean",
                                            other_categorical_on = "reference")
          }
        if (graph_type == "tumble") {
            mf_list <- mapply(plot_df_meansd_w_numeric,
                              x_levels = x_levels_list,
                              w_levels = w_levels,
                              w_levels_labels = w_levels_labels,
                              MoreArgs = list(output = output,
                                              x = x,
                                              w = w,
                                              x_levels_labels = c("Low", "High"),
                                              other_numeric_on = "mean",
                                              other_categorical_on = "reference"),
                              SIMPLIFY = FALSE)
            mf2 <- do.call(rbind, mf_list)
          }
      } else {
        if (graph_type == "default") {
            mf2 <- plot_df_meansd_w_categorical(output = output,
                                            x = x,
                                            w = w,
                                            x_levels,
                                            w_levels,
                                            x_levels_labels = c("Low", "High"),
                                            other_numeric_on = "mean",
                                            other_categorical_on = "reference")
          }
        if (graph_type == "tumble") {
            mf_list <- mapply(plot_df_meansd_w_categorical,
                              x_levels = x_levels_list,
                              w_levels = w_levels,
                              MoreArgs = list(output = output,
                                              x = x,
                                              w = w,
                                              x_levels_labels = c("Low", "High"),
                                              other_numeric_on = "mean",
                                              other_categorical_on = "reference"),
                              SIMPLIFY = FALSE)
            mf2 <- do.call(rbind, mf_list)
          }
      }

    mf2$predicted <- stats::predict(output, mf2)

    if (missing(x_label)) x_label <- x
    if (missing(w_label)) w_label <- w
    if (missing(y_label)) y_label <- y

    x_standardized <- is_standardized(mf0[, x])
    w_standardized <- is_standardized(mf0[, w])
    y_standardized <- is_standardized(mf0[, y])

    if (note_standardized) {
        if (any(c(x_standardized,
                  w_standardized,
                  y_standardized))) {
            tmp <- ifelse(c(x_standardized,
                              w_standardized,
                              y_standardized),
                            c(x_label,
                              w_label,
                              y_label),
                            c(NA, NA, NA))
            cap_std <-
              paste0(stats::na.omit(tmp), collapse = ", ")
            cap_std <- paste0(cap_std, " standardized")
          } else {
            cap_std <- NULL
          }
      }
    if (missing(title)) {
        title <- "Moderation Effect"
      }
    if (utils::packageVersion("ggplot2") >= "3.0.0") {
        ggtmp1 <- "predicted"
        ggtmp2 <- "w_level"
        p <- ggplot2::ggplot() +
              ggplot2::geom_point(ggplot2::aes(x = .data[[x]],
                                               y = .data[[ggtmp1]],
                                               color = .data[[ggtmp2]]),
                                  data = mf2,
                                  size = point_size)
      } else {
        p <- ggplot2::ggplot() +
              ggplot2::geom_point(ggplot2::aes_string(x = x,
                                                      y = "predicted",
                                                      colour = "w_level"),
                                  data = mf2,
                                  size = point_size)
      }
    if (utils::packageVersion("ggplot2") >= "3.4.0") {
        p <- p +
              ggplot2::geom_segment(ggplot2::aes(
                  x = mf2[mf2$x_level == "Low", x],
                  xend = mf2[mf2$x_level == "High", x],
                  y = mf2[mf2$x_level == "Low", "predicted"],
                  yend = mf2[mf2$x_level == "High", "predicted"],
                  colour = mf2[mf2$x_level == "Low", "w_level"]
                ), linewidth = line_width)
      } else {
        p <- p +
              ggplot2::geom_segment(ggplot2::aes(
                  x = mf2[mf2$x_level == "Low", x],
                  xend = mf2[mf2$x_level == "High", x],
                  y = mf2[mf2$x_level == "Low", "predicted"],
                  yend = mf2[mf2$x_level == "High", "predicted"],
                  colour = mf2[mf2$x_level == "Low", "w_level"]
                ), size = line_width)
      }

    b_all <- find_bs(mf = mf2, x = x, w = w, w_levels = w_levels)
    b_format <- paste0("%.", digits, "f")

    if (w_numeric) {
        tmp <- paste0(w_label, " ", w_levels_labels, ": ",
                      x_label, " effect = ",
                      sprintf(b_format, b_all))
        subtxt <- paste(tmp, collapse = "\n")
      } else {
        subtxt <- paste0(w_levels,
                         ": ",
                         x_label,
                         " effect = ",
                         sprintf(b_format, b_all),
                         collapse = "\n")
      }
    if (w_numeric) {
        if (w_method == "percentile") {
            tmp <- paste0(w_levels_labels, ": ",
                          round(w_percentiles * 100, 0),
                          "th percentile")
            cap_txt <- paste0(w_label, " levels: ",
                              paste0(tmp, collapse = "; "))
          }
        if (w_method == "sd") {
            cap_txt <- paste0(w_label, " levels: ",
                              "Low: ", w_from_mean_in_sd,
                              "SD below mean; Hi: ",
                              w_from_mean_in_sd, " SD above mean")
          }
        if (w_method == "user") {
            cap_txt <- NULL
          }
      } else {
        cap_txt <- NULL
      }
    if (note_standardized & !is.null(cap_std)) {
        if (!is.null(cap_txt)) {
            cap_txt <- paste0(cap_txt, "\n", cap_std)
          } else {
            cap_txt <- cap_std
          }
      }
    out <- p +
      ggplot2::labs(title = title,
                    subtitle = subtxt,
                    caption = cap_txt) +
      ggplot2::theme(legend.position = "top",
                     plot.caption = ggplot2::element_text(hjust = .5,
                                                          size = 9),
                     plot.title = ggplot2::element_text(hjust = .5),
                     plot.subtitle = ggplot2::element_text(hjust = .5,
                                                           size = 9)) +
      ggplot2::xlab(x_label) +
      ggplot2::ylab(y_label) +
      ggplot2::guides(colour = ggplot2::guide_legend(title = w_label))
    if (no_title) {
        out <- out + ggplot2::labs(title = NULL)
      }
    out
  }

#' @title Standardized Moderation Effect Given an 'lm' Output
#'
#' @description Compute the standardized moderation effect in a moderated
#'                regression model.
#'
#' @details Two more general functions, [std_selected()] and
#' [std_selected_boot()], have been developed and can do what these functions
#' do and more. Users are recommended to use them instead of [stdmod()] and
#' [stdmod_boot()]. These two functions will not be updated in the near
#' future.
#'
#' Nevertheless, if computing the standardized moderation effect and
#' forming its nonparametric
#' bootstrap interval are all required, then these functions can still
#' be used.
#'
#' [stdmod()] computes the standardized moderation effect given an
#' [lm()] output using the formula from Cheung, Cheung, Lau, Hui, and Vong
#' (2022). Users specify
#' the moderator, the focal variable (the variable with its effect on
#' the outcome variable moderated), the outcome variable (dependent variable)
#' , and the corresponding
#' standardized moderation
#' effect. Users can also select which variable(s) will be standardized.
#'
#' @return
#' [stdmod()] returns a scalar: The standardized moderation effect.
#'
#' @param lm_out The output from [lm()].
#' @param x      The focal variable, that is, the variable with its effect
#'              being moderated. If supplied, its standard deviation will be
#'              used
#'              for rescaling. Also called the independent variable in some
#'              models. Default is `NULL`.
#' @param w      The moderator. If supplied, its standard deviation will be
#'              used
#'              for rescaling. Default is `NULL`.
#' @param y      The outcome variable (dependent variable) . If supplied,
#'              its standard
#'              deviation will be used for rescaling. Default is NULL.
#' @param x_rescale  If `TRUE`, will rescale x by its standard deviation.
#'                   Default is `TRUE`.
#' @param w_rescale  If `TRUE`, will rescale w by its standard deviation.
#'                    Default is `TRUE`.
#' @param y_rescale  If `TRUE`, will rescale y by its standard deviation.
#'                   Default is `TRUE`.
#'
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#' @references
#' Cheung, S. F., Cheung, S.-H., Lau, E. Y. Y., Hui, C. H., & Vong, W. N.
#' (2022) Improving an old way to measure moderation effect in standardized
#' units. *Health Psychology*, *41*(7), 502-505.
#' \doi{10.1037/hea0001188}
#'
#' @examples
#'
#' # Load a test data of 500 cases
#'
#' dat <- test_x_1_w_1_v_2_n_500
#'
#' # Do regression as usual:
#' lm_raw <- lm(dv ~ iv*mod + v1 + v2, dat)
#' summary(lm_raw)
#'
#' # The standard deviations of iv, dv, and mod:
#' sds <- apply(dat, 2, sd)
#' sds
#'
#' # Compute the standardized moderation effect:
#' stdmod_xyw <- stdmod(lm_raw, x = iv, y = dv, w = mod)
#' stdmod_xyw
#' # By default, all three variables will be standardized.
#'
#' # Check against self-computed standardized moderation effect:
#' coef(lm_raw)["iv:mod"] * sds["iv"] * sds["mod"] / sds["dv"]
#'
#' # Standardize only the iv, i.e., do not standardized dv and the moderator:
#' stdmod_x <- stdmod(lm_raw, x = iv, y = dv, w = mod,
#'                    x_rescale = TRUE,  y_rescale = FALSE, w_rescale = FALSE)
#' stdmod_x
#' # Check against self-computed moderation effect with only iv standardized:
#' coef(lm_raw)["iv:mod"] * sds["iv"]
#'
#' @export
#' @describeIn stdmod The base function for computing standardized
#'             moderation effect
#' @order 1

stdmod <- function(lm_out, x = NULL, w = NULL, y = NULL,
                           x_rescale = TRUE,
                           w_rescale = TRUE,
                           y_rescale = TRUE) {
    mycall <- match.call()
    if (any(c(is.null(mycall$x), is.null(mycall$w), is.null(mycall$y)))) {
        stop("The arguments x, w, and y cannot be NULL.")
      }
    x_name <- deparse(substitute(x))
    w_name <- deparse(substitute(w))
    y_name <- deparse(substitute(y))
    b_names <- names(stats::coef(lm_out))
    mod_name1 <- paste0(x_name, ":", w_name)
    mod_name2 <- paste0(w_name, ":", x_name)
    mod_pos <- grepl(mod_name1, b_names) | grepl(mod_name2, b_names)
    if (!any(mod_pos)) {
        stop("The product term not found in the lm output.")
      }
    b_xw <- stats::coef(lm_out)[mod_pos]
    model_names <- colnames(lm_out$model)
    if (x_rescale) {
        if (!(x_name %in% model_names)) {
            stop("x not in the data frame")
          }
        x_dat <- eval(substitute(x),lm_out$model, parent.frame())
        x_sd  <- stats::sd(x_dat)
      } else {
        x_dat <- NULL
        x_sd  <- 1
      }
     if (w_rescale) {
        if (!(w_name %in% model_names)) {stop("w not in the data frame")}
        w_dat <- eval(substitute(w), lm_out$model, parent.frame())
        w_sd  <- stats::sd(w_dat)
      } else {
        w_dat <- NULL
        w_sd  <- 1
      }
    if (y_rescale) {
        if (!(y_name %in% model_names)) {stop("y not in the data frame")}
        y_dat <- eval(substitute(y), lm_out$model, parent.frame())
        y_sd  <- stats::sd(y_dat)
      } else {
        y_dat <- NULL
        y_sd  <- 1
      }
    out <- ((x_sd) * (w_sd) / (y_sd)) * b_xw
    out
  }

library(testthat)
library(stdmod)

# context("Check standardized moderation effect")

dat <- test_x_1_w_1_v_2_n_500

transform0 <- function(data, vars) {
    for (x in vars) {
        data[x] <- scale(data[[x]])[, 1]
      }
    data
  }

lm_raw <- lm(dv ~ iv*mod + v1 + v2, dat)
lm_zx  <- lm(dv ~ iv*mod + v1 + v2, transform0(dat, c("iv")))
lm_zw  <- lm(dv ~ iv*mod + v1 + v2, transform0(dat, c("mod")))
lm_zy  <- lm(dv ~ iv*mod + v1 + v2, transform0(dat, c("dv")))
lm_zxzw  <- lm(dv ~ iv*mod + v1 + v2, transform0(dat, c("iv", "mod")))
lm_zxzy  <- lm(dv ~ iv*mod + v1 + v2, transform0(dat, c("iv", "dv")))
lm_zyzw  <- lm(dv ~ iv*mod + v1 + v2, transform0(dat, c("dv", "mod")))
lm_zall  <- lm(dv ~ iv*mod + v1 + v2, transform0(dat, c("iv", "dv", "mod")))

stdmod_x <- stdmod(lm_raw, x = iv, y = dv, w = mod,
                   x_rescale = TRUE,  y_rescale = FALSE, w_rescale = FALSE)
stdmod_y <- stdmod(lm_raw, x = iv, y = dv, w = mod,
                   x_rescale = FALSE, y_rescale = TRUE,  w_rescale = FALSE)
stdmod_w <- stdmod(lm_raw, x = iv, y = dv, w = mod,
                   x_rescale = FALSE, y_rescale = FALSE, w_rescale = TRUE )
stdmod_xw <- stdmod(lm_raw, x = iv, y = dv, w = mod,
                   x_rescale = TRUE , y_rescale = FALSE, w_rescale = TRUE )
stdmod_yw <- stdmod(lm_raw, x = iv, y = dv, w = mod,
                   x_rescale = FALSE, y_rescale = TRUE , w_rescale = TRUE )
stdmod_xy <- stdmod(lm_raw, x = iv, y = dv, w = mod,
                   x_rescale = TRUE , y_rescale = TRUE,  w_rescale = FALSE)
stdmod_xyw <- stdmod(lm_raw, x = iv, y = dv, w = mod,
                     x_rescale = TRUE, y_rescale = TRUE, w_rescale = TRUE)
stdmod_xyw2 <- stdmod(lm_raw, x = iv, y = dv, w = mod)

test_that("Standardize x", {
    expect_equal(
        stdmod_x, coef(lm_zx)["iv:mod"],
        ignore_attr = TRUE
      )
  })

test_that("Standardize y", {
    expect_equal(
        stdmod_y, coef(lm_zy)["iv:mod"],
        ignore_attr = TRUE
      )
  })

test_that("Standardize w", {
    expect_equal(
        stdmod_w, coef(lm_zw)["iv:mod"],
        ignore_attr = TRUE
      )
  })

test_that("Standardize xy", {
    expect_equal(
        stdmod_xy, coef(lm_zxzy)["iv:mod"],
        ignore_attr = TRUE
      )
  })

test_that("Standardize xw", {
    expect_equal(
        stdmod_xw, coef(lm_zxzw)["iv:mod"],
        ignore_attr = TRUE
      )
  })

test_that("Standardize yw", {
    expect_equal(
        stdmod_yw, coef(lm_zyzw)["iv:mod"],
        ignore_attr = TRUE
      )
  })

test_that("Standardize x, y, and w", {
    expect_equal(
        stdmod_xyw, coef(lm_zall)["iv:mod"],
        ignore_attr = TRUE
      )
  })

test_that("Standardize x, y, and w with default", {
    expect_equal(
        stdmod_xyw2, coef(lm_zall)["iv:mod"],
        ignore_attr = TRUE
      )
  })

x_for_w <- function(w_i, mf, x, w, w_numeric, ...) {
    if (w_numeric) {
        return(x_for_w_numeric(w_i = w_i,
                               mf = mf,
                               x = x,
                               w = w,
                               ...))
      } else {
        return(x_for_w_categorical(w_i = w_i,
                               mf = mf,
                               x = x,
                               w = w,
                               ...))
      }
  }

x_for_w_numeric <- function(w_i, mf, x, w, band = .16) {
    w0 <- mf[, w]
    w_ecdf <- stats::ecdf(w0)
    w_i_p <- w_ecdf(w_i)
    w_i_r <- c(w_i_p - band, w_i_p + band)
    w_i_r[1] <- max(w_i_r[1], 0)
    w_i_r[2] <- min(w_i_r[2], 1)
    w_i_b <- stats::quantile(w0, w_i_r, na.rm = TRUE, names = FALSE)
    x1 <- mf[(w0 >= w_i_b[1]) & (w0 <= w_i_b[2]), x]
    x1
  }

x_for_w_categorical <- function(w_i, mf, x, w) {
    w0 <- mf[, w]
    x1 <- mf[w0 == w_i, x]
    x1
  }

#' @title Print Basic Information of a 'std_selected' Class Object
#'
#' @description Provide information of centering and scaling, along with
#'  basic model information printed by the [print()] method of [lm()].
#'
#' @return
#'  `x` is returned invisibly.
#'
#' @param x The output of [std_selected()] or [std_selected_boot()].
#' @param ...  Arguments to be passed to [print()] method of [lm()].
#'
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#' @examples
#'
#' # Load a sample data set
#'
#' dat <- test_x_1_w_1_v_1_cat1_n_500
#'
#' # Do a moderated regression by lm
#' lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
#' summary(lm_raw)
#'
#' # Standardize all variables except for categorical variables.
#' # Interaction terms are formed after standardization.
#' lm_std <- std_selected(lm_raw, to_scale = ~ .,
#'                                to_center = ~ .)
#' lm_std
#'
#' # With bootstrapping
#' # nboot = 100 just for illustration. nboot >= 2000 should be used in read
#' # research.
#' lm_std_boot <- std_selected_boot(lm_raw, to_scale = ~ .,
#'                                          to_center = ~ .,
#'                                          nboot = 100)
#' lm_std_boot
#'
#' @export

print.std_selected <- function(x, ...) {
    cat("\n- Variable(s) requested to center:", x$centered_terms)
    cat("\n- Variable(s) requested to scale:", x$scaled_terms)
    cat("\nNote: categorical variables will not be centered nor scaled even if requested to do so.")
    if (!is.null(x$nboot)) {
        cat("\n- Nonparametric bootstrap 95% percentile confidence intervals computed.")
      }
    cat("\n")
    NextMethod()
  }

#' @title The 'update' Method for a 'std_selected' Class Object
#'
#' @description This should be used only to update the call to [lm()],
#'          not to the call to [std_selected()] or [std_selected_boot()].
#'
#' @details Although supported, it is not recommended to update an analysis
#'          processed by [std_selected()] or [std_selected_boot()]. It is
#'          recommended to call [lm()] again and pass the output to
#'          [std_selected()] or [std_selected_boot()].
#'
#' @param object The output of the class [std_selected()].
#' @param formula. Changes to the formula.
#' @param ...  Optional arguments to be changed.
#' @param evaluate Whether the call will be evaluated.
#'
#' @return If `evaluate` = `TRUE`, it returns the updated fitted object,
#'         otherwise, the updated call.
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#' @examples
#'
#' # Load a sample data set
#'
#' dat <- test_x_1_w_1_v_1_cat1_n_500
#' head(dat)
#'
#' # Do a moderated regression by lm
#' lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
#' summary(lm_raw)
#'
#' # Standardize all variables except for categorical variables.
#' # Interaction terms are formed after standardization.
#' lm_std <- std_selected(lm_raw, to_scale = ~ .,
#'                                to_center = ~ .)
#' summary(lm_std)
#'
#' # Update the model
#' lm_std2 <- update(lm_std, . ~ . - v1)
#' summary(lm_std2)
#'
#' @export

update.std_selected <- function(object, formula., ..., evaluate = TRUE) {
    # Adapted from update.default in stats
    # if (is.null(call <- stats::getCall(object))) {
    #     stop("need an object with call component")
    #   }
    call <- object$lm_out_call
    extras <- match.call(expand.dots = FALSE)$...
    if (!missing(formula.)) {
        call$formula <- stats::update.formula(stats::formula(object), formula.)
      }
    if(length(extras)) {
        existing <- !is.na(match(names(extras), names(call)))
        ## do these individually to allow NULL to remove entries.
        for (a in names(extras)[existing]) call[[a]] <- extras[[a]]
        if(any(!existing)) {
            call <- c(as.list(call), extras[!existing])
            call <- as.call(call)
          }
      }
    lm_out_call <- call
    if (is.null(extras$data)) {
        lm_out_call$data <- object$lm_out_call$data
      }
    if (!is.null(object$std_selected_boot_call)) {
        new_call <- object$std_selected_boot_call
        new_call$lm_out <- lm_out_call
      } else {
        new_call <- object$std_selected_call
        new_call$lm_out <- lm_out_call
      }
    if (evaluate) {
        # lm_out <- eval(lm_out_call, parent.frame())
        out <- eval(new_call, parent.frame())
        return(out)
      } else {
        return(new_call)
      }
  }

#' @title Sample Dataset: One IV, One Moderator, Two Covariates
#'
#' @description All variables are continuous. For testing.
#'
#' @format A data frame with 500 rows and five variables:
#' \describe{
#'   \item{dv}{Dependent variable, continuous}
#'   \item{iv}{Independent variable, continuous}
#'   \item{mod}{Moderator variable, continuous}
#'   \item{v1}{Covariate, continuous}
#'   \item{v2}{Covariate, continuous}
#' }
"test_x_1_w_1_v_2_n_500"

#' @title Sample Dataset: One IV, One Moderator, Two Covariates
#'
#' @description A covariate
#'              (cat1) is
#' categorical. For testing.
#'
#' @format A data frame with 500 rows and five variables:
#' \describe{
#'   \item{dv}{Dependent variable, continuous}
#'   \item{iv}{Independent variable, continuous}
#'   \item{mod}{Moderator variable, continuous}
#'   \item{v1}{Covariate, continuous}
#'   \item{cat1}{Covariate, categorical (string) with three values: "gp1",
#'               "gp2", and "gp3"}
#' }
#'
"test_x_1_w_1_v_1_cat1_n_500"

#' @title Sample Dataset: Predicting Sleep Duration
#'
#' @description A random subset from a real dataset. For
#' illustration.
#'
#' @format A data frame with 500 rows and six variables:
#' \describe{
#'   \item{case_id}{Case ID, integer}
#'   \item{sleep_duration}{Sleep duration in hours}
#'   \item{conscientiousness}{Conscientiousness score, continuous}
#'   \item{emotional_stability}{Emotional stability score, continuous}
#'   \item{age}{Age in years}
#'   \item{gender}{Gender, string, "female" or "male"}
#' }
#'
"sleep_emo_con"

#' @details
#'
#' [stdmod_boot()] is a wrapper of [stdmod()]. It computes the nonparametric
#' bootstrap confidence interval of the standardized moderation effect, as
#' suggested by Cheung, Cheung, Lau, Hui, and Vong (2022), given
#' the output of [lm()]
#'
#' Percentile interval from [boot::boot.ci()] is returned by this function.
#' If other types of
#' confidence intervals are desired, set `full_output = TRUE` and use
#' [boot::boot.ci()] on the element `boot_out` in the output of this
#' function.
#'
#' @return
#' [stdmod_boot()] returns a list with two elements. The element `ci` is
#' a numeric vector of the bootstrap confidence interval. The element `boot_out`,
#' if not `NA`, is the output of [boot::boot()], which is used to do the
#' bootstrapping.
#'
#' @param lm_out The output from [lm()].
#' @param ...    Parameters to be passed to [stdmod()].
#' @param nboot  The number of bootstrap samples. Default is 100.
#' @param conf The level of confidence for the confidence interval.
#'             Default is .95.
#' @param boot_args A named list of arguments to be passed to [boot::boot()].
#'                  Default is `NULL`.
#' @param full_output Whether the full output from [boot::boot()] is returned.
#'                   Default is `FALSE`.
#'
#'
#' @examples
#'
#' dat <- test_x_1_w_1_v_2_n_500
#' # Do regression as usual:
#' lm_raw <- lm(dv ~ iv*mod + v1 + v2, dat)
#'
#' # Compute the standardized moderation effect.
#' # Form its confidence interval by nonparametric bootstrapping.
#' set.seed(85740917)
#' stdmod_xyw_boot <- stdmod_boot(lm_raw, x = iv, w = mod, y = dv, nboot = 100)
#' # In real analysis, nboot should be at least 2000.
#'
#' # Print the ci
#' stdmod_xyw_boot$ci
#'
#' # Repeat the analysis but keep the results from boot:
#' set.seed(85740917)
#' stdmod_xyw_boot <- stdmod_boot(lm_raw, x = iv, w = mod, y = dv,
#'                                  nboot = 200, full_output = TRUE)
#' # In real analysis, nboot should be at least 2000.
#'
#' # Print the 95% percentile confidence interval
#' stdmod_xyw_boot$ci
#'
#'
#' @export
#' @describeIn stdmod A wrapper of [stdmod()] that computes the nonparametric
#'                     bootstrap confidence interval of the standardized
#'                     moderation effect.
#' @order 2

stdmod_boot <- function(lm_out, ...,
                          nboot = 100, conf = .95,
                          boot_args = NULL, full_output = FALSE) {
    dat <- lm_out$model
    bootfct <- create_boot(lm_out, ...)
    boot_out <- do.call(boot::boot,
                  c(list(data = dat, statistic = bootfct, R = nboot),
                  boot_args))
#    boot_out <- boot::boot(dat, bootfct, R = nboot)
    ci <- boot::boot.ci(boot_out, conf = conf, type = "perc")$percent[4:5]
    out <- list(ci = ci)
    if (full_output) {
        out$boot_out <- boot_out
      } else {
        out$boot_out <- NA
      }
    out
  }
   
create_boot <- function(lm_out, ...) {
  function(d, ind) {
#        print(head(ind))
#        print(head(d[ind, ]))
#        print(colMeans(d[ind, ]))
        stdmod(stats::update(lm_out, data = d[ind, ]), ...)
      }
  }

skip("Test in interactive sessions")
library(testthat)
library(stdmod)

set.seed(64252)
n <- 1000
x1 <- rnorm(n)
x2 <- rnorm(n) / 1000
x3 <- rnorm(n)
y <- .2 * x1 + .3 * x2 + 0 * x3 + rnorm(n, 0, 1)

dat <- data.frame(x1, x2, x3 = x3 * 100, y)

lm_out <- lm(y ~ x1 + x2 + x3, dat)
lm_std <- std_selected(lm_out, to_standardize = ~ y + x1 + x2)
lm_std_boot <- std_selected_boot(lm_out, to_standardize = ~ y + x1 + x2)
lm_out2 <- std_selected(lm_out, to_standardize = ~ y + x1)
lm_std2 <- std_selected(lm_out, to_standardize = ~ y + x1)
lm_std_boot2 <- std_selected_boot(lm_out2, to_standardize = ~ y + x1)

summary(lm_std)
print(summary(lm_std), est_digits = 4, pvalue_less_than = 1e-3)
print(summary(lm_std), est_digits = 2, pvalue_less_than = 1e-3)
print(summary(lm_std), est_digits = 2, pvalue_less_than = 1e-4, t_digits = 2)

summary(lm_std_boot)
print(summary(lm_std_boot), est_digits = 4, pvalue_less_than = 1e-3)
print(summary(lm_std_boot), est_digits = 2, pvalue_less_than = 1e-2, t_digits = 1)
print(summary(lm_std_boot), est_digits = 2, pvalue_less_than = 1e-2, t_digits = 5)

print(summary(lm_std_boot), default_style = TRUE)

summary(lm_std2)
print(summary(lm_std2), est_digits = 4, pvalue_less_than = 1e-3)
print(summary(lm_std2), est_digits = 2, pvalue_less_than = 1e-3)
print(summary(lm_std2), est_digits = 2, pvalue_less_than = 1e-4, t_digits = 2)

summary(lm_std_boot2)
print(summary(lm_std_boot2), est_digits = 4, pvalue_less_than = 1e-3)
print(summary(lm_std_boot2), est_digits = 2, pvalue_less_than = 1e-2, t_digits = 1)
print(summary(lm_std_boot2), est_digits = 2, pvalue_less_than = 1e-2, t_digits = 5)

print(summary(lm_std_boot2), default_style = TRUE)

#' @title Print a 'cond_effect' Class Object
#'
#' @description Print the output of [cond_effect()] or [cond_effect_boot()].
#'
#' @return
#'  `x` is returned invisibility.
#'
#' @param x The output of [cond_effect()] or [cond_effect_boot()].
#' @param nd The number of digits for the variables.
#' @param nd_stat The number of digits for test statistics (e.g., *t*).
#' @param nd_p The number of digits for *p*-values.
#' @param title If `TRUE`, print a title. Default is `TRUE`.
#' @param model If `TRUE`, print the regression model. Default is `TRUE`.
#' @param level_info If `TRUE`, print information for interpreting the
#'                    levels of the moderator, such as the values of
#'                     the levels and distance from the mean. Default
#'                      is `TRUE`.
#' @param standardized If `TRUE` and one or more variables are
#'                     standardized, report it. Default is `TRUE`.`
#' @param boot_info If `TRUE` and bootstrap estimates are in `x`,
#'                  print information about the bootstrapping,
#'                  such as the number of bootstrap samples.
#'                  Default is `TRUE`.
#' @param table_only If `TRUE`, will suppress of other elements except
#'                    for the table of conditional effects. Override
#'                    arguments such as `title`, `model`, and `level_info`.
#' @param t_ci If `TRUE`, will print the confidence intervals based on
#'             t statistics. These confidence intervals should not be
#'             used if some variables are standardized.
#' @param t_ci_level The level of confidence of the confidence intervals
#'                   based on t statistics. Default is .95.
#' @param ...  Additional arguments. Ignored by this function.
#'
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#' @examples
#'
#' # Load a sample data set
#'
#' dat <- test_x_1_w_1_v_1_cat1_n_500
#'
#' # Do a moderated regression by lm
#' lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
#'
#' cond_effect(lm_raw, x = iv, w = mod)
#'
#' lm_std <- std_selected(lm_raw, to_scale = ~ iv + mod,
#'                                to_center = ~ iv + mod)
#'
#' cond_effect(lm_std, x = iv, w = mod)
#'
#' @export

print.cond_effect <- function(x,
                              nd = 3,
                              nd_stat = 3,
                              nd_p = 3,
                              title = TRUE,
                              model = TRUE,
                              level_info = TRUE,
                              standardized = TRUE,
                              boot_info = TRUE,
                              table_only = FALSE,
                              t_ci = FALSE,
                              t_ci_level = .95,
                              ...) {
    xdf <- as.data.frame(x)
    orgcall <- attr(x, "call")
    orgoutput <- attr(x, "output")
    has_bootci <- is.call(attr(x, "cond_effect_boot_call"))
    w_numeric <- is.numeric(x[, 2])
    iv <- attr(x, "x")
    w <- attr(x, "w")
    y <- attr(x, "y")
    cnames <- colnames(x)
    ci_se <- which(cnames == "Std. Error")
    ci_t <- which(cnames == "t value")
    ci_p <- which(cnames == "Pr(>|t|)")
    if (has_bootci) {
        ci_bcilo <- which(cnames == "CI Lower")
        ci_bcihi <- which(cnames == "CI Upper")
      } else {
        ci_bcilo <- ci_bcihi <- NA
      }
    if (title & !table_only) {
        cat("The effects of ",
            iv,
            " on ",
            y,
            ", conditional on ",
            w, ":\n\n", sep = "")
      }
    colnames(xdf)[3] <- paste0(iv, " Effect")
    colnames(xdf)[ci_se] <- "S.E."
    colnames(xdf)[ci_t] <- "t"
    colnames(xdf)[ci_p] <- "p"
    if (w_numeric) {
        xdf[, w] <- formatC(x[, w], nd, format = "f")
      }
    xdf[, 3] <- formatC(x[, 3], nd, format = "f")
    xdf[, ci_se] <- formatC(x[, ci_se], nd, format = "f")
    xdf[, ci_t] <- formatC(x[, ci_t], nd_stat, format = "f")
    xdf[, ci_p] <- formatC(x[, ci_p], nd_p, format = "f")
    if (has_bootci) {
        xdf[, ci_bcilo] <- formatC(x[, ci_bcilo], nd, format = "f")
        xdf[, ci_bcihi] <- formatC(x[, ci_bcihi], nd, format = "f")
      }
    sig <- stats::symnum(x[, ci_p],
                         corr = FALSE,
                         na = FALSE,
                         cutpoints = c(0, 0.001, 0.01, 0.05, 1),
                         symbols = c("***", "**", "*", " "))
    xdf$Sig <- format(sig)

    if (t_ci) {
        ci_x <- stats::confint(x, level = t_ci_level, type = "lm")
        xdf$`CI.Lo(t)` <- formatC(ci_x[, 1], nd, format = "f")
        xdf$`CI.Hi(t)` <- formatC(ci_x[, 2], nd, format = "f")
      }

    print(xdf, row.names = FALSE)

    y_std <- attr(x, "y_standardized")
    x_std <- attr(x, "x_standardized")
    w_std <- attr(x, "w_standardized")

    if (has_bootci) {
        nboot <- attr(x, "nboot")
        conf <- attr(x, "conf")
        if (boot_info && !table_only) {
            tmp <- paste("[CI Lower, CI Upper] shows the ",
                round(conf * 100, 2),
                "% nonparametric bootstrap confidence interval(s)",
                " (based on ", nboot, " bootstrap samples).",
                sep = "")
            cat("\n")
            cat(strwrap(tmp, exdent = 0), sep = "\n")
          }
      }

    if (t_ci) {
        t_ci_level_txt <- paste0(as.character(t_ci_level * 100), "%")
        tmp <- paste("[CI.Lo(t), CI.Hi(t)] shows the ",
                     t_ci_level_txt, " confidence interval(s)",
                     " based on t statistics.",
                     sep = "")
        cat("\n")
        cat(strwrap(tmp, exdent = 0), sep = "\n")
        if (any(y_std, x_std, w_std)) {
            cat("They should not be used when one or more",
                " variables are standardized.",
                sep = "")
          }
      }

    if (model & !table_only) {
        cat("\n\nThe regression model:\n")
        cat("\n\t", deparse(stats::formula(orgoutput)), "\n", sep = "")
      }

    if (w_numeric) {
        w_df <- xdf[, 1:2]
        w_df$`% Below` <- formatC(100 * attr(x, "w_empirical_percentiles"), 2, format = "f")
        w_df$`From Mean (in SD)` <- formatC(attr(x, "w_empirical_z"), 2, format = "f")
        if (level_info & !table_only) {
            cat(paste0("\nInterpreting the levels of ", w, ":\n\n"))
            print(w_df, row.names = FALSE)
            cat("\n")
            cat("- % Below: The percent of cases equal to or less than a level.\n")
            cat(strwrap(paste(
                "- From Mean (in SD): Distance of a level from the mean,",
                "in standard deviation (+ve above, -ve below).", sep = " "),
                exdent = 2), sep = "\n")
          }
      }
    if (any(y_std, x_std, w_std)) {
        tmp <- ifelse(c(y_std, x_std, w_std),
                      c(y, iv, w),
                      c(NA, NA, NA))
        v_std <- paste0(stats::na.omit(tmp), collapse = ", ")
        v_txt <- paste0("- The variable(s) ", v_std, " is/are standardized.")
        if (standardized & !table_only) {
            cat("\nNote:\n\n")
            cat(strwrap(v_txt, exdent = 2), sep = "\n")
            if (y_std & x_std) {
                cat(strwrap(paste(
                    "- The conditional effects are the standardized effects of ",
                    iv,
                    " on ",
                    y, ".", sep = ""), exdent = 2), sep = "\n")
              }
            if (!has_bootci) {
                tmp1 <- paste("- One or more variables are scaled by SD or",
                    "standardized. OLS standard errors and",
                    "confidence intervals may be biased for their",
                    "coefficients.",
                    "Please use `cond_effect_boot()`.", collapse = " ")
                cat(strwrap(tmp1, exdent = 2), sep = "\n")
              }
          }
      }
    invisible(x)
  }

#' @title The 'add1' Method for a 'std_selected' Class Object
#'
#' @description Intercept the [add1()] method and raise an error.
#'
#' @details [add1()] should not be used after the output of [lm()] is
#'  processed by [std_selected()] or [std_selected_boot()].
#'
#' @param object The output of [std_selected()] or [std_selected_boot()].
#' @param ...  Additional arguments. They will be ignored.
#'
#' @return It returns nothing. It is called for its side effect.
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#' @export

add1.std_selected <- function(object, ...) {
    stop(paste0("add1 should be used on the lm() call first and then ",
                "call std_selected() or std_selected_boot() again."))
  }

#' @details
#' [cond_effect_boot()] function is a wrapper of [cond_effect()].
#' It calls [cond_effect()]
#' once for each bootstrap sample, and then computes the nonparametric
#' bootstrap percentile confidence intervals (Cheung, Cheung, Lau, Hui,
#' & Vong, 2022). If the output object is the output of [std_selected()]
#' or [std_selected_boot()], in which mean-centering and/or standardization
#' have been conducted, they will be repeated in each bootstrap sample.
#' Therefore, like [std_selected_boot()], it can be used for form
#' nonparametric bootstrap confidence intervals for standardized
#' effects, though [cond_effect_boot()] does this for the standardized
#' conditional effects.
#'
#' This function ignores bootstrapping done by [std_selected_boot()]. It will
#' do its own bootstrapping.
#'
#' If `do_boot` is `FALSE`, then the object it returns is identical to that
#' by [cond_effect()].
#'
#' This function intentionally does not have an argument for setting the seed
#' for
#' random number. Users are recommended to set the seed, e.g., using
#' [set.seed()]
#' before calling it, to ensure reproducibility.
#'
#' @return
#'
#' [cond_effect_boot()] also returns a data-frame-like object of the
#' conditional effects of the class
#' `cond_effect`, with additional information from the bootstrapping stored
#' in these attributes:
#'
#' - `boot_ci`: A data frame of the bootstrap confidence intervals of the
#'              conditional effects.
#'
#' - `nboot`: The number of bootstrap samples requested.
#'
#' - `conf`: The level of confidence, in proportion.
#'
#' - `boot_est`: A matrix of the bootstrap estimates of the conditional effects.
#'               The number of rows equal to `nboot`, and the number of columns
#'               equal to the number of levels of the moderator.
#'
#' - `cond_effect_boot_call`: The call to `cond_effect_boot()`.
#'
#' - `boot_out`: If available, the original output from `boot::boot()`.
#'
#'
#' @param output The output from [stats::lm()]. It can also accept the output
#'               from
#'               [std_selected()] or [std_selected_boot()].
#' @param x      The focal variable (independent variable), that is, the
#'               variable with its effect on the outcome variable (dependent)
#'              being moderated. It must be a numeric variable.
#' @param w      The moderator. Both numeric variables and categorical variables
#'               (character or factor) are supported.
#' @param ...  Arguments to be passed to [cond_effect()].
#' @param conf The level of confidence for the confidence interval.
#'              Default is .95, to get 95% confidence intervals.
#' @param nboot The number of bootstrap samples. Default is 100.
#' @param boot_args A named list of arguments to be passed to [boot::boot()].
#'                 Default is `NULL`.
#' @param save_boot_est If `TRUE`, the default, the bootstrap estimates will
#'                      be saved in the element
#'                      `boot_est` of the output.
#' @param full_output Whether the full output from [boot::boot()] will be
#'                    returned.
#'                    Default is `FALSE`. If `TRUE`, the full output from
#'                    [boot::boot()] will be saved in the element `boot_out`
#'                    of the output.
#' @param do_boot Whether bootstrapping confidence intervals will be formed.
#'                Default is `TRUE`. If `FALSE`, all arguments related to
#'                bootstrapping will be ignored.
#'
#' @examples
#'
#' # Load a sample data set
#'
#' dat <- test_x_1_w_1_v_1_cat1_n_500
#'
#' # Do a moderated regression by lm
#' lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
#' summary(lm_raw)
#'
#' lm_std <- std_selected(lm_raw, to_standardize = ~ iv + mod)
#' cond_effect(lm_std, x = iv, w = mod)
#'
#' # Form nonparametric bootstrap confidence intervals
#' # Use 2000 or even 5000 for nboot in real research
#' out <- cond_effect_boot(lm_std, x = iv, w = mod, nboot = 50)
#' out
#'
#' @export
#' @describeIn cond_effect A wrapper of [cond_effect()] that forms
#'             nonparametric bootstrap confidence intervals.
#' @order 2

cond_effect_boot <- function(output,
                              x = NULL,
                              w = NULL,
                              ...,
                              conf = .95,
                              nboot = 100,
                              boot_args = NULL,
                              save_boot_est = TRUE,
                              full_output = FALSE,
                              do_boot = TRUE) {
    x0 <- deparse(substitute(x))
    if (inherits(tryCatch(x00 <- as.character(x), error = function(e) e),
                 "error")) {
        x <- x0
      } else {
        x <- x00
      }
    w0 <- deparse(substitute(w))
    if (inherits(tryCatch(w00 <- as.character(w), error = function(e) e),
                 "error")) {
        w <- w0
      } else {
        w <- w00
      }
    cond_effect_out <- cond_effect(output = output,
                                   x = x,
                                   w = w,
                                   ...)
    if (do_boot) {
        dat <- stats::model.frame(output)
        bootfct <- create_boot_cond_effect(output, x, w, ...)
        boot_out <- do.call(boot::boot,
                  c(list(data = dat,
                        statistic = bootfct,
                        R = nboot),
                        boot_args))
        p <- length(boot_out$t0)
        cis <- t(sapply(seq_len(p), function(x) {
                    boot::boot.ci(boot_out, conf = conf,
                                  type = "perc", index = x)$percent[4:5]
                  }))
        colnames(cis) <- c("CI Lower", "CI Upper")
        rownames(cis) <- cond_effect_out$Level
        attr(cond_effect_out, "boot_ci") <- cis
        cse <- which(colnames(cond_effect_out) == "Std. Error")
        cond_effect_out <- insert_columns(dat = cond_effect_out,
                                          x = cis,
                                          after = cse - 1)
        attr(cond_effect_out, "nboot") <- nboot
        attr(cond_effect_out, "conf") <- conf
        tmp <- boot_out$t
        colnames(tmp) <- cond_effect_out$Level
        attr(cond_effect_out, "boot_est") <- tmp
        attr(cond_effect_out, "cond_effect_boot_call") <- match.call()
        if (full_output) {
            attr(cond_effect_out, "boot_out") <- boot_out
          }
      }
    cond_effect_out
  }

create_boot_cond_effect <- function(output, x, w, ...) {
  if (is.null(output$std_selected_boot_call)) {
      out <- function(d, ind) {
                force(output)
                dat_i <- d[ind, ]
                out_i <- stats::update(output, data = dat_i)
                cond_effect(output = out_i,
                            x = x,
                            w = w,
                            ...)[, "x's Effect"]
              }
      return(out)
    } else {
      output$std_selected_boot_call$do_boot <- FALSE
      out <- function(d, ind) {
                force(output)
                dat_i <- d[ind, ]
                out_i <- stats::update(output, data = dat_i)
                cond_effect(output = out_i,
                            x = x,
                            w = w,
                            ...)[, "x's Effect"]
              }
    }
  }

insert_columns <- function(dat, x, after = 1) {
    k0 <- ncol(dat)
    names0 <- colnames(dat)
    if (is.null(dim(x))) {
        x <- cbind(x)
      }
    names1 <- colnames(x)
    k1 <- ncol(x)
    out <- dat
    names2 <- c(names0[1:after], names1, names0[(after + 1):k0])
    out[, (after + k1):(k0 + k1)] <- out[, after:k0]
    out[, (after + 1):(after + k1)] <- x
    colnames(out) <- names2
    out
  }

#'@title The 'vcov' Method for a 'std_selected' Class Object
#'
#'@description Compute the variance-covariance matrix
#' of estimates in the output of [std_selected()] or
#' [std_selected_boot()].
#'
#'@details If bootstrapping was used to form the confidence intervals,
#' users can request the variance-covariance matrix of the bootstrap estimates.
#'
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#'@return
#'  A matrix of the variances and covariances of the parameter estimates.
#'
#'@param object The output of [std_selected()] or [std_selected_boot()].
#'@param type The type of variance-covariance matrix. If set to `"lm"`,
#'            returns the results of the [stats::vcov()] method for the output of
#'            [lm()]. If set to `"boot"`,
#'            the variance-covariance matrix of the bootstrap estimates
#'            is returned.
#'            Default depends on `object`. If bootstrap estimates were stored,
#'            then the default is `"boot"`. Otherwise, the default is `"lm"`.
#'@param ...  Arguments to be passed to [stats::vcov()].
#'
#'@examples
#' # Load a sample data set
#'
#' dat <- test_x_1_w_1_v_1_cat1_n_500
#' head(dat)
#'
#' # Do a moderated regression by lm
#' lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
#'
#' # Standardize all variables except for categorical variables.
#' # Interaction terms are formed after standardization.
#' lm_std <- std_selected(lm_raw, to_scale = ~ .,
#'                                to_center = ~ .)
#'
#' # VCOV of lm output
#' vcov(lm_std)
#'
#' # Standardize all variables as in std_selected above, and compute the
#' # nonparametric bootstrapping percentile confidence intervals.
#' lm_std_boot <- std_selected_boot(lm_raw,
#'                                  to_scale = ~ .,
#'                                  to_center = ~ .,
#'                                  conf = .95,
#'                                  nboot = 100)
#' # In real analysis, nboot should be at least 2000.
#'
#' # VCOV of bootstrap estimates, default when bootstrap was conducted
#' vcov(lm_std_boot)
#'
#' # For OLS VCOV
#' vcov(lm_std_boot, type = "lm")
#'
#' @export


vcov.std_selected <- function(object, type, ...) {
    if (!missing(type)) {
        if (!(type %in% c("lm", "boot"))) {
            stop("type must be either lm or boot.")
          }
      } else {
        if (is.null(object$boot_est)) {
            type <- "lm"
          } else {
            type <- "boot"
          }
      }
    if (type == "boot") {
        if (is.null(object$boot_est)) {
            stop("Bootstrap estimates not available in the object.")
          }
        out <- stats::cov(object$boot_est)
        return(out)
      }
    if (type == "lm") {
        NextMethod()
      }
  }

#' @title Conditional Effect in a 'cond_effect'-Class Object
#'
#' @description Return the estimates of the conditional
#'              effects
#'              in the output of
#'              [cond_effect()] or [cond_effect_boot()].
#'
#' @details It just extracts and returns the column of
#'  conditional effects in a `cond_effect`-class object.
#'
#' @return
#'  A numeric vector: The estimates of the conditional
#'  effects in a `cond_effect`-class object.
#'
#' @param object The output of [cond_effect()] or [cond_effect_boot()].
#' @param ...  Optional arguments. Ignored by the function.
#'
#' @author Shu Fai Cheung <https://orcid.org/0000-0002-9871-9448>
#'
#'
#' @examples
#'
#' # Load a sample data set
#'
#' dat <- test_x_1_w_1_v_1_cat1_n_500
#'
#' # Do a moderated regression by lm
#' lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
#' summary(lm_raw)
#' out <- cond_effect(lm_raw, x = iv, w = mod)
#' out
#' coef(out)
#'
#' lm_std <- std_selected(lm_raw, to_standardize = ~ iv + mod)
#' out <- cond_effect(lm_std, x = iv, w = mod)
#' out
#' coef(out)
#'
#' # Categorical moderator
#' lm_cat <- lm(dv ~ iv*cat1 + v1, dat)
#' summary(lm_cat)
#' out <- cond_effect(lm_cat, x = iv, w = cat1)
#' out
#' coef(out)
#'
#' @export

coef.cond_effect <- function(object, ...) {
    out <- as.data.frame(object)[, "x's Effect"]
    names(out) <- object$Level
    out
  }

---
title: "Moderation Effects Plots by plotmod()"
author: "Shu Fai Cheung"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Moderation Effects Plots by plotmod()}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width  = 6,
  fig.height = 4,
  fig.align = "center"
)
```

# Introduction

This vignette illustrates how to use
`plotmod()` from the `stdmod` package.
More about this package can be found
in `vignette("stdmod", package = "stdmod")`
or at [https://sfcheung.github.io/stdmod/](https://sfcheung.github.io/stdmod/).

# What `plotmod()` Can Do

It can visualize a moderation effect by plotting the effect of
the predictor (focal variable) on the outcome variable (dependent variable)
for different levels of
the moderator:

```{r plotmod_example, echo = FALSE}
library(stdmod)
data(sleep_emo_con)
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration")
```

It can also plot a tumble graph (Bodner, 2016), in which the width
of each line depends on the distribution of the predictor (focal variable)
in each level
of the moderator:

```{r plotmod_example2, echo = FALSE}
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration",
        graph_type = "tumble")
```

`plotmod()` is not designed to be a versatile tool. It is designed to
be a function "good-enough" for common scenarios. The output of
`plotmod()` is a `ggplot` object and so can be further manipulated
by functions from the `ggplot2` package.

# Major Arguments

## Model, Predictor (`x`), and Moderator (`w`)

- `output`: The output of `lm()`, `std_selected()`, or `std_selected_boot()`,
          with at least one interaction term.

- `x`: The predictor (focal variable). The variable in the horizontal axis.

- `w`: The moderator. The variable for which the conditional effects will
       be plotted.

These are the only required arguments. Just setting them can generate the
graph:

```{r plotmod_example_raw}
library(stdmod)
data(sleep_emo_con)
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness")
```

## Levels of the Moderator

### Numeric Moderators

If the moderator is a numeric variable, then two lines will
  be drawn, by default. One line for one standard deviation (SD) below the mean
  on the
  moderator ("Low"), and the other for one SD above the mean ("High").

Users can also use percentiles to define "Low" and "High" by setting
  `w_method` to `"percentile"`. By default, 16th percentile and
  84th percentile are used, which correspond approximately to one SD below
  and above mean, respectively, for a normal distribution.

```{r plotmod_example_raw_numeric}
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        w_method = "percentile")
```

### Categorical Moderators

If the moderator is a categorical variable (a string variable or
  a factor), then one line will be drawn for each category.

```{r plotmod_example_raw_cat}
set.seed(61452)
sleep_emo_con$city <- sample(c("Alpha", "Beta", "Gamma"),
                             nrow(sleep_emo_con),
                             replace = TRUE)
lm_cat <- lm(sleep_duration ~ age + gender +
                              emotional_stability * city,
             sleep_emo_con)
plotmod(lm_cat,
        x = "emotional_stability",
        w = "city")
```

## Tumble Graph

By default, each line starts from one SD below mean of the predictor to
one SD above mean of the predictor. This assumes that variation of the
predictor is the same for different levels of the moderator. However,
this may not be the case when the predictor and the moderator are associated.
Bodner (2016) proposed a type of graph called tumble graph that can show
the association between the predictor and the moderator, if any.

The idea is simple. Instead of using the SD of the predictor in the whole
sample, the mean and SD of the predictor for the selected level of the
moderator is used. (See the technical notes below on how it is implemented
in `plotmod()`.)

To plot a tumble graph, just set `graph_type` to `"tumble"`:

```{r plotmod_tumble}
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        graph_type = "tumble")
```

The graph above shows that, the distribution of the predictor shifts to
the left when the moderator is low.

# Decoration and Annotation

## Variable Labels

By default, the variable names are used in the graph. Users can supply
labels that will be used instead of variable names by setting
`x_label`, `w_label`, and `y_label` to strings for the predictor,
moderator, and outcome variable, respectively.

```{r plotmod_annotate}
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        x_label = "EMO",
        w_label = "CON",
        y_label = "SLEEP")
```

## Title

The default title is "Moderation Effect". This can be changed via the
argument `title`:

```{r plotmod_title}
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        title = "EMO Effects For Low/High CON")
```

Some journals require submitted figures to have no title because they will
be described by figure captions, supplied separately. The title can be
disabled by setting `no_title` to `TRUE`.

## Line Width and Point Size

`plotmod()` supports basic control of the lines. The width of the lines
can be set by `line_width` and the size on the end-points can be set
by `point_size`:

```{r plotmod_line_width}
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
plotmod(lm_raw,
        x = "emotional_stability",
        w = "conscientiousness",
        point_size = 8,
        line_width = 2)
```

# Information

## Conditional Effects

By default, `plotmod()` prints the conditional effects of the predictor
as a subtitle. This can be disabled by adding a `ggplot2::theme()` call
to the output and set `plot.subtitle` to `element_blank()`, which removes
the subtitle:

```{r plotmod_cond}
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
p <- plotmod(lm_raw,
              x = "emotional_stability",
              w = "conscientiousness")
library(ggplot2)
p + theme(plot.subtitle = element_blank())
```

## Definitions of the Levels of the Moderator

By default, the definitions of the levels of moderator (e.g.,
one SD below mean for "Low" and one SD above mean for "High")
are printed as a caption. This can also be disabled by adding
a `ggplot2::theme()` call and set `plot.caption` to `element_blank()`:

```{r plotmod_levels}
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
p <- plotmod(lm_raw,
             x = "emotional_stability",
             w = "conscientiousness")
p + theme(plot.caption = element_blank())
```

## Any Variables Standardized?

If the output is generated by `std_selected()` or `std_selected_boot()`,
`plotmod()` will try to detect if a variable is standardized or not,
and report this in the graph:

```{r plotmod_any_std}
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
lm_std <- std_selected(lm_raw,
                       to_standardize = ~ emotional_stability + conscientiousness)
# Before Version 0.2.6.3 of stdmod, to_center and to_scale need to be used:
# lm_std <- std_selected(lm_raw,
#                        to_center = ~ emotional_stability + conscientiousness,
#                        to_scale = ~ emotional_stability + conscientiousness)
```

The output of `std_selected()` can then be passed to `plotmod()`:

```{r plotmod_std_selected}
plotmod(lm_std,
        x = "emotional_stability",
        w = "conscientiousness")
```

If the predictor, moderator, and outcome variable are all standardized,
then the moderation effect is standardized moderation effect
([Cheung, Cheung, Lau, Hui, & Vong, 2022](https://doi.org/10.1037/hea0001188)), and the
conditional effect printed on the graph are the standardized effects
of the predictor for different levels of the moderator.

If which variables are standardized will be reported in the main text
and so the figure does not need to report this, set `note_standardized`
to `FALSE`.


# Tweak the Graph

The output of `plotmod()` is a `ggplot` object. Therefore, it can be manipulated
by functions that modify a `ggplot` object. For example, users can change
the colors of the lines and the theme:

```{r plotmod_tweak}
lm_raw <- lm(sleep_duration ~ age + gender +
                              emotional_stability * conscientiousness,
             sleep_emo_con)
p <- plotmod(lm_std,
              x = "emotional_stability",
              w = "conscientiousness")
p + scale_color_manual(values = c("blue", "red")) +
    theme_classic()
```

# Technical Notes for Tumble Graph

Bodner (2016) proposed regressing the predictor on the moderator. `plotmod()`
uses a nonparametric approach.

If the moderator is numerical, for a level of moderator, say, one SD
below mean, a certain percentage of cases (about 16% of cases above
and below this value of the moderator) is used to compute the mean
and SD of the predictor, which will be used to determine the end-points
of the line segment for this level.

If the moderator is categorical, then the mean and SD of all cases
  in each group will be used to the determine the end-points of a line
  segment.

# Further Information

Please refer to the help page of `plotmod()` for other options available.

# Reference

Bodner, T. E. (2016). Tumble graphs: Avoiding misleading end point
extrapolation when graphing interactions from a moderated multiple
regression analysis. *Journal of Educational and Behavioral
Statistics, 41*(6), 593-604. \doi{10.3102/1076998616657080}

Cheung, S. F., Cheung, S.-H., Lau, E. Y. Y., Hui, C. H., & Vong, W. N. (2022)
Improving an old way to measure moderation effect in standardized units.
*Health Psychology*, *41*(7), 502-505. https://doi.org/10.1037/hea0001188.

---
title: "Standardized Moderation Effect by std_selected()"
author: "Shu Fai Cheung and David Weng Ngai Vong"
date: "2026-01-04"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Standardized Moderation Effect by std_selected()}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



# Purpose

This document demonstrates how to use `std_selected()` from
the `stdmod` package to compute the correct
standardized solution of moderated regression.
More about this package can be found
in `vignette("stdmod", package = "stdmod")`
or at [https://sfcheung.github.io/stdmod/](https://sfcheung.github.io/stdmod/).

# Setup the Environment


``` r
library(stdmod) # For computing the standardized moderation effect conveniently
```

# Load the Dataset


``` r
data(sleep_emo_con)
head(sleep_emo_con, 3)
#> # A tibble: 3 × 6
#>   case_id sleep_duration conscientiousness emotional_stability   age gender
#>     <int>          <dbl>             <dbl>               <dbl> <dbl> <chr> 
#> 1       1              6               3.6                 3.6    20 female
#> 2       2              4               3.8                 2.4    20 female
#> 3       3              7               4.3                 2.7    20 female
```

This data set has 500 cases of data. The variables are sleep duration, age, gender,
and the scores from two personality scales, emotional stability and
conscientiousness of the IPIP Big Five markers. Please refer to
(citation to be added) for the detail of the data set.

The names of some variables are shortened for readability:


``` r
colnames(sleep_emo_con)[3:4] <- c("cons", "emot")
head(sleep_emo_con, 3)
#> # A tibble: 3 × 6
#>   case_id sleep_duration  cons  emot   age gender
#>     <int>          <dbl> <dbl> <dbl> <dbl> <chr> 
#> 1       1              6   3.6   3.6    20 female
#> 2       2              4   3.8   2.4    20 female
#> 3       3              7   4.3   2.7    20 female
```


# Moderated Regression

Suppose we are interested in predicting sleep duration by emotional stability,
after controlling for gender and age. However, we suspect that the effect of
emotional stability, if any, may be moderated by conscientiousness. Therefore,
we conduct a moderated regression as follow:


``` r
lm_out <- lm(sleep_duration ~ age + gender + emot * cons,
             data = sleep_emo_con)
summary(lm_out)
#> 
#> Call:
#> lm(formula = sleep_duration ~ age + gender + emot * cons, data = sleep_emo_con)
#> 
#> Residuals:
#>     Min      1Q  Median      3Q     Max 
#> -6.0841 -0.7882  0.0089  0.9440  6.1189 
#> 
#> Coefficients:
#>             Estimate Std. Error t value Pr(>|t|)   
#> (Intercept)  1.85154    1.35224   1.369  0.17155   
#> age          0.01789    0.02133   0.838  0.40221   
#> gendermale  -0.26127    0.16579  -1.576  0.11570   
#> emot         1.32151    0.45039   2.934  0.00350 **
#> cons         1.20385    0.37062   3.248  0.00124 **
#> emot:cons   -0.33140    0.13273  -2.497  0.01286 * 
#> ---
#> Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
#> 
#> Residual standard error: 1.384 on 494 degrees of freedom
#> Multiple R-squared:  0.0548,	Adjusted R-squared:  0.04523 
#> F-statistic: 5.728 on 5 and 494 DF,  p-value: 3.768e-05
plotmod(lm_out,
        x = "emot",
        w = "cons",
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration")
```

<div class="figure" style="text-align: center">
<img src="mod_reg-1.png" alt="plot of chunk mod_reg"  />
<p class="caption">plot of chunk mod_reg</p>
</div>

The results show that conscientiousness significantly moderates the effect of
emotional stability on sleep duration.

# Standardized Moderation Effect

To get the correct standardized solution of the moderated regression, with the
product term formed *after* standardization, we can use `std_selected()`.

- The first argument is the regression output from `lm()`.

- The argument `to_center` specifies variables to be mean
  centered.

- The argument `to_scale` specifies variables to be rescaled
  by their standard deviations after centering.

- In `stdmod` 0.2.6.3, the argument `to_standardize` was introduced
  as a shortcut. Listing a variable in `to_standardize` is
  equivalent to listing it in `to_center` and `to_scale`.

If we want to standardize or mean center all variables, we can use `~ .` as a
shortcut. Note that `std_selected()` will  automatically skip categorical
variables (i.e., factors or string variables in the regression model of `lm()`).


``` r
lm_stdall <- std_selected(lm_out,
                          to_standardize = ~ .)
```

Before 0.2.6.3, to standardize all variables except for
categorical variables, we need to use both `to_center = ~ .`
and `to_scale = ~ .`. Since 0.2.6.3,
we can just use `to_standardize = ~ .`, as shown above.
If `to_standardize = ~ .` does not work, just use
`to_center` and `to_scale` as shown below:

```r
lm_stdall <- std_selected(lm_out,
                          to_center = ~ .,
                          to_scale = ~ .)
```

A summary of the results of `std_selected()` can be
generated by `summary()`:


``` r
summary(lm_stdall)
#> 
#> Call to std_selected():
#> std_selected(lm_out = lm_out, to_standardize = ~.)
#> 
#> Selected variable(s) are centered by mean and/or scaled by SD
#> - Variable(s) centered: sleep_duration age gender emot cons
#> - Variable(s) scaled: sleep_duration age gender emot cons
#> 
#>                centered_by scaled_by                            Note
#> sleep_duration    6.776333 1.4168291 Standardized (mean = 0, SD = 1)
#> age              22.274000 2.9407857 Standardized (mean = 0, SD = 1)
#> gender                  NA        NA Nonnumeric                     
#> emot              2.713200 0.7629613 Standardized (mean = 0, SD = 1)
#> cons              3.343200 0.6068198 Standardized (mean = 0, SD = 1)
#> 
#> Note:
#> - Categorical variables will not be centered or scaled even if
#>   requested.
#> 
#> Call:
#> lm(formula = sleep_duration ~ age + gender + emot * cons, data = dat_mod)
#> 
#> Residuals:
#>     Min      1Q  Median      3Q     Max 
#> -4.2941 -0.5563  0.0063  0.6663  4.3187 
#> 
#> Coefficients:
#>             Estimate Std. Error t value Pr(>|t|)   
#> (Intercept)   0.0549     0.0488  1.1248  0.26124   
#> age           0.0371     0.0443  0.8384  0.40221   
#> gendermale   -0.1844     0.1170 -1.5759  0.11570   
#> emot          0.1150     0.0449  2.5600  0.01076 * 
#> cons          0.1305     0.0452  2.8893  0.00403 **
#> emot:cons    -0.1083     0.0434 -2.4967  0.01286 * 
#> ---
#> Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
#> 
#> Residual standard error: 0.9771 on 494 degrees of freedom
#> 
#> R-squared                : 0.0548
#> Adjusted R-squared       : 0.0452
#> ANOVA test of R-squared  : F(5, 494) = 5.7277, p < 0.001
#> 
#> = Test the highest order term =
#> The highest order term             : emot:cons
#> R-squared increase adding this term: 0.0119
#> F test of R-squared increase       : F(1, 494) = 6.2335, p = 0.013
#> 
#> Note:
#> - Estimates and their statistics are based on the data after
#>   mean-centering, scaling, or standardization.
#> - One or more variables are scaled by SD or standardized. OLS standard
#>   errors and confidence intervals may be biased for their coefficients.
#>   Please use `std_selected_boot()`.
```

The coefficient in this solution,
-0.10829,
can be interpreted as the change in the standardized effect of
emotional stability for each one standard deviation increase of
conscientiousness. Naturally, this can be called the
*standardized moderation effect* of conscientiousness
([Cheung, Cheung, Lau, Hui, & Vong, 2022](https://doi.org/10.1037/hea0001188)).

The output of `std_selected()` can be passed to other functions that accept the
output of `lm()`. This package also has a simple function,
`plotmod()`, for generating a typical plot of the moderation effect:


``` r
plotmod(lm_stdall,
        x = "emot",
        w = "cons",
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration")
```

<div class="figure" style="text-align: center">
<img src="mod_reg_stdall-1.png" alt="plot of chunk mod_reg_stdall"  />
<p class="caption">plot of chunk mod_reg_stdall</p>
</div>

The function `plotmod()` also prints the conditional effects of the predictor
(focal variable), emotional stability in this example.

# The Common (Incorrect) Standardized Solution

For comparison, this is the results of standardizing all variables, including
the product term and the categorical variable.


``` r
library(lm.beta) # For generating the typical standardized solution
packageVersion("lm.beta")
#> [1] '1.7.3'
lm_beta <- lm.beta(lm_out)
summary(lm_beta)
#> 
#> Call:
#> lm(formula = sleep_duration ~ age + gender + emot * cons, data = sleep_emo_con)
#> 
#> Residuals:
#>     Min      1Q  Median      3Q     Max 
#> -6.0841 -0.7882  0.0089  0.9440  6.1189 
#> 
#> Coefficients:
#>             Estimate Standardized Std. Error t value Pr(>|t|)   
#> (Intercept)  1.85154           NA    1.35224   1.369  0.17155   
#> age          0.01789      0.03712    0.02133   0.838  0.40221   
#> gendermale  -0.26127     -0.06934    0.16579  -1.576  0.11570   
#> emot         1.32151      0.71163    0.45039   2.934  0.00350 **
#> cons         1.20385      0.51560    0.37062   3.248  0.00124 **
#> emot:cons   -0.33140     -0.78201    0.13273  -2.497  0.01286 * 
#> ---
#> Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
#> 
#> Residual standard error: 1.384 on 494 degrees of freedom
#> Multiple R-squared:  0.0548,	Adjusted R-squared:  0.04523 
#> F-statistic: 5.728 on 5 and 494 DF,  p-value: 3.768e-05
```

The coefficient of the *standardized* product term is
-0.78201, which
*cannot* be interpreted as the change in the standardized effect of
emotional stability for each one standard deviation increase of
conscientiousness because the product term is standardized and can no longer
be interpreted as the product of two variables in the model.

# Improved Confidence Intervals

It has been shown (e.g., [Yuan & Chan, 2011](https://doi.org/10.1007/s11336-011-9224-6))
that the standard errors of
standardized regression coefficients computed just by standardizing the variables
are biased, and consequently the confidence intervals are also invalid. The
function `std_selected_boot()` is a wrapper of `std_selected()` that also
forms the confidence interval of the regression coefficients when standardizing
is conducted, using nonparametric bootstrapping as suggested by
Cheung, Cheung, Lau, Hui, and Vong (2022).

We use the same example above that standardizes all variables except for
categorical variables to illustrate this function. The argument `nboot`
specifies the number of nonparametric bootstrap samples.
The level of confidence is set by `conf`. The default is .95, denoting 95%
confidence intervals. If this is the desired level, this argument can be
omitted.




``` r
set.seed(649017)
lm_xwy_std_ci <- std_selected_boot(lm_out,
                                   to_standardize = ~ .,
                                   nboot = 2000)
```

If the default options are acceptable, the only additional argument is `nboot`.


``` r
summary(lm_xwy_std_ci)
#> 
#> Call to std_selected_boot():
#> std_selected_boot(lm_out = lm_out, to_standardize = ~., nboot = 2000)
#> 
#> Selected variable(s) are centered by mean and/or scaled by SD
#> - Variable(s) centered: sleep_duration age gender emot cons
#> - Variable(s) scaled: sleep_duration age gender emot cons
#> 
#>                centered_by scaled_by                            Note
#> sleep_duration    6.776333 1.4168291 Standardized (mean = 0, SD = 1)
#> age              22.274000 2.9407857 Standardized (mean = 0, SD = 1)
#> gender                  NA        NA Nonnumeric                     
#> emot              2.713200 0.7629613 Standardized (mean = 0, SD = 1)
#> cons              3.343200 0.6068198 Standardized (mean = 0, SD = 1)
#> 
#> Note:
#> - Categorical variables will not be centered or scaled even if
#>   requested.
#> - Nonparametric bootstrapping 95% confidence intervals computed.
#> - The number of bootstrap samples is 2000.
#> 
#> Call:
#> lm(formula = sleep_duration ~ age + gender + emot * cons, data = dat_mod)
#> 
#> Residuals:
#>     Min      1Q  Median      3Q     Max 
#> -4.2941 -0.5563  0.0063  0.6663  4.3187 
#> 
#> Coefficients:
#>             Estimate CI Lower CI Upper Std. Error t value Pr(>|t|)   
#> (Intercept)   0.0549   0.0030   0.1043     0.0488  1.1248  0.26124   
#> age           0.0371  -0.0363   0.1036     0.0443  0.8384  0.40221   
#> gendermale   -0.1844  -0.4389   0.0876     0.1170 -1.5759  0.11570   
#> emot          0.1150   0.0236   0.2024     0.0449  2.5600  0.01076 * 
#> cons          0.1305   0.0324   0.2242     0.0452  2.8893  0.00403 **
#> emot:cons    -0.1083  -0.2040  -0.0097     0.0434 -2.4967  0.01286 * 
#> ---
#> Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1
#> 
#> Residual standard error: 0.9771 on 494 degrees of freedom
#> 
#> R-squared                : 0.0548
#> Adjusted R-squared       : 0.0452
#> ANOVA test of R-squared  : F(5, 494) = 5.7277, p < 0.001
#> 
#> = Test the highest order term =
#> The highest order term             : emot:cons
#> R-squared increase adding this term: 0.0119
#> F test of R-squared increase       : F(1, 494) = 6.2335, p = 0.013
#> 
#> Note:
#> - Estimates and their statistics are based on the data after
#>   mean-centering, scaling, or standardization.
#> - [CI Lower, CI Upper] are bootstrap percentile confidence intervals.
#> - Std. Error are not bootstrap SEs.
```



The standardized moderation effect is
-0.1083,
and the 95% nonparametric bootstrap confidence interval is
-0.2040 to
-0.0097.

Note: As a side product, the nonparametric bootstrap percentile confidence of the other
coefficients are also reported. They can be used for other variables that are
standardized in the same model, whether they are involved in the moderation or not.

# Further Information

`vignette("plotmod", package = "stdmod")` illustrates how to use `plotmod()` to plot a moderation
effect. If variables are standardized by `std_selected()`, `plotmod()` can
indicate this in the plot.

`vignette("cond_effect", package = "stdmod")` illustrates how to use `cond_effect()` to compute
conditional effects, the effect of a predictor (focal variable) for selected
levels of the moderator.
`cond_effect()` supports outputs from `std_selected()`.

# Reference(s)

Cheung, S. F., Cheung, S.-H., Lau, E. Y. Y., Hui, C. H., & Vong, W. N. (2022)
Improving an old way to measure moderation effect in standardized units.
*Health Psychology*, *41*(7), 502-505. https://doi.org/10.1037/hea0001188.

Yuan, K.-H., & Chan, W. (2011). Biases and standard errors of standardized
regression coefficients. *Psychometrika, 76*(4), 670-690.
https://doi.org/10.1007/s11336-011-9224-6

---
title: "Standardized Moderation Effect in a Path Model by stdmod_lavaan()"
author: "Shu Fai Cheung and David Weng Ngai Vong"
date: "2026-01-04"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Standardized Moderation Effect in a Path Model by stdmod_lavaan()}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



# Purpose

This document demonstrates how to use `stdmod_lavaan()` from
the package `stdmod` to compute the
standardized moderation effect in a path model fitted by `lavaan::sem()`.

More about this package can be found
in `vignette("stdmod", package = "stdmod")`
or at [https://sfcheung.github.io/stdmod/](https://sfcheung.github.io/stdmod/).

# Setup the Environment


``` r
library(stdmod) # For computing the standardized moderation effect conveniently
library(lavaan) # For doing path analysis in lavaan.
```

# Load the Dataset


``` r
data(test_mod1)
round(head(test_mod1, 3), 3)
#>       dv     iv    mod    med   cov1   cov2
#> 1 23.879 -0.133 -0.544 10.310 -0.511 -0.574
#> 2 23.096  1.456  1.539 11.384  0.094 -0.264
#> 3 23.201  0.319  1.774  9.615 -0.172  0.488
```

This test data set has 300 cases, six variables, all continuous.


# Fit the Model by `lavaan::sem()`

The product term can be formed manually or by the colon operator, `:`.
`stdmod_lavaan()` will work in both cases.

This is the model to be tested:


``` r
mod <-
"
med ~ iv + mod + iv:mod + cov1
dv ~ med + cov2
"
fit <- sem(mod, test_mod1, fixed.x = FALSE)
summary(fit)
#> lavaan 0.6-21.2434 ended normally after 1 iteration
#> 
#>   Estimator                                         ML
#>   Optimization method                           NLMINB
#>   Number of model parameters                        23
#> 
#>   Number of observations                           300
#> 
#> Model Test User Model:
#>                                                       
#>   Test statistic                                 1.058
#>   Degrees of freedom                                 5
#>   P-value (Chi-square)                           0.958
#> 
#> Parameter Estimates:
#> 
#>   Standard errors                             Standard
#>   Information                                 Expected
#>   Information saturated (h1) model          Structured
#> 
#> Regressions:
#>                    Estimate  Std.Err  z-value  P(>|z|)
#>   med ~                                               
#>     iv                0.221    0.030    7.264    0.000
#>     mod               0.104    0.030    3.489    0.000
#>     iv:mod            0.257    0.025   10.169    0.000
#>     cov1              0.104    0.025    4.099    0.000
#>   dv ~                                                
#>     med               0.246    0.041    5.962    0.000
#>     cov2              0.191    0.023    8.324    0.000
#> 
#> Covariances:
#>                    Estimate  Std.Err  z-value  P(>|z|)
#>   iv ~~                                               
#>     mod               0.481    0.063    7.606    0.000
#>     iv:mod           -0.149    0.059   -2.501    0.012
#>     cov1             -0.033    0.058   -0.575    0.565
#>     cov2             -0.071    0.059   -1.216    0.224
#>   mod ~~                                              
#>     iv:mod           -0.180    0.062   -2.923    0.003
#>     cov1             -0.060    0.059   -1.010    0.313
#>     cov2             -0.107    0.061   -1.763    0.078
#>   iv:mod ~~                                           
#>     cov1             -0.051    0.061   -0.837    0.403
#>     cov2              0.063    0.063    1.001    0.317
#>   cov1 ~~                                             
#>     cov2              0.071    0.061    1.158    0.247
#> 
#> Variances:
#>                    Estimate  Std.Err  z-value  P(>|z|)
#>    .med               0.201    0.016   12.247    0.000
#>    .dv                0.169    0.014   12.247    0.000
#>     iv                0.954    0.078   12.247    0.000
#>     mod               1.017    0.083   12.247    0.000
#>     iv:mod            1.088    0.089   12.247    0.000
#>     cov1              1.039    0.085   12.247    0.000
#>     cov2              1.076    0.088   12.247    0.000
```

The results show that `mod` significantly moderates the effect of
`iv` on `med`.

# Compute the Standardized Moderation Effect

As in the case of regression, the coefficient of `iv:mod` in
the standardized solution is not the desired standardized coefficient because
it standardizes the product term.


``` r
standardizedSolution(fit)[3, ]
#>   lhs op    rhs est.std    se      z pvalue ci.lower ci.upper
#> 3 med  ~ iv:mod   0.466 0.043 10.842      0    0.382     0.55
```

After fitting the path model by `lavaan::lavaan()`, we can use `stdmod_lavaan()`
to compute the standardized moderation effect using the standard deviations
of the focal variable, the moderator, and the outcome variable
[(Cheung, Cheung, Lau, Hui, & Vong, 2022)](https://doi.org/10.1037/hea0001188).

The minimal arguments are:

- `fit`: The output from `lavaan::lavaan()` and its wrappers, such
          as `lavaan::sem()`.
- `x`: The focal variable, the variable with its effect on the
       outcome variable being moderated.
- `y`: The outcome variable.
- `w`: The moderator.
- `x_w`: The product term.


``` r
fit_iv_mod_std <- stdmod_lavaan(fit = fit,
                                x = "iv",
                                y = "med",
                                w = "mod",
                                x_w = "iv:mod")
fit_iv_mod_std
#> 
#> Call:
#> stdmod_lavaan(fit = fit, x = "iv", y = "med", w = "mod", x_w = "iv:mod")
#> 
#>                  Variable
#> Focal Variable         iv
#> Moderator             mod
#> Outcome Variable      med
#> Product Term       iv:mod
#> 
#>              lhs op    rhs   est    se      z pvalue ci.lower ci.upper
#> Original     med  ~ iv:mod 0.257 0.025 10.169      0    0.208    0.307
#> Standardized med  ~ iv:mod 0.440    NA     NA     NA       NA       NA
```

The standardized moderation effect of `mod` on the `iv`-`med` path is
0.440.

# Form Bootstrap Confidence Interval

`stdmod_lavaan()` can also be used to form nonparametric bootstrap
confidence interval for the standardized moderation effect.

There are two approaches to do this. First, if bootstrap
confidence intervals was requested when fitting the model,
the stored bootstrap estimates will be used. This is
efficient because there is no need to do bootstrapping
again.

We fit the model again, with bootstrapping:




``` r
fit <- sem(mod, test_mod1, fixed.x = FALSE,
           se = "boot",
           bootstrap = 2000,
           iseed = 987543)
```

If bootstrapping has been done when fitting the model,
just adding `boot_ci = TRUE` is enough to request
nonparametric percentile bootstrap confidence interval:


``` r
fit_iv_mod_std_ci <- stdmod_lavaan(fit = fit,
                                   x = "iv",
                                   y = "med",
                                   w = "mod",
                                   x_w = "iv:mod",
                                   boot_ci = TRUE)
fit_iv_mod_std_ci
#> 
#> Call:
#> stdmod_lavaan(fit = fit, x = "iv", y = "med", w = "mod", x_w = "iv:mod", 
#>     boot_ci = TRUE)
#> 
#>                  Variable
#> Focal Variable         iv
#> Moderator             mod
#> Outcome Variable      med
#> Product Term       iv:mod
#> 
#>              lhs op    rhs   est    se     z pvalue ci.lower ci.upper
#> Original     med  ~ iv:mod 0.257 0.035 7.298      0    0.184    0.322
#> Standardized med  ~ iv:mod 0.440    NA    NA     NA    0.322    0.539
#> 
#> Confidence interval of standardized moderation effect:
#> - Level of confidence: 95%
#> - Bootstrapping Method: Nonparametric
#> - Type: Percentile
#> - Number of bootstrap samples requests: 
#> - Number of bootstrap samples with valid results: 2000
#> 
#> NOTE: Bootstrapping conducted by the method in 0.2.7.5 or later. To use
#> the method in the older versions for reproducing previous results, set
#> 'use_old_version' to 'TRUE'.
```

The 95% confidence interval of the standardized moderation effect is
0.322 to
0.539.

The second approach, not covered here, uses
[`do_boot()`](https://sfcheung.github.io/manymome/articles/do_boot.html)
from
the [`manymome`](https://sfcheung.github.io/manymome/index.html) package.
to generate bootstrap estimates. To use the stored bootstrap
estimates, set `boot_out` to the output of `do_boot()`.
The stored bootstrap estimates will then be used. This method
can be used when non-bootstrapping confidence intervals are
needed when fitting the model.

# Remarks

The function `stdmod_lavaan()` can be used for more complicated path models.
The computation of the standardized moderation effect in a path model depends
only on the standard deviations of the three variables involved
(`x`, `w`, and `y`).

# Reference(s)

The computation of the standardized moderation effect is based on the simple
formula presented in the following manuscript, using the standard deviations of
the outcome variable, focal variable, and the moderator:

Cheung, S. F., Cheung, S.-H., Lau, E. Y. Y., Hui, C. H., & Vong, W. N. (2022)
Improving an old way to measure moderation effect in standardized units.
*Health Psychology*, *41*(7), 502-505. https://doi.org/10.1037/hea0001188.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_3.R
\docType{data}
\name{test_mod2}
\alias{test_mod2}
\title{Sample Dataset: A Path Model With A Moderator}
\format{
A data frame with 300 rows and 6 variables:
\describe{
\item{dv}{Dependent variable, continuous}
\item{iv}{Independent variable, continuous}
\item{med}{Mediator, continuous}
\item{mod}{Moderator, continuous}
\item{cov1}{Covariate, continuous}
\item{cov2}{Covariate, continuous}
}
}
\usage{
test_mod2
}
\description{
For testing. Generated from the following
model.

\if{html}{\out{<div class="sourceCode">}}\preformatted{mod <-
"
med ~ iv + cov1
dv ~ med + mod + med:mod + cov2
"
}\if{html}{\out{</div>}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotmod.R
\name{plotmod}
\alias{plotmod}
\title{Moderation Effect Plot}
\usage{
plotmod(
  output,
  x,
  w,
  x_label,
  w_label,
  y_label,
  title,
  digits = 3,
  x_from_mean_in_sd = 1,
  w_from_mean_in_sd = 1,
  w_method = c("sd", "percentile"),
  w_percentiles = c(0.16, 0.84),
  x_method = c("sd", "percentile"),
  x_percentiles = c(0.16, 0.84),
  w_sd_to_percentiles = NA,
  x_sd_to_percentiles = NA,
  w_values = NULL,
  note_standardized = TRUE,
  no_title = FALSE,
  line_width = 1,
  point_size = 5,
  graph_type = c("default", "tumble")
)
}
\arguments{
\item{output}{The output
of \code{\link[stats:lm]{stats::lm()}}, \code{\link[=std_selected]{std_selected()}}, or
\code{\link[=std_selected_boot]{std_selected_boot()}}.}

\item{x}{The name of the focal variable (x-axis) in `output``. It
can be the name of the variable, with or without quotes.
Currently only numeric variables are supported.}

\item{w}{The name of the moderator in \code{output}. It
can be the name of the variable, with or without quotes.}

\item{x_label}{The label for the X-axis. Default is the value of \code{x}.}

\item{w_label}{The label for the legend for the lines.
Default is the value of\code{w}.}

\item{y_label}{The label for the Y-axis. Default is the
name of the response variable in the model.}

\item{title}{The title of the graph. If not supplied, it will be
generated from the variable
names or labels (in \code{x_label}, \code{y_label},
and \code{w_label}). If \code{""}, no title will be printed.
This can be used when the plot is for manuscript
submission and figures are required to have no
titles.}

\item{digits}{Number of decimal places to print. Default is 3.}

\item{x_from_mean_in_sd}{How many SD from mean is used to define
"low" and
"high" for the focal variable.
Default is 1.}

\item{w_from_mean_in_sd}{How many SD from mean is used to define
"low" and
"high" for the moderator. Default is 1.
Ignored if \code{w} is categorical.}

\item{w_method}{How to define "high" and "low" for the moderator levels.
Default is in terms of the
standard deviation of the moderator, \code{"sd"}.
If equal to
\code{"percentile"}, then the percentiles of the moderator in
the
dataset are used.
Ignored if \code{w} is categorical.}

\item{w_percentiles}{If \code{w_method} is \code{"percentile"}, then this
argument
specifies the two percentiles to be used,
divided by 100.
It must be a
vector of two numbers. The default is
\code{c(.16, .84)},
the 16th and 84th percentiles,
which corresponds approximately
to one SD below and above mean for a
normal distribution, respectively.
Ignored if \code{w} is categorical.}

\item{x_method}{How to define "high" and "low" for the focal
variable levels.
Default is in terms of the
standard deviation of the focal variable, \code{"sd"}.
If equal to
\code{"percentile"}, then the percentiles of the
focal variable in the
dataset is used.}

\item{x_percentiles}{If \code{x_method} is \code{"percentile"}, then this
argument
specifies the two percentiles to be used,
divided by 100.
It must be a
vector of two numbers. The default is
\code{c(.16, .84)},
the 16th and 84th percentiles,
which corresponds approximately
to one SD below and above mean for a
normal distribution, respectively.}

\item{w_sd_to_percentiles}{If \code{w_method} is \code{"percentile"} and
this argument is
set to a number, this number will be
used
to determine the percentiles to be used.
The
lower percentile is the percentile in a
normal
distribution
that is \code{w_sd_to_percentiles} SD below
the mean.
The upper percentile is the percentile in
a normal
distribution that is \code{w_sd_to_percentiles}
SD
above the mean. Therefore, if
\code{w_sd_to_percentiles} is set to 1, then the
lower
and upper percentiles are 16th and 84th,
respectively. Default is \code{NA}.}

\item{x_sd_to_percentiles}{If \code{x_method} is \code{"percentile"} and this
argument is
set to a number, this number will be used
to determine the percentiles to be used. The
lower percentile is the percentile in a
normal
distribution
that is \code{x_sd_to_percentiles} SD below the mean.
The upper percentile is the percentile in a normal
distribution that is \code{x_sd_to_percentiles} SD
above the mean. Therefore, if
\code{x_sd_to_percentiles} is set to 1, then the lower
and upper percentiles are 16th and 84th,
respectively. Default is \code{NA}.}

\item{w_values}{The values of \code{w} to
be used. Default is \code{NULL}. If a
numeric vector is supplied, these
values will be used to compute the
conditional effects. Other arguments
on generating levels are ignored.
Note that, if \code{w} has been standardized
or centered, these values are for
the standardized or centered \code{w}.
The values will always be sorted.
This argument is ignored if \code{w} is
categorical.}

\item{note_standardized}{If \code{TRUE}, will check whether a variable has SD
nearly equal to one. If yes, will report this in the
plot.
Default is \code{TRUE}.}

\item{no_title}{If \code{TRUE}, title will be suppressed. Default is \code{FALSE}.}

\item{line_width}{The width of the lines as used in
\code{\link[ggplot2:geom_segment]{ggplot2::geom_segment()}}.
Default is 1.}

\item{point_size}{The size of the points as used in \code{\link[ggplot2:geom_point]{ggplot2::geom_point()}}.
Default is 5.}

\item{graph_type}{If \code{"default"}, the typical line-graph with equal end-points
will be plotted. If \code{"tubmle"}, then the tumble graph
proposed by Bodner (2016) will be plotted. Default is
\code{"default"}.}
}
\value{
A \code{ggplot2} graph. Plotted if not assigned to a name. It can
be further modified like a usual \code{ggplot2} graph.
}
\description{
Plot the moderation effect in a regression model
}
\details{
This function generate a basic \code{ggplot2} graph
typically found in psychology manuscripts. It tries to
check whether one or more variables are standardized, and
report this in the plot if required.

This function only has features for typical plots of moderation effects.
It is not intended to be a flexible tool for a fine control on the plots.
}
\examples{

# Do a moderated regression by lm
lm_out <- lm(sleep_duration ~ age + gender + emotional_stability*conscientiousness, sleep_emo_con)
plotmod(lm_out,
        x = emotional_stability,
        w = conscientiousness,
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration")

# Standardize all variables except for categorical variables
# Alternative: use to_standardize as a shortcut
# lm_std <- std_selected(lm_out,
#                        to_standardize = ~ .)
lm_std <- std_selected(lm_out,
                       to_scale = ~ .,
                       to_center = ~ .)
plotmod(lm_std,
        x = emotional_stability,
        w = conscientiousness,
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration")

# Tumble Graph
plotmod(lm_std,
        x = emotional_stability,
        w = conscientiousness,
        x_label = "Emotional Stability",
        w_label = "Conscientiousness",
        y_label = "Sleep Duration",
        graph_type = "tumble")

}
\references{
Bodner, T. E. (2016). Tumble graphs: Avoiding misleading end point
extrapolation when graphing interactions from a moderated multiple
regression analysis. \emph{Journal of Educational and Behavioral
Statistics, 41}(6), 593-604. \doi{10.3102/1076998616657080}
}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coef_condeff.R
\name{coef.cond_effect}
\alias{coef.cond_effect}
\title{Conditional Effect in a 'cond_effect'-Class Object}
\usage{
\method{coef}{cond_effect}(object, ...)
}
\arguments{
\item{object}{The output of \code{\link[=cond_effect]{cond_effect()}} or \code{\link[=cond_effect_boot]{cond_effect_boot()}}.}

\item{...}{Optional arguments. Ignored by the function.}
}
\value{
A numeric vector: The estimates of the conditional
effects in a \code{cond_effect}-class object.
}
\description{
Return the estimates of the conditional
effects
in the output of
\code{\link[=cond_effect]{cond_effect()}} or \code{\link[=cond_effect_boot]{cond_effect_boot()}}.
}
\details{
It just extracts and returns the column of
conditional effects in a \code{cond_effect}-class object.
}
\examples{

# Load a sample data set

dat <- test_x_1_w_1_v_1_cat1_n_500

# Do a moderated regression by lm
lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
summary(lm_raw)
out <- cond_effect(lm_raw, x = iv, w = mod)
out
coef(out)

lm_std <- std_selected(lm_raw, to_standardize = ~ iv + mod)
out <- cond_effect(lm_std, x = iv, w = mod)
out
coef(out)

# Categorical moderator
lm_cat <- lm(dv ~ iv*cat1 + v1, dat)
summary(lm_cat)
out <- cond_effect(lm_cat, x = iv, w = cat1)
out
coef(out)

}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_stdmod.R
\name{update.std_selected}
\alias{update.std_selected}
\title{The 'update' Method for a 'std_selected' Class Object}
\usage{
\method{update}{std_selected}(object, formula., ..., evaluate = TRUE)
}
\arguments{
\item{object}{The output of the class \code{\link[=std_selected]{std_selected()}}.}

\item{formula.}{Changes to the formula.}

\item{...}{Optional arguments to be changed.}

\item{evaluate}{Whether the call will be evaluated.}
}
\value{
If \code{evaluate} = \code{TRUE}, it returns the updated fitted object,
otherwise, the updated call.
}
\description{
This should be used only to update the call to \code{\link[=lm]{lm()}},
not to the call to \code{\link[=std_selected]{std_selected()}} or \code{\link[=std_selected_boot]{std_selected_boot()}}.
}
\details{
Although supported, it is not recommended to update an analysis
processed by \code{\link[=std_selected]{std_selected()}} or \code{\link[=std_selected_boot]{std_selected_boot()}}. It is
recommended to call \code{\link[=lm]{lm()}} again and pass the output to
\code{\link[=std_selected]{std_selected()}} or \code{\link[=std_selected_boot]{std_selected_boot()}}.
}
\examples{

# Load a sample data set

dat <- test_x_1_w_1_v_1_cat1_n_500
head(dat)

# Do a moderated regression by lm
lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
summary(lm_raw)

# Standardize all variables except for categorical variables.
# Interaction terms are formed after standardization.
lm_std <- std_selected(lm_raw, to_scale = ~ .,
                               to_center = ~ .)
summary(lm_std)

# Update the model
lm_std2 <- update(lm_std, . ~ . - v1)
summary(lm_std2)

}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_stdmod.R
\name{print.std_selected}
\alias{print.std_selected}
\title{Print Basic Information of a 'std_selected' Class Object}
\usage{
\method{print}{std_selected}(x, ...)
}
\arguments{
\item{x}{The output of \code{\link[=std_selected]{std_selected()}} or \code{\link[=std_selected_boot]{std_selected_boot()}}.}

\item{...}{Arguments to be passed to \code{\link[=print]{print()}} method of \code{\link[=lm]{lm()}}.}
}
\value{
\code{x} is returned invisibly.
}
\description{
Provide information of centering and scaling, along with
basic model information printed by the \code{\link[=print]{print()}} method of \code{\link[=lm]{lm()}}.
}
\examples{

# Load a sample data set

dat <- test_x_1_w_1_v_1_cat1_n_500

# Do a moderated regression by lm
lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
summary(lm_raw)

# Standardize all variables except for categorical variables.
# Interaction terms are formed after standardization.
lm_std <- std_selected(lm_raw, to_scale = ~ .,
                               to_center = ~ .)
lm_std

# With bootstrapping
# nboot = 100 just for illustration. nboot >= 2000 should be used in read
# research.
lm_std_boot <- std_selected_boot(lm_raw, to_scale = ~ .,
                                         to_center = ~ .,
                                         nboot = 100)
lm_std_boot

}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stdmod-package.R
\docType{package}
\name{stdmod-package}
\alias{stdmod-package}
\title{stdmod: Standardized Moderation Effect and Its Confidence Interval}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Functions for computing a standardized moderation effect in moderated regression and forming its confidence interval by nonparametric bootstrapping as proposed in Cheung, Cheung, Lau, Hui, and Vong (2022) \doi{10.1037/hea0001188}. Also includes simple-to-use functions for computing conditional effects (unstandardized or standardized) and plotting moderation effects.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://sfcheung.github.io/stdmod/}
  \item Report bugs at \url{https://github.com/sfcheung/stdmod/issues}
}

}
\author{
\strong{Maintainer}: Shu Fai Cheung \email{shufai.cheung@gmail.com} (\href{https://orcid.org/0000-0002-9871-9448}{ORCID})

Other contributors:
\itemize{
  \item David Weng Ngai Vong [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_stdmod.R
\name{summary.std_selected}
\alias{summary.std_selected}
\title{Summary Method for a 'std_selected' Class Object}
\usage{
\method{summary}{std_selected}(object, ...)
}
\arguments{
\item{object}{The output of \code{\link[=std_selected]{std_selected()}} or \code{\link[=std_selected_boot]{std_selected_boot()}}.}

\item{...}{Additional arguments. Ignored by this function.}
}
\value{
An object of class \code{summary.std_selected}, with
bootstrap confidence intervals added if present in the object.
The object is a list. Its main element \code{coefficients} is similar to
the
coefficient table in the \code{\link[=summary]{summary()}} printout of \code{\link[=lm]{lm()}}.
This object is for printing summary information of the results
from \code{\link[=std_selected]{std_selected()}} or \code{\link[=std_selected_boot]{std_selected_boot()}}.
}
\description{
Summarize the results of \code{\link[=std_selected]{std_selected()}} or
\code{\link[=std_selected_boot]{std_selected_boot()}}.
}
\examples{

# Load a sample data set

dat <- test_x_1_w_1_v_1_cat1_n_500

# Do a moderated regression by lm
lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
summary(lm_raw)

# Standardize all variables except for categorical variables.
# Interaction terms are formed after standardization.
lm_std <- std_selected(lm_raw, to_scale = ~ .,
                               to_center = ~ .)
summary(lm_std)

# With bootstrapping
# nboot = 100 just for illustration. nboot >= 2000 should be used in read
# research.
lm_std_boot <- std_selected_boot(lm_raw, to_scale = ~ .,
                                         to_center = ~ .,
                                         nboot = 100)
summary(lm_std_boot)

}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{sleep_emo_con}
\alias{sleep_emo_con}
\title{Sample Dataset: Predicting Sleep Duration}
\format{
A data frame with 500 rows and six variables:
\describe{
\item{case_id}{Case ID, integer}
\item{sleep_duration}{Sleep duration in hours}
\item{conscientiousness}{Conscientiousness score, continuous}
\item{emotional_stability}{Emotional stability score, continuous}
\item{age}{Age in years}
\item{gender}{Gender, string, "female" or "male"}
}
}
\usage{
sleep_emo_con
}
\description{
A random subset from a real dataset. For
illustration.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/std_selected.R, R/std_selected_boot.R
\name{std_selected}
\alias{std_selected}
\alias{std_selected_boot}
\title{Standardize Variables in a Regression Model}
\usage{
std_selected(lm_out, to_scale = NULL, to_center = NULL, to_standardize = NULL)

std_selected_boot(
  lm_out,
  to_scale = NULL,
  to_center = NULL,
  to_standardize = NULL,
  conf = 0.95,
  nboot = 100,
  boot_args = NULL,
  save_boot_est = TRUE,
  full_output = FALSE,
  do_boot = TRUE
)
}
\arguments{
\item{lm_out}{The output from \code{\link[=lm]{lm()}}.}

\item{to_scale}{The terms to be rescaled by standard deviation,
specified by a formula as in \code{\link[=lm]{lm()}}. For example, if the terms to be scaled
are \code{x1} and \code{x3}, use \code{~ x1 + x3}. No need to specify the
interaction
term. To scale the outcome variable, list it on the \emph{right hand side}
as a predictor.
Specify only the original variables. If \code{NULL}, then no terms
will be rescaled by their standard deviations. Variables that are not
numeric will be ignored. Default is \code{NULL}.}

\item{to_center}{The terms to be mean centered, specified by a formula
as in \code{\link[=lm]{lm()}}. For example, if the terms to be centered
is \code{x1} and \code{x3}, use \code{~ x1 + x3}. No need to specify the
interaction term. To center  the outcome variable, list it on the
\emph{right hand side}
as a predictor.
Specify only the original variables. If \code{NULL}, then no term
will be centered. Default is \code{NULL}.}

\item{to_standardize}{The terms to be standardized, specified by a formula
as in \code{\link[=lm]{lm()}}. For example, if the terms to be standardized
is \code{x1} and \code{x3}, use \code{~ x1 + x3}. No need to specify the
interaction term. To standardize the outcome variable, list it on the
\emph{right hand side}
as a predictor.
Specify only the original variables.
This is a shortcut to \code{to_center} and \code{to_scale}. Listing a variable
in \code{to_standardize} is equivalent to listing this variable in
both \code{to_center} and \code{to_scale}.
Default is \code{NULL}.}

\item{conf}{The level of confidence for the confidence interval.
Default is .95.}

\item{nboot}{The number of bootstrap samples. Default is 100.}

\item{boot_args}{A named list of arguments to be passed to \code{\link[boot:boot]{boot::boot()}}.
Default is \code{NULL}.}

\item{save_boot_est}{If \code{TRUE}, the default, the bootstrap estimates will
be saved in the element
\code{boot_est} of the output.}

\item{full_output}{Whether the full output from \code{\link[boot:boot]{boot::boot()}} is returned.
Default is \code{FALSE}. If \code{TRUE}, the full output from
\code{\link[boot:boot]{boot::boot()}} will be saved in the element \code{boot_out}
of the output.}

\item{do_boot}{Whether bootstrapping confidence intervals will be formed.
Default is \code{TRUE}. If \code{FALSE}, all arguments related to
bootstrapping will be ignored.}
}
\value{
The updated \code{\link[=lm]{lm()}} output, with the class \code{std_selected} added. It will be
treated as a usual \code{\link[=lm]{lm()}} object by most functions. These are the major
additional element in the list:
\itemize{
\item \code{scaled_terms}: If not \code{NULL}, a character vector of the variables scaled.
\item \code{centered_terms}: If not \code{NULL}, a character vector of the variables mean-centered.
\item \code{scaled_by}: A numeric vector of the scaling factors for all the variables in
the model. The value is 1 for terms not scaled.
\item \code{centered_by}: A numeric vector of the numbers used for centering for all
the variables in the model. The value is 0 for
terms not centered.
\item \code{std_selected_call}: The original call.
\item \code{lm_out_call}: The call in \code{lm_out}.
}

Like \code{\link[=std_selected]{std_selected()}}, \code{\link[=std_selected_boot]{std_selected_boot()}} returns the updated \code{\link[=lm]{lm()}}
output, with the class \code{std_selected} added. The output of \code{\link[=std_selected_boot]{std_selected_boot()}}
contain these additional elements in the list:
\itemize{
\item \code{boot_ci}: A data frame of the bootstrap confidence intervals of the
regression coefficient.
\item \code{nboot}: The number of bootstrap samples requested.
\item \code{conf}: The level of confidence, in proportion.
\item \code{boot_est}: A matrix of the bootstrap estimates of the regression coefficients.
The number of rows equal to \code{nboot}, and the number of columns
equal to the number of terms in the regression model.
\item \code{std_selected_boot_call}: The call to \code{std_selected_boot()}.
\item \code{boot_out}: If available, the original output from \code{boot::boot()}.
}
}
\description{
Standardize, mean center, or scale by standard deviation
selected variables in a regression model and refit the model
}
\details{
\code{\link[=std_selected]{std_selected()}} was originally developed to compute the standardized
moderation effect and the standardized coefficients for other predictors
given an \code{\link[=lm]{lm()}} output (Cheung, Cheung, Lau, Hui, & Vong, 2022).
It has been extended such that users can specify
which variables in a regression model are to be mean-centered and/or
rescaled by
their standard deviations. If the model has one or more interaction terms,
they will be formed after the transformation, yielding the correct
standardized solution for a moderated regression model. Moreover,
categorical predictors will be automatically skipped in mean-centering
and rescaling.

Standardization is conducted when a variable is mean-centered and then
rescaled by its standard deviation. Therefore, if the goal is to get the
standardized solution of a moderated regression, users
just instruct the function to standardize all non-categorical variables
in the regression model.

\code{\link[=std_selected_boot]{std_selected_boot()}} is a wrapper of \code{\link[=std_selected]{std_selected()}}. It calls
\code{\link[=std_selected]{std_selected()}}
once
for each bootstrap sample, and then computes the nonparametric
bootstrap
percentile confidence intervals (Cheung, Cheung, Lau, Hui, & Vong, 2022).

If \code{do_boot} is \code{FALSE}, then the object it returns is identical to that
by \code{\link[=std_selected]{std_selected()}}.

This function intentionally does not have an argument for setting the seed
for
random number. Users are recommended to set the seed, e.g., using
\code{\link[=set.seed]{set.seed()}}
before calling it, to ensure reproducibility.
}
\section{Functions}{
\itemize{
\item \code{std_selected()}: The base function to center or
scale selected variables in a regression model

\item \code{std_selected_boot()}: A wrapper of \code{\link[=std_selected]{std_selected()}} that forms
nonparametric bootstrap confidence intervals.

}}
\examples{

# Load a sample data set

dat <- test_x_1_w_1_v_1_cat1_n_500
head(dat)

# Do a moderated regression by lm
lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
summary(lm_raw)

# Mean center mod only
lm_cw <- std_selected(lm_raw, to_center = ~ mod)
summary(lm_cw)

# Mean center mod and iv
lm_cwx <- std_selected(lm_raw, to_center = ~ mod + iv)
summary(lm_cwx)

# Standardize both mod and iv
lm_stdwx <- std_selected(lm_raw, to_scale = ~ mod + iv,
                               to_center = ~ mod + iv)
summary(lm_stdwx)

# Standardize all variables except for categorical variables.
# Interaction terms are formed after standardization.
lm_std <- std_selected(lm_raw, to_scale = ~ .,
                               to_center = ~ .)
summary(lm_std)

# Use to_standardize as a shortcut
lm_stdwx2 <- std_selected(lm_raw, to_standardize = ~ mod + iv)
# The results are the same
coef(lm_stdwx)
coef(lm_stdwx2)
all.equal(coef(lm_stdwx), coef(lm_stdwx2))



dat <- test_x_1_w_1_v_1_cat1_n_500
head(dat)

# Do a moderated regression by lm
lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
summary(lm_raw)
# Standardize all variables as in std_selected above, and compute the
# nonparametric bootstrapping percentile confidence intervals.
set.seed(87053)
lm_std_boot <- std_selected_boot(lm_raw,
                                 to_scale = ~ .,
                                 to_center = ~ .,
                                 conf = .95,
                                 nboot = 100)
# In real analysis, nboot should be at least 2000.
summary(lm_std_boot)

# Use to_standardize as a shortcut
set.seed(87053)
lm_std_boot2 <- std_selected_boot(lm_raw,
                                  to_standardize = ~ .,
                                  conf = .95,
                                  nboot = 100)
# The results are the same
confint(lm_std_boot)
confint(lm_std_boot2)
all.equal(confint(lm_std_boot), confint(lm_std_boot2))


}
\references{
Cheung, S. F., Cheung, S.-H., Lau, E. Y. Y., Hui, C. H., & Vong, W. N.
(2022) Improving an old way to measure moderation effect in standardized
units. \emph{Health Psychology}, \emph{41}(7), 502-505.
\doi{10.1037/hea0001188}
}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_summary_stdmod.R
\name{print.summary.std_selected}
\alias{print.summary.std_selected}
\title{Print the Summary of a 'std_selected' Class Object}
\usage{
\method{print}{summary.std_selected}(
  x,
  ...,
  est_digits = 4,
  t_digits = 4,
  pvalue_less_than = 0.001,
  default_style = FALSE
)
}
\arguments{
\item{x}{The output of \code{\link[=summary]{summary()}}.}

\item{...}{Arguments to be passed to \code{\link[=summary]{summary()}}.}

\item{est_digits}{The number of digits
after the decimal to be displayed for
the coefficient estimates, their
standard errors, and bootstrap
confidence intervals (if present). Note
that the values will be rounded to
this number of digits before printing.
If all digits at this position are
zero for all values, the values may
be displayed with fewer digits.
Note that the coefficient table is
printed by \code{\link[stats:printCoefmat]{stats::printCoefmat()}}.
If some numbers are vary large, the
number of digits after the decimal
may be smaller than \code{est_digits} due
to a limit on the column width.
This value also determines the number
of digits for displayed R-squared
if \code{default_style} is \code{FALSE}.
Default if 4.}

\item{t_digits}{The number of digits
after the decimal to be displayed
for the \emph{t} statistic (in the column
\code{"t value"}). This value also
determines the number of digits for
the \emph{F} statistic for the R-squared
if \code{default_style} is \code{FALSE}.
Default is 4.}

\item{pvalue_less_than}{If a \emph{p}-value
is less than this value, it will be
displayed with \verb{"<(this value)".}
For example, if \code{pvalue_less_than}
is .001, the default, \emph{p}-values less
than .001 will be displayed as
\verb{<.001}. This value also determines
the printout of the \emph{p}-value of
the \emph{F} statistic if \code{default_style}
is \code{FALSE}. (This argument does what
\code{eps.Pvalue} does in
\code{\link[stats:printCoefmat]{stats::printCoefmat()}}.)}

\item{default_style}{Logical. If
\code{FALSE}, the default, R-squared
and \emph{F} statistic will be displayed
in a more readable style. If \code{TRUE},
then the default style in the
printout of the \code{summary} of
\code{\link[=lm]{lm()}} output will be used.}
}
\value{
\code{x} is returned invisibly.
}
\description{
Print the summary generated by \code{\link[=summary]{summary()}} on the output
of \code{\link[=std_selected]{std_selected()}} or \code{\link[=std_selected_boot]{std_selected_boot()}}.
}
\examples{

# Load a sample data set

dat <- test_x_1_w_1_v_1_cat1_n_500

# Do a moderated regression by lm
lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)

# Standardize all variables except for categorical variables.
# Interaction terms are formed after standardization.
lm_std <- std_selected(lm_raw, to_scale = ~ .,
                               to_center = ~ .)
summary(lm_std)

# With bootstrapping
# nboot = 100 just for illustration. nboot >= 2000 should be used in read
# research.
lm_std_boot <- std_selected_boot(lm_raw, to_scale = ~ .,
                                         to_center = ~ .,
                                         nboot = 100)
summary(lm_std_boot)

}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stdmod.R, R/stdmod_bootci.R
\name{stdmod}
\alias{stdmod}
\alias{stdmod_boot}
\title{Standardized Moderation Effect Given an 'lm' Output}
\usage{
stdmod(
  lm_out,
  x = NULL,
  w = NULL,
  y = NULL,
  x_rescale = TRUE,
  w_rescale = TRUE,
  y_rescale = TRUE
)

stdmod_boot(
  lm_out,
  ...,
  nboot = 100,
  conf = 0.95,
  boot_args = NULL,
  full_output = FALSE
)
}
\arguments{
\item{lm_out}{The output from \code{\link[=lm]{lm()}}.}

\item{x}{The focal variable, that is, the variable with its effect
being moderated. If supplied, its standard deviation will be
used
for rescaling. Also called the independent variable in some
models. Default is \code{NULL}.}

\item{w}{The moderator. If supplied, its standard deviation will be
used
for rescaling. Default is \code{NULL}.}

\item{y}{The outcome variable (dependent variable) . If supplied,
its standard
deviation will be used for rescaling. Default is NULL.}

\item{x_rescale}{If \code{TRUE}, will rescale x by its standard deviation.
Default is \code{TRUE}.}

\item{w_rescale}{If \code{TRUE}, will rescale w by its standard deviation.
Default is \code{TRUE}.}

\item{y_rescale}{If \code{TRUE}, will rescale y by its standard deviation.
Default is \code{TRUE}.}

\item{...}{Parameters to be passed to \code{\link[=stdmod]{stdmod()}}.}

\item{nboot}{The number of bootstrap samples. Default is 100.}

\item{conf}{The level of confidence for the confidence interval.
Default is .95.}

\item{boot_args}{A named list of arguments to be passed to \code{\link[boot:boot]{boot::boot()}}.
Default is \code{NULL}.}

\item{full_output}{Whether the full output from \code{\link[boot:boot]{boot::boot()}} is returned.
Default is \code{FALSE}.}
}
\value{
\code{\link[=stdmod]{stdmod()}} returns a scalar: The standardized moderation effect.

\code{\link[=stdmod_boot]{stdmod_boot()}} returns a list with two elements. The element \code{ci} is
a numeric vector of the bootstrap confidence interval. The element \code{boot_out},
if not \code{NA}, is the output of \code{\link[boot:boot]{boot::boot()}}, which is used to do the
bootstrapping.
}
\description{
Compute the standardized moderation effect in a moderated
regression model.
}
\details{
Two more general functions, \code{\link[=std_selected]{std_selected()}} and
\code{\link[=std_selected_boot]{std_selected_boot()}}, have been developed and can do what these functions
do and more. Users are recommended to use them instead of \code{\link[=stdmod]{stdmod()}} and
\code{\link[=stdmod_boot]{stdmod_boot()}}. These two functions will not be updated in the near
future.

Nevertheless, if computing the standardized moderation effect and
forming its nonparametric
bootstrap interval are all required, then these functions can still
be used.

\code{\link[=stdmod]{stdmod()}} computes the standardized moderation effect given an
\code{\link[=lm]{lm()}} output using the formula from Cheung, Cheung, Lau, Hui, and Vong
(2022). Users specify
the moderator, the focal variable (the variable with its effect on
the outcome variable moderated), the outcome variable (dependent variable)
, and the corresponding
standardized moderation
effect. Users can also select which variable(s) will be standardized.

\code{\link[=stdmod_boot]{stdmod_boot()}} is a wrapper of \code{\link[=stdmod]{stdmod()}}. It computes the nonparametric
bootstrap confidence interval of the standardized moderation effect, as
suggested by Cheung, Cheung, Lau, Hui, and Vong (2022), given
the output of \code{\link[=lm]{lm()}}

Percentile interval from \code{\link[boot:boot.ci]{boot::boot.ci()}} is returned by this function.
If other types of
confidence intervals are desired, set \code{full_output = TRUE} and use
\code{\link[boot:boot.ci]{boot::boot.ci()}} on the element \code{boot_out} in the output of this
function.
}
\section{Functions}{
\itemize{
\item \code{stdmod()}: The base function for computing standardized
moderation effect

\item \code{stdmod_boot()}: A wrapper of \code{\link[=stdmod]{stdmod()}} that computes the nonparametric
bootstrap confidence interval of the standardized
moderation effect.

}}
\examples{

# Load a test data of 500 cases

dat <- test_x_1_w_1_v_2_n_500

# Do regression as usual:
lm_raw <- lm(dv ~ iv*mod + v1 + v2, dat)
summary(lm_raw)

# The standard deviations of iv, dv, and mod:
sds <- apply(dat, 2, sd)
sds

# Compute the standardized moderation effect:
stdmod_xyw <- stdmod(lm_raw, x = iv, y = dv, w = mod)
stdmod_xyw
# By default, all three variables will be standardized.

# Check against self-computed standardized moderation effect:
coef(lm_raw)["iv:mod"] * sds["iv"] * sds["mod"] / sds["dv"]

# Standardize only the iv, i.e., do not standardized dv and the moderator:
stdmod_x <- stdmod(lm_raw, x = iv, y = dv, w = mod,
                   x_rescale = TRUE,  y_rescale = FALSE, w_rescale = FALSE)
stdmod_x
# Check against self-computed moderation effect with only iv standardized:
coef(lm_raw)["iv:mod"] * sds["iv"]


dat <- test_x_1_w_1_v_2_n_500
# Do regression as usual:
lm_raw <- lm(dv ~ iv*mod + v1 + v2, dat)

# Compute the standardized moderation effect.
# Form its confidence interval by nonparametric bootstrapping.
set.seed(85740917)
stdmod_xyw_boot <- stdmod_boot(lm_raw, x = iv, w = mod, y = dv, nboot = 100)
# In real analysis, nboot should be at least 2000.

# Print the ci
stdmod_xyw_boot$ci

# Repeat the analysis but keep the results from boot:
set.seed(85740917)
stdmod_xyw_boot <- stdmod_boot(lm_raw, x = iv, w = mod, y = dv,
                                 nboot = 200, full_output = TRUE)
# In real analysis, nboot should be at least 2000.

# Print the 95\% percentile confidence interval
stdmod_xyw_boot$ci


}
\references{
Cheung, S. F., Cheung, S.-H., Lau, E. Y. Y., Hui, C. H., & Vong, W. N.
(2022) Improving an old way to measure moderation effect in standardized
units. \emph{Health Psychology}, \emph{41}(7), 502-505.
\doi{10.1037/hea0001188}
}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vcov_stdmod.R
\name{vcov.std_selected}
\alias{vcov.std_selected}
\title{The 'vcov' Method for a 'std_selected' Class Object}
\usage{
\method{vcov}{std_selected}(object, type, ...)
}
\arguments{
\item{object}{The output of \code{\link[=std_selected]{std_selected()}} or \code{\link[=std_selected_boot]{std_selected_boot()}}.}

\item{type}{The type of variance-covariance matrix. If set to \code{"lm"},
returns the results of the \code{\link[stats:vcov]{stats::vcov()}} method for the output of
\code{\link[=lm]{lm()}}. If set to \code{"boot"},
the variance-covariance matrix of the bootstrap estimates
is returned.
Default depends on \code{object}. If bootstrap estimates were stored,
then the default is \code{"boot"}. Otherwise, the default is \code{"lm"}.}

\item{...}{Arguments to be passed to \code{\link[stats:vcov]{stats::vcov()}}.}
}
\value{
A matrix of the variances and covariances of the parameter estimates.
}
\description{
Compute the variance-covariance matrix
of estimates in the output of \code{\link[=std_selected]{std_selected()}} or
\code{\link[=std_selected_boot]{std_selected_boot()}}.
}
\details{
If bootstrapping was used to form the confidence intervals,
users can request the variance-covariance matrix of the bootstrap estimates.
}
\examples{
# Load a sample data set

dat <- test_x_1_w_1_v_1_cat1_n_500
head(dat)

# Do a moderated regression by lm
lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)

# Standardize all variables except for categorical variables.
# Interaction terms are formed after standardization.
lm_std <- std_selected(lm_raw, to_scale = ~ .,
                               to_center = ~ .)

# VCOV of lm output
vcov(lm_std)

# Standardize all variables as in std_selected above, and compute the
# nonparametric bootstrapping percentile confidence intervals.
lm_std_boot <- std_selected_boot(lm_raw,
                                 to_scale = ~ .,
                                 to_center = ~ .,
                                 conf = .95,
                                 nboot = 100)
# In real analysis, nboot should be at least 2000.

# VCOV of bootstrap estimates, default when bootstrap was conducted
vcov(lm_std_boot)

# For OLS VCOV
vcov(lm_std_boot, type = "lm")

}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coef_stdmod_lavaan.R
\name{coef.stdmod_lavaan}
\alias{coef.stdmod_lavaan}
\title{Standardized Moderation Effect in a 'stdmod_lavaan' Class Object}
\usage{
\method{coef}{stdmod_lavaan}(object, ...)
}
\arguments{
\item{object}{The output of \code{\link[=stdmod_lavaan]{stdmod_lavaan()}}.}

\item{...}{Optional arguments. Ignored by the function.}
}
\value{
A scalar: The estimate of the standardized moderation effect.
}
\description{
Return the estimate of the standardized
moderation effect
in the output of \code{\link[=stdmod_lavaan]{stdmod_lavaan()}}.
}
\details{
It just extracts and returns the element \code{stdmod}.
}
\examples{

# Load a test data of 500 cases
dat <- test_mod1
library(lavaan)

mod <-
"
med ~ iv + mod + iv:mod + cov1
dv ~ med + cov2
"
fit <- sem(mod, dat)
coef(fit)

# Compute the standardized moderation effect
out_noboot <- stdmod_lavaan(fit = fit,
                            x = "iv",
                            y = "med",
                            w = "mod",
                            x_w = "iv:mod")
coef(out_noboot)

# Compute the standardized moderation effect and
# its confidence interval based on nonparametric bootstrapping
# Fit the model with bootstrap confidence intervals
# At least 2000 bootstrap samples should be used
# in real research. 50 is used here only for
# illustration.
fit <- sem(mod, dat, se = "boot", bootstrap = 50,
           iseed = 89574)
out_boot <- stdmod_lavaan(fit = fit,
                          x = "iv",
                          y = "med",
                          w = "mod",
                          x_w = "iv:mod",
                          boot_ci = TRUE)
coef(out_boot)

}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/confint_stdmod_lavaan.R
\name{confint.stdmod_lavaan}
\alias{confint.stdmod_lavaan}
\title{Confidence Intervals for a 'stdmod_lavaan' Class Object}
\usage{
\method{confint}{stdmod_lavaan}(object, parm, level = 0.95, ...)
}
\arguments{
\item{object}{The output of \code{\link[=stdmod_lavaan]{stdmod_lavaan()}}.}

\item{parm}{Ignored. Always return the bootstrap confidence interval
of the standardized moderation effect.}

\item{level}{The level of confidence, default is .95, returning the
95\% confidence interval.}

\item{...}{Additional arguments. Ignored by the function.}
}
\value{
A one-row matrix of the confidence intervals.
}
\description{
Return the confidence interval of the standardized
moderation effect
in the output of \code{\link[=stdmod_lavaan]{stdmod_lavaan()}}.
}
\details{
If bootstrapping is used to form the confidence interval by
\code{\link[=stdmod_lavaan]{stdmod_lavaan()}},
users can request the percentile confidence interval of
using the stored bootstrap estimate.
}
\examples{

# Load a test data of 500 cases
dat <- test_mod1
library(lavaan)

mod <-
"
med ~ iv + mod + iv:mod + cov1
dv ~ med + cov2
"
fit <- sem(mod, dat)
coef(fit)

# Compute the standardized moderation effect and
# its confidence interval based on nonparametric bootstrapping
# Fit the model with bootstrap confidence intervals
# At least 2000 bootstrap samples should be used
# in real research. 50 is used here only for
# illustration.
fit <- sem(mod, dat, se = "boot", bootstrap = 50,
           iseed = 89574)
out_boot <- stdmod_lavaan(fit = fit,
                          x = "iv",
                          y = "med",
                          w = "mod",
                          x_w = "iv:mod",
                          boot_ci = TRUE)
confint(out_boot)

}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/confint_condeff.R
\name{confint.cond_effect}
\alias{confint.cond_effect}
\title{Confidence Intervals for a 'cond_effect' Class Object}
\usage{
\method{confint}{cond_effect}(object, parm, level = 0.95, type, ...)
}
\arguments{
\item{object}{The output of \code{\link[=cond_effect]{cond_effect()}} or \code{\link[=cond_effect_boot]{cond_effect_boot()}}.}

\item{parm}{Ignored by this function. The confidence intervals for all
available levels will be returned.}

\item{level}{The level of confidence. For the confidence intervals returned
by \code{\link[=lm]{lm()}}, default is .95, i.e., 95\%. For the bootstrap
percentile confidence
intervals, default is the level used in calling
\code{\link[=cond_effect_boot]{cond_effect_boot()}}.}

\item{type}{The type of the confidence intervals. If est to \code{"lm"},
returns the confidence interval given by the \code{\link[=confint]{confint()}} method
of \code{\link[=lm]{lm()}}.
If set to \code{"boot"}, the bootstrap percentile confidence intervals
are returned.
Default is \code{"boot"} if bootstrap estimates are stored in
\code{object}, and \code{"lm"} if bootstrap estimates are not stored.}

\item{...}{Additional arguments. Ignored.}
}
\value{
A matrix of the confidence intervals.
}
\description{
Return the confidence intervals of estimates
conditional effect in the output of
\code{\link[=cond_effect]{cond_effect()}} or \code{\link[=cond_effect_boot]{cond_effect_boot()}}.
}
\details{
If bootstrapping is used to form the confidence interval by
\code{\link[=cond_effect_boot]{cond_effect_boot()}},
users can request the percentile confidence intervals of
the bootstrap estimates. This method does not do the bootstrapping itself.
}
\examples{

# Load a sample data set

dat <- test_x_1_w_1_v_1_cat1_n_500

# Do a moderated regression by lm
lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
summary(lm_raw)
out <- cond_effect(lm_raw, x = iv, w = mod)
print(out, t_ci = TRUE)
confint(out)

lm_std <- std_selected(lm_raw, to_center = ~ iv + mod, to_scale = ~ iv + mod)
# Alternative: use to_standardize as a shortcut
# lm_std <- std_selected(lm_raw, to_standardize = ~ iv + mod)
out <- cond_effect(lm_std, x = iv, w = mod)
print(out, t_ci = TRUE)
confint(out)

# Categorical moderator
lm_cat <- lm(dv ~ iv*cat1 + v1, dat)
summary(lm_cat)
out <- cond_effect(lm_cat, x = iv, w = cat1)
print(out, t_ci = TRUE)
confint(out)

}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{test_x_1_w_1_v_1_cat1_n_500}
\alias{test_x_1_w_1_v_1_cat1_n_500}
\title{Sample Dataset: One IV, One Moderator, Two Covariates}
\format{
A data frame with 500 rows and five variables:
\describe{
\item{dv}{Dependent variable, continuous}
\item{iv}{Independent variable, continuous}
\item{mod}{Moderator variable, continuous}
\item{v1}{Covariate, continuous}
\item{cat1}{Covariate, categorical (string) with three values: "gp1",
"gp2", and "gp3"}
}
}
\usage{
test_x_1_w_1_v_1_cat1_n_500
}
\description{
A covariate
(cat1) is
categorical. For testing.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/condeff.R, R/condeff_boot.R
\name{cond_effect}
\alias{cond_effect}
\alias{cond_effect_boot}
\title{Conditional Effects}
\usage{
cond_effect(
  output,
  x = NULL,
  w = NULL,
  w_method = c("sd", "percentile"),
  w_percentiles = c(0.16, 0.5, 0.84),
  w_sd_to_percentiles = NA,
  w_from_mean_in_sd = 1,
  w_values = NULL
)

cond_effect_boot(
  output,
  x = NULL,
  w = NULL,
  ...,
  conf = 0.95,
  nboot = 100,
  boot_args = NULL,
  save_boot_est = TRUE,
  full_output = FALSE,
  do_boot = TRUE
)
}
\arguments{
\item{output}{The output from \code{\link[stats:lm]{stats::lm()}}. It can also accept the output
from
\code{\link[=std_selected]{std_selected()}} or \code{\link[=std_selected_boot]{std_selected_boot()}}.}

\item{x}{The focal variable (independent variable), that is, the
variable with its effect on the outcome variable (dependent)
being moderated. It must be a numeric variable.}

\item{w}{The moderator. Both numeric variables and categorical variables
(character or factor) are supported.}

\item{w_method}{How to define "low", "medium", and "high" for the moderator
levels.
Default is in terms of mean and
standard deviation (SD) of the moderator, \code{"sd"}:
"low", "medium", and "high" are one SD below mean,
mean, and one SD above mean, respectively.
If equal to
\code{"percentile"}, then percentiles of the moderator in
the
dataset are used: "low", "medium", and "high" are
16th, 50th (median), and 84th percentiles, respectively.
Ignored if \code{w} is categorical.}

\item{w_percentiles}{If \code{w_method} is \code{"percentile"}, then this
argument
specifies the three percentiles to be used,
divided by 100.
It must be a
vector of two numbers. The default is
\code{c(.16, .50, .84)},
the 16th, 50th, and 84th percentiles,
which corresponds approximately
to one SD below and above mean in a
normal distribution, respectively.
Ignored if \code{w} is categorical.}

\item{w_sd_to_percentiles}{If \code{w_method} is \code{"percentile"} and
this argument is
set to a number, this number will be
used to
to determine the percentiles to be used.
The
lower percentile is the percentile in a
normal
distribution
that is \code{w_sd_to_percentiles} SD below
the mean.
The upper percentile is the percentile in
a normal
distribution that is \code{w_sd_to_percentiles}
SD
above the mean. Therefore, if
\code{w_sd_to_percentiles} is set to 1, then the
lower
and upper percentiles are 16th and 84th,
respectively. Default is \code{NA}.}

\item{w_from_mean_in_sd}{How many SD from mean is used to define
"low" and
"high" for the moderator. Default is 1.
Ignored if \code{w} is categorical.}

\item{w_values}{The values of \code{w} to
be used. Default is \code{NULL}. If a
numeric vector is supplied, these
values will be used to compute the
conditional effects. Other arguments
on generating levels are ignored.
Note that, if \code{w} has been standardized
or centered, these values are for
the standardized or centered \code{w}.
The values will always be sorted.
This argument is ignored if \code{w} is
categorical.}

\item{...}{Arguments to be passed to \code{\link[=cond_effect]{cond_effect()}}.}

\item{conf}{The level of confidence for the confidence interval.
Default is .95, to get 95\% confidence intervals.}

\item{nboot}{The number of bootstrap samples. Default is 100.}

\item{boot_args}{A named list of arguments to be passed to \code{\link[boot:boot]{boot::boot()}}.
Default is \code{NULL}.}

\item{save_boot_est}{If \code{TRUE}, the default, the bootstrap estimates will
be saved in the element
\code{boot_est} of the output.}

\item{full_output}{Whether the full output from \code{\link[boot:boot]{boot::boot()}} will be
returned.
Default is \code{FALSE}. If \code{TRUE}, the full output from
\code{\link[boot:boot]{boot::boot()}} will be saved in the element \code{boot_out}
of the output.}

\item{do_boot}{Whether bootstrapping confidence intervals will be formed.
Default is \code{TRUE}. If \code{FALSE}, all arguments related to
bootstrapping will be ignored.}
}
\value{
\code{\link[=cond_effect]{cond_effect()}} returns a data-frame-like object of the conditional effects.
The class is
\code{cond_effect} and the print method will print additional information of
the conditional effects. Additional information is stored in the
following attributes:
\itemize{
\item \code{call}: The original call.
\item \code{output}: The \code{output} object, such as the output from \code{lm()}.
\item \code{x}, \code{y}, and \code{w}: The three variables used to compute the conditional
effects: focal variable (\code{x}), outcome variable (\code{y}),
and the moderator (\code{w}).
\item \code{w_method}: The method used to determine the values of the moderator
at the selected levels.
\item \code{w_percentiles} The percentiles to use if \code{w_method} = \code{"percentile"}.
\item \code{w_sd_to_percentiles}: If not equal to \code{NA}, this is a scalar, the
number of standard deviation from the mean used to
determine the percentiles for the "low" and "high"
levels of the moderator.
\item \code{w_from_mean_in_sd}: The number of SD above or below the mean, for
determining the "low" and "high" levels of the
moderator if \code{w_method} is \code{"sd"}.
\item \code{w_empirical_percentiles}: The actual percentile levels in the dataset
for the selected
levels of the moderator. A numeric vector.
\item \code{w_empirical_z}: The actual distance from the mean, in SD, of each
selected level of the moderator. A numeric vector.
\item \code{y_standardized}, \code{x_standardized}, and \code{w_standardized}: Each of them
is a logical scalar, indicating whether the outcome
variable, focal variable, and moderator are standardized.
}

\code{\link[=cond_effect_boot]{cond_effect_boot()}} also returns a data-frame-like object of the
conditional effects of the class
\code{cond_effect}, with additional information from the bootstrapping stored
in these attributes:
\itemize{
\item \code{boot_ci}: A data frame of the bootstrap confidence intervals of the
conditional effects.
\item \code{nboot}: The number of bootstrap samples requested.
\item \code{conf}: The level of confidence, in proportion.
\item \code{boot_est}: A matrix of the bootstrap estimates of the conditional effects.
The number of rows equal to \code{nboot}, and the number of columns
equal to the number of levels of the moderator.
\item \code{cond_effect_boot_call}: The call to \code{cond_effect_boot()}.
\item \code{boot_out}: If available, the original output from \code{boot::boot()}.
}
}
\description{
Compute the conditional effects in a moderated regression model.
}
\details{
\code{\link[=cond_effect]{cond_effect()}} uses the centering approach to find the conditional
effect of the focal variable. For each level of the moderator, the value for
this level is subtracted from the moderator scores, and the model is
fitted to the modified data.
The coefficient of the focal variable is then the conditional effect of the
focal variable when the moderator's score is equal this value.

\code{\link[=cond_effect_boot]{cond_effect_boot()}} function is a wrapper of \code{\link[=cond_effect]{cond_effect()}}.
It calls \code{\link[=cond_effect]{cond_effect()}}
once for each bootstrap sample, and then computes the nonparametric
bootstrap percentile confidence intervals (Cheung, Cheung, Lau, Hui,
& Vong, 2022). If the output object is the output of \code{\link[=std_selected]{std_selected()}}
or \code{\link[=std_selected_boot]{std_selected_boot()}}, in which mean-centering and/or standardization
have been conducted, they will be repeated in each bootstrap sample.
Therefore, like \code{\link[=std_selected_boot]{std_selected_boot()}}, it can be used for form
nonparametric bootstrap confidence intervals for standardized
effects, though \code{\link[=cond_effect_boot]{cond_effect_boot()}} does this for the standardized
conditional effects.

This function ignores bootstrapping done by \code{\link[=std_selected_boot]{std_selected_boot()}}. It will
do its own bootstrapping.

If \code{do_boot} is \code{FALSE}, then the object it returns is identical to that
by \code{\link[=cond_effect]{cond_effect()}}.

This function intentionally does not have an argument for setting the seed
for
random number. Users are recommended to set the seed, e.g., using
\code{\link[=set.seed]{set.seed()}}
before calling it, to ensure reproducibility.
}
\section{Functions}{
\itemize{
\item \code{cond_effect_boot()}: A wrapper of \code{\link[=cond_effect]{cond_effect()}} that forms
nonparametric bootstrap confidence intervals.

}}
\examples{

# Load a sample data set

dat <- test_x_1_w_1_v_1_cat1_n_500

# Do a moderated regression by lm
lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
summary(lm_raw)
cond_effect(lm_raw, x = iv, w = mod)

lm_std <- std_selected(lm_raw, to_standardize = ~ iv + mod)
cond_effect(lm_std, x = iv, w = mod)

# Categorical moderator
lm_cat <- lm(dv ~ iv*cat1 + v1, dat)
summary(lm_cat)
cond_effect(lm_cat, x = iv, w = cat1)


# Load a sample data set

dat <- test_x_1_w_1_v_1_cat1_n_500

# Do a moderated regression by lm
lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
summary(lm_raw)

lm_std <- std_selected(lm_raw, to_standardize = ~ iv + mod)
cond_effect(lm_std, x = iv, w = mod)

# Form nonparametric bootstrap confidence intervals
# Use 2000 or even 5000 for nboot in real research
out <- cond_effect_boot(lm_std, x = iv, w = mod, nboot = 50)
out

}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stdmod_lavaan.R
\name{stdmod_lavaan}
\alias{stdmod_lavaan}
\title{Standardized Moderation Effect and Its Bootstrap CI in 'lavaan'}
\usage{
stdmod_lavaan(
  fit,
  x,
  y,
  w,
  x_w,
  standardized_x = TRUE,
  standardized_y = TRUE,
  standardized_w = TRUE,
  boot_ci = FALSE,
  boot_out = NULL,
  R = 100,
  conf = 0.95,
  use_old_version = FALSE,
  ...
)
}
\arguments{
\item{fit}{The SEM output by \code{\link[lavaan:lavaan]{lavaan::lavaan()}} or its wrappers.}

\item{x}{The name of the focal variable in the model, the variable with
its effect on the outcome variable being moderated.}

\item{y}{The name of the outcome variable (dependent variable) in the model.}

\item{w}{The name of the moderator in the model.}

\item{x_w}{The name of the product term (x * w) in the model. It can be
the variable generated by the colon operator, e.g., \code{"x:w"},
which is only in the model and not in the original data set.}

\item{standardized_x}{If \code{TRUE}, the default, \code{x} is standardized when
computing the standardized moderation effect.}

\item{standardized_y}{If \code{TRUE}, the default, \code{y} is standardized when
computing the standardized moderation effect.}

\item{standardized_w}{If \code{TRUE}, the default, \code{w} is standardized when
computing the standardized moderation effect.}

\item{boot_ci}{Boolean. Whether nonparametric bootstrapping will be
conducted. Default is \code{FALSE}.}

\item{boot_out}{If set to the output of \code{\link[manymome:do_boot]{manymome::do_boot()}}, the stored
bootstrap estimates will be retrieved to form the bootstrap confidence
interval. If set, bootstrap estimates stored in \code{fit}, if any, will not
be used. Default is \code{NULL}.}

\item{R}{(Not used in the current version. Used when \code{use_old_version}
is set to \code{TRUE}.)
The number of nonparametric bootstrapping samples. Default is 100.
Set this to at least 2000 in actual use.}

\item{conf}{The level of confidence. Default is .95, i.e., 95\%.}

\item{use_old_version}{If set to \code{TRUE}, it will use the bootstrapping
method used in 0.2.7.4 or before. Included only for reproducing
previous results if necessary. Default is \code{FALSE}.}

\item{...}{(Not used in the current version. Used when \code{use_old_version}
is set to \code{TRUE}.) Optional arguments to be passed to \code{\link[boot:boot]{boot::boot()}}. Parallel
processing can be used by adding the appropriate arguments in
\code{\link[boot:boot]{boot::boot()}}.}
}
\value{
A list of class \code{stdmod_lavaan} with these elements:
\itemize{
\item \code{stdmod}: The standardized moderation effect.
\item \code{ci}: The nonparametric bootstrap confidence interval. \code{NA} if
confidence interval not requested.
\item \code{boot_out}: The raw output from \code{\link[boot:boot]{boot::boot()}}. \code{NA} if
confidence interval not requested.
\item \code{fit}: The original fit object.
}
}
\description{
Compute the standardized moderation effect in a structural
equation model fitted by \code{\link[lavaan:lavaan]{lavaan::lavaan()}} or its wrappers and
form the nonparametric bootstrap confidence interval.
}
\details{
\subsection{Important Notes}{

Starting from Version 0.2.7.5, of \code{\link[=stdmod_lavaan]{stdmod_lavaan()}} adopts an approach
to bootstrapping different from that in the previous
versions (0.2.7.4 and before),
yielding bootstrapping results different from those in
previous versions (for reasons explained later).

To reproduce results from the older version of this function,
set \code{use_old_version} to \code{TRUE}.
}

\subsection{How it works}{

\code{\link[=stdmod_lavaan]{stdmod_lavaan()}} accepts a \link[lavaan:lavaan-class]{lavaan::lavaan} object, the
structural equation model output returned
by \code{\link[lavaan:lavaan]{lavaan::lavaan()}} and its wrappers (e.g, \code{\link[lavaan:sem]{lavaan::sem()}}) and computes
the standardized moderation effect using the formula in the appendix of
Cheung, Cheung, Lau, Hui, and Vong (2022).

The standard deviations of the focal variable (the variable with its effect
on the outcome variable being moderated), moderator, and outcome
variable (dependent variable) are computed from the implied
covariance matrix returned by
\code{\link[lavaan:lavInspect]{lavaan::lavInspect()}}. Therefore, models fitted to data sets with missing
data (e.g., with \code{missing = "fiml"}) are also supported.

Partial standardization can also be requested. For example, standardization
can be requested for only the focal variable and the outcome variable.

There are two ways to request nonparametric bootstrap
confidence interval. First, the model is fitted with \code{se = "boot"}
or \code{se = "bootstrap"} in \code{lavaan}. The stored bootstrap
estimates will then be retrieved automatically to compute the
standardized moderation effect. This is the most efficient approach
if the bootstrap confidence intervals are also needed for
other parameters in the model. Bootstrapping needs to be
done only once.

Second, bootstrap estimates can be generated by \code{\link[manymome:do_boot]{manymome::do_boot()}}.
The output is then supplied through the argument \code{boot_out}.
Bootstrapping also only needs to be done once. This approach
is appropriate when bootstrapping confidence intervals are
not needed for other model parameters, or another type
of confidence interval is needed when fitting the model.
Please refer to the help page of \code{\link[manymome:do_boot]{manymome::do_boot()}}
on how to use this function.

In both approaches, the standard deviations are also computed
in each bootstrap samples. This ensures that the sampling
variability of the standard deviations is also taken into
account in computing the bootstrap confidence interval of
the standardized moderation effect.
}

\subsection{Note on the differences between the current version (Version 0.2.7.5 or later) and previous versions (0.2.7.4 and before)}{

In older versions, \code{\link[=stdmod_lavaan]{stdmod_lavaan()}} does not allow for
partial standardization. Moreover,
it uses \code{\link[boot:boot]{boot::boot()}} to do the bootstrapping. Even with
the same seed, the results from \code{\link[boot:boot]{boot::boot()}} are not
identical to those of \code{lavaan} with \code{se = "boot"}
because they differ in the way the indices
of resampling are generated. Both approaches are correct,
They just use the generated random numbers differently.
To have results consistent with those from \code{lavaan},
the current version of \code{\link[=stdmod_lavaan]{stdmod_lavaan()}} adopts a
resampling algorithm identical to that of \code{lavaan}.
Last, in older versions, \code{\link[=stdmod_lavaan]{stdmod_lavaan()}} does
bootstrapping every time it is called. This is
inefficient.

The bootstrapping results in the current version are not
identical to those in older versions due to the use
of different resampling algorithms, To reproduce
previous results, set \code{use_old_version} to \code{TRUE}
}
}
\examples{

#Load a test data of 500 cases

dat <- test_mod1
library(lavaan)
mod <-
"
med ~ iv + mod + iv:mod + cov1
dv ~ med + cov2
"

fit <- sem(mod, dat)

# Compute the standardized moderation effect
out_noboot <- stdmod_lavaan(fit = fit,
                            x = "iv",
                            y = "med",
                            w = "mod",
                            x_w = "iv:mod")
out_noboot

# Compute the standardized moderation effect and
# its percentile confidence interval using
# nonparametric bootstrapping
# Fit the model with bootstrap confidence intervals
# At least 2000 bootstrap samples should be used
# in real research. 50 is used here only for
# illustration.
fit <- sem(mod, dat, se = "boot", bootstrap = 50,
           iseed = 89574)
out_boot <- stdmod_lavaan(fit = fit,
                          x = "iv",
                          y = "med",
                          w = "mod",
                          x_w = "iv:mod",
                          boot_ci = TRUE)
out_boot

}
\references{
Cheung, S. F., Cheung, S.-H., Lau, E. Y. Y., Hui, C. H., & Vong, W. N.
(2022) Improving an old way to measure moderation effect in standardized
units. \emph{Health Psychology}, \emph{41}(7), 502-505.
\doi{10.1037/hea0001188}
}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print_cond_eff.R
\name{print.cond_effect}
\alias{print.cond_effect}
\title{Print a 'cond_effect' Class Object}
\usage{
\method{print}{cond_effect}(
  x,
  nd = 3,
  nd_stat = 3,
  nd_p = 3,
  title = TRUE,
  model = TRUE,
  level_info = TRUE,
  standardized = TRUE,
  boot_info = TRUE,
  table_only = FALSE,
  t_ci = FALSE,
  t_ci_level = 0.95,
  ...
)
}
\arguments{
\item{x}{The output of \code{\link[=cond_effect]{cond_effect()}} or \code{\link[=cond_effect_boot]{cond_effect_boot()}}.}

\item{nd}{The number of digits for the variables.}

\item{nd_stat}{The number of digits for test statistics (e.g., \emph{t}).}

\item{nd_p}{The number of digits for \emph{p}-values.}

\item{title}{If \code{TRUE}, print a title. Default is \code{TRUE}.}

\item{model}{If \code{TRUE}, print the regression model. Default is \code{TRUE}.}

\item{level_info}{If \code{TRUE}, print information for interpreting the
levels of the moderator, such as the values of
the levels and distance from the mean. Default
is \code{TRUE}.}

\item{standardized}{If \code{TRUE} and one or more variables are
standardized, report it. Default is \code{TRUE}.`}

\item{boot_info}{If \code{TRUE} and bootstrap estimates are in \code{x},
print information about the bootstrapping,
such as the number of bootstrap samples.
Default is \code{TRUE}.}

\item{table_only}{If \code{TRUE}, will suppress of other elements except
for the table of conditional effects. Override
arguments such as \code{title}, \code{model}, and \code{level_info}.}

\item{t_ci}{If \code{TRUE}, will print the confidence intervals based on
t statistics. These confidence intervals should not be
used if some variables are standardized.}

\item{t_ci_level}{The level of confidence of the confidence intervals
based on t statistics. Default is .95.}

\item{...}{Additional arguments. Ignored by this function.}
}
\value{
\code{x} is returned invisibility.
}
\description{
Print the output of \code{\link[=cond_effect]{cond_effect()}} or \code{\link[=cond_effect_boot]{cond_effect_boot()}}.
}
\examples{

# Load a sample data set

dat <- test_x_1_w_1_v_1_cat1_n_500

# Do a moderated regression by lm
lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)

cond_effect(lm_raw, x = iv, w = mod)

lm_std <- std_selected(lm_raw, to_scale = ~ iv + mod,
                               to_center = ~ iv + mod)

cond_effect(lm_std, x = iv, w = mod)

}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add1_stdmod.R
\name{add1.std_selected}
\alias{add1.std_selected}
\title{The 'add1' Method for a 'std_selected' Class Object}
\usage{
\method{add1}{std_selected}(object, ...)
}
\arguments{
\item{object}{The output of \code{\link[=std_selected]{std_selected()}} or \code{\link[=std_selected_boot]{std_selected_boot()}}.}

\item{...}{Additional arguments. They will be ignored.}
}
\value{
It returns nothing. It is called for its side effect.
}
\description{
Intercept the \code{\link[=add1]{add1()}} method and raise an error.
}
\details{
\code{\link[=add1]{add1()}} should not be used after the output of \code{\link[=lm]{lm()}} is
processed by \code{\link[=std_selected]{std_selected()}} or \code{\link[=std_selected_boot]{std_selected_boot()}}.
}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/confint_stdmod.R
\name{confint.std_selected}
\alias{confint.std_selected}
\title{Confidence Intervals for a 'std_selected' Class Object}
\usage{
\method{confint}{std_selected}(object, parm, level = 0.95, type, ...)
}
\arguments{
\item{object}{The output of \code{\link[=std_selected]{std_selected()}} or \code{\link[=std_selected_boot]{std_selected_boot()}}.}

\item{parm}{The parameters (coefficients) for which confidence intervals
should be returned.
If missing, the confidence intervals of all parameters will be
returned.}

\item{level}{The level of confidence. For the confidence intervals returned
by \code{\link[=lm]{lm()}}, default is .95, i.e., 95\%. For the bootstrap
percentile confidence
intervals, default is the level used in calling
\code{\link[=std_selected_boot]{std_selected_boot()}}. If a level different from that in the
original
call is specified, \code{full_output} needs to be set in the call
to \code{\link[=std_selected_boot]{std_selected_boot()}} such that the original bootstrapping
output
is stored.}

\item{type}{The type of the confidence intervals. If est to \code{"lm"},
returns the confidence interval given by the \code{\link[=confint]{confint()}} method
of \code{\link[=lm]{lm()}}.
If set to \code{"boot"}, the bootstrap percentile confidence intervals
are returned.
Default is \code{"boot"} if bootstrap estimates are stored in
\code{object}, and \code{"lm"} if bootstrap estimates are not stored.}

\item{...}{Arguments to be passed to \code{\link[=summary.lm]{summary.lm()}}.}
}
\value{
A matrix of the confidence intervals.
}
\description{
Return the confidence intervals of estimates
in the output of \code{\link[=std_selected]{std_selected()}} or \code{\link[=std_selected_boot]{std_selected_boot()}}.
}
\details{
If bootstrapping is used to form the confidence interval by
\code{\link[=std_selected_boot]{std_selected_boot()}},
users can request the percentile confidence intervals of
the bootstrap estimates. This method does not do the bootstrapping itself.
}
\examples{

# Load a sample data set

dat <- test_x_1_w_1_v_1_cat1_n_500

# Do a moderated regression by lm
lm_raw <- lm(dv ~ iv*mod + v1 + cat1, dat)
summary(lm_raw)

# Standardize all variables except for categorical variables.
# Interaction terms are formed after standardization.
lm_std <- std_selected(lm_raw, to_center = ~ .,
                               to_scale = ~ .)
# Alternative: use to_standardize as a shortcut
# lm_std <- std_selected(lm_raw, to_standardize = ~ .)
summary(lm_std)

confint(lm_std)

# Use to_standardize as a shortcut
lm_std2 <- std_selected(lm_raw, to_standardize = ~ .)
# The results are the same
confint(lm_std)
confint(lm_std2)
all.equal(confint(lm_std), confint(lm_std2))

# With bootstrapping
# nboot = 100 just for illustration. nboot >= 2000 should be used in read
# research.
set.seed(89572)
lm_std_boot <- std_selected_boot(lm_raw, to_scale = ~ .,
                                         to_center = ~ .,
                                         nboot = 100)
summary(lm_std_boot)

# Bootstrap percentile intervals, default when bootstrap was conduced

confint(lm_std_boot)

# Force OLS confidence intervals

confint(lm_std_boot, type = "lm")

# Use to_standardize as a shortcut
set.seed(89572)
lm_std_boot2 <- std_selected_boot(lm_raw, to_standardize = ~ .,
                                          nboot = 100)
# The results are the same
confint(lm_std_boot)
confint(lm_std_boot2)
all.equal(confint(lm_std_boot), confint(lm_std_boot2))

}
\author{
Shu Fai Cheung \url{https://orcid.org/0000-0002-9871-9448}
}
